@@ -1,9 +1,9 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2003 Robert Rostek ( tecxx@rrs.at )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2011 Robert Rostek ( tecxx@rrs.at )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -32,6 +32,7 @@
 
 #include <common/StringFunctions.h>
 #include <common/FileFunctions.h>
+#include "amule.h"			// Needed for theApp
 #include "muuli_wdr.h"		// Needed for amuleSpecial
 
 
@@ -46,19 +47,14 @@ class CItemData : public wxTreeItemData
 {
 	public:
 		CItemData(const CPath& pathComponent)
-			: m_data(0)
-			, m_path(pathComponent)
+			: m_path(pathComponent)
 		{
 		}
 
 		~CItemData() {}
 
-		void AddCount() { m_data++; }
-		void SubCount() { m_data--; }
-		int GetCount() const { return m_data; }
 		const CPath& GetPathComponent() const { return m_path; }
 	private:
-		int	m_data;
 		CPath	m_path;
 };
 
@@ -66,7 +62,13 @@ class CItemData : public wxTreeItemData
 CDirectoryTreeCtrl::CDirectoryTreeCtrl(wxWindow* parent, int id, const wxPoint& pos, wxSize siz, int flags)
 	: wxTreeCtrl(parent,id,pos,siz,flags,wxDefaultValidator,wxT("ShareTree"))
 {
-	Init();
+	m_IsInit = false;
+	HasChanged = false;
+#ifdef CLIENT_GUI
+	m_IsRemote = !theApp->m_connect->IsConnectedToLocalHost();
+#else
+	m_IsRemote = false;
+#endif
 }
 
 
@@ -82,6 +84,12 @@ enum {
 
 void CDirectoryTreeCtrl::Init()
 {
+	// already done ?
+	if (m_IsInit) {
+		return;
+	}
+	m_IsInit = true;
+
 	// init image(s)
 	wxImageList* images = new wxImageList(16, 16);
 	images->Add(wxBitmap(amuleSpecial(1)));
@@ -92,23 +100,39 @@ void CDirectoryTreeCtrl::Init()
 	
 	// Create an empty root item, which we can
 	// safely append when creating a full path.
-	wxTreeItemId root = AddRoot(wxEmptyString, IMAGE_FOLDER, -1,
+	m_root = AddRoot(wxEmptyString, IMAGE_FOLDER, -1,
 					new CItemData(CPath()));
 	
+	if (!m_IsRemote) {
+	#ifndef __WXMSW__
+		AddChildItem(m_root, CPath(wxT("/")));
+	#else
+		// this might take awhile, so change the cursor
+		::wxSetCursor(*wxHOURGLASS_CURSOR);
+		// retrieve bitmask of all drives available
+		uint32 drives = GetLogicalDrives();
+		drives >>= 1;
+		for (char drive = 'C'; drive <= 'Z'; drive++) {
+			drives >>= 1;
+			if (! (drives & 1)) { // skip non existant drives
+				continue;
+			}
+			wxString driveStr = CFormat(wxT("%c:")) % drive;
+			uint32 type = GetDriveType(driveStr + wxT("\\"));
 
-#ifndef __WXMSW__
-	AddChildItem(root, CPath(wxT("/")));
-#else
-	for (char drive = 'C'; drive <= 'Z'; drive++) {
-		wxString driveStr = wxString::Format(wxT("%c:"), drive);
-		
-		if (CPath::DirExists(driveStr)) {
-			AddChildItem(root, CPath(driveStr));
+			// skip removable/undefined drives, share only fixed or remote drives
+			if ((type == 3 || type == 4)   // fixed drive / remote drive
+				&& CPath::DirExists(driveStr)) {
+				AddChildItem(m_root, CPath(driveStr));
+			}
 		}
+		::wxSetCursor(*wxSTANDARD_CURSOR);
+	#endif
 	}
-#endif
 
 	HasChanged = false;
+
+	UpdateSharedDirectories();
 }
 
 
@@ -126,19 +150,28 @@ void CDirectoryTreeCtrl::OnItemExpanding(wxTreeEvent& evt)
 
 void CDirectoryTreeCtrl::OnItemActivated(wxTreeEvent& evt)
 {
-	CheckChanged(evt.GetItem(), !IsBold(evt.GetItem()));
-	HasChanged = true;
+	if (!m_IsRemote) {
+		CheckChanged(evt.GetItem(), !IsBold(evt.GetItem()), false);
+		HasChanged = true;
+	}
 }
 
 
 void CDirectoryTreeCtrl::OnRButtonDown(wxTreeEvent& evt)
 {
-	MarkChildren(evt.GetItem(), !IsBold(evt.GetItem()));
-	HasChanged = true;
+	if (m_IsRemote) {
+		SelectItem(evt.GetItem()); // looks weird otherwise
+	} else {
+		// this might take awhile, so change the cursor
+		::wxSetCursor(*wxHOURGLASS_CURSOR);
+		MarkChildren(evt.GetItem(), !IsBold(evt.GetItem()), false);
+		::wxSetCursor(*wxSTANDARD_CURSOR);
+		HasChanged = true;
+	}
 }
 
 
-void CDirectoryTreeCtrl::MarkChildren(wxTreeItemId hChild, bool mark)
+void CDirectoryTreeCtrl::MarkChildren(wxTreeItemId hChild, bool mark, bool recursed)
 {
 	// Ensure that children are added, otherwise we might only get a "." entry.
 	if (!IsExpanded(hChild) && ItemHasChildren(hChild)) {
@@ -149,13 +182,16 @@ void CDirectoryTreeCtrl::MarkChildren(wxTreeItemId hChild, bool mark)
 	
 	wxTreeItemIdValue cookie;
 	wxTreeItemId hChild2 = GetFirstChild(hChild, cookie);
+	if (hChild2.IsOk()) {
+		SetHasSharedSubdirectory(hChild, mark);
+	}
 	while (hChild2.IsOk()) {
-		MarkChildren(hChild2, mark);
+		MarkChildren(hChild2, mark, true);
 
 		hChild2 = GetNextSibling(hChild2);
 	}
 
-	CheckChanged(hChild, mark);
+	CheckChanged(hChild, mark, recursed);
 }
 
 
@@ -168,13 +204,16 @@ void CDirectoryTreeCtrl::AddChildItem(wxTreeItemId hBranch, const CPath& item)
 						IMAGE_FOLDER, -1,
 						new CItemData(item));
 	
+	// BUG: wxGenericTreeControl won't set text calculated sizes when the item is created in AppendItem.
+	// This causes asserts on Mac and possibly other systems, so we have to repeat setting the string here.
+	SetItemText(treeItem, item.GetPrintable()); 
+
 	if (IsShared(fullPath)) {
 		SetItemBold(treeItem, true);
-		UpdateParentItems(treeItem, true);
 	}
 	
 	if (HasSharedSubdirectory(fullPath)) {
-		SetItemImage(treeItem, IMAGE_FOLDER_SUB_SHARED);
+		SetHasSharedSubdirectory(treeItem, true);
 	}
 	
 	if (HasSubdirectories(fullPath)) {
@@ -186,7 +225,7 @@ void CDirectoryTreeCtrl::AddChildItem(wxTreeItemId hBranch, const CPath& item)
 
 CPath CDirectoryTreeCtrl::GetFullPath(wxTreeItemId hItem)
 {
-	wxCHECK_MSG(hItem.IsOk(), CPath(), wxT("Invalid item in GetFullPath"));
+	{ wxCHECK_MSG(hItem.IsOk(), CPath(), wxT("Invalid item in GetFullPath")); }
 	
 	CPath result;
 	for (; hItem.IsOk(); hItem = GetItemParent(hItem)) {
@@ -228,7 +267,9 @@ void CDirectoryTreeCtrl::GetSharedDirectories(PathList* list)
 {
 	wxCHECK_RET(list, wxT("Invalid list in GetSharedDirectories"));
 	
-	list->insert(list->end(), m_lstShared.begin(), m_lstShared.end());
+	for (SharedMap::iterator it = m_lstShared.begin(); it != m_lstShared.end(); it++) {
+		list->push_back(it->second);
+	}
 }
 
 
@@ -237,7 +278,41 @@ void CDirectoryTreeCtrl::SetSharedDirectories(PathList* list)
 	wxCHECK_RET(list, wxT("Invalid list in SetSharedDirectories"));
 	
 	m_lstShared.clear();
-	m_lstShared.insert(m_lstShared.end(), list->begin(), list->end());
+	for (PathList::iterator it = list->begin(); it != list->end(); it++) {
+		m_lstShared.insert(SharedMapItem(GetKey(*it), *it));
+	}
+
+	if (m_IsInit) {
+		UpdateSharedDirectories();
+	}
+}
+
+
+wxString CDirectoryTreeCtrl::GetKey(const CPath& path)
+{
+	if (m_IsRemote) {
+		return path.GetRaw();
+	}
+
+	// Sanity check, see IsSameAs() in Path.cpp
+	const wxString cwd = wxGetCwd();
+	const int flags = (wxPATH_NORM_ALL | wxPATH_NORM_CASE) & ~wxPATH_NORM_ENV_VARS;
+	wxFileName fn(path.GetRaw());
+	fn.Normalize(flags, cwd);
+	return fn.GetFullPath();
+}
+
+
+void CDirectoryTreeCtrl::UpdateSharedDirectories()
+{
+	// ugly hack to at least show shared dirs in remote gui
+	if (m_IsRemote) {
+		DeleteChildren(m_root);
+		for (SharedMap::iterator it = m_lstShared.begin(); it != m_lstShared.end(); it++) {
+			AppendItem(m_root, it->second.GetPrintable(), IMAGE_FOLDER, -1, new CItemData(it->second));
+		}
+		return;
+	}
 
 	// Mark all shared root items (on windows this can be multiple
 	// drives, on unix there is only the root dir).
@@ -247,7 +322,7 @@ void CDirectoryTreeCtrl::SetSharedDirectories(PathList* list)
 	while (hChild.IsOk()) {
 		// Does this drive have shared subfolders?
 		if (HasSharedSubdirectory(GetFullPath(hChild))) { 
-			SetItemImage(hChild, IMAGE_FOLDER_SUB_SHARED);
+			SetHasSharedSubdirectory(hChild, true);
 		}
 		
 		// Is this drive shared?
@@ -262,19 +337,22 @@ void CDirectoryTreeCtrl::SetSharedDirectories(PathList* list)
 
 bool CDirectoryTreeCtrl::HasSharedSubdirectory(const CPath& path)
 {
-	PathList::iterator it = m_lstShared.begin();
-	for (; it != m_lstShared.end(); ++it) {
-		// IsSameDir to avoid the case where 'path' itself is shared.
-		if (it->StartsWith(path) && (!it->IsSameDir(path))) {
-			return true;
-		}
+	SharedMap::iterator it = m_lstShared.upper_bound(GetKey(path) + wxFileName::GetPathSeparator());
+	if (it == m_lstShared.end()) {
+		return false;
 	}
-
-	return false;
+	// upper_bound() doesn't find the directory itself, so no need to check for that.
+	return it->second.StartsWith(path);
 }
 
 
-void CDirectoryTreeCtrl::CheckChanged(wxTreeItemId hItem, bool bChecked)
+void CDirectoryTreeCtrl::SetHasSharedSubdirectory(wxTreeItemId hItem, bool add)
+{
+	SetItemImage(hItem, add ? IMAGE_FOLDER_SUB_SHARED : IMAGE_FOLDER);
+}
+
+
+void CDirectoryTreeCtrl::CheckChanged(wxTreeItemId hItem, bool bChecked, bool recursed)
 {
 	if (IsBold(hItem) != bChecked) {
 		SetItemBold(hItem, bChecked);
@@ -285,7 +363,9 @@ void CDirectoryTreeCtrl::CheckChanged(wxTreeItemId hItem, bool bChecked)
 			DelShare(GetFullPath(hItem));
 		}
 
-		UpdateParentItems(hItem, bChecked);
+		if (!recursed) {
+			UpdateParentItems(hItem, bChecked);
+		}
 	}
 }
 
@@ -294,14 +374,7 @@ bool CDirectoryTreeCtrl::IsShared(const CPath& path)
 {
 	wxCHECK_MSG(path.IsOk(), false, wxT("Invalid path in IsShared"));
 
-	PathList::iterator it = m_lstShared.begin();
-	for (; it != m_lstShared.end(); ++it) {
-		if (it->IsSameDir(path)) {
-			return true;
-		}
-	}
-
-	return false;
+	return m_lstShared.find(GetKey(path)) != m_lstShared.end();
 }
 
 
@@ -313,7 +386,7 @@ void CDirectoryTreeCtrl::AddShare(const CPath& path)
 		return;
 	}
 	
-	m_lstShared.push_back(path);
+	m_lstShared.insert(SharedMapItem(GetKey(path), path));
 }
 
 
@@ -321,13 +394,7 @@ void CDirectoryTreeCtrl::DelShare(const CPath& path)
 {
 	wxCHECK_RET(path.IsOk(), wxT("Invalid path in DelShare"));
 	
-	PathList::iterator it = m_lstShared.begin();
-	for (; it != m_lstShared.end(); ++it ) {
-		if (it->IsSameDir(path)) {
-			m_lstShared.erase(it);
-			return;
-		}
-	}
+	m_lstShared.erase(GetKey(path));
 }
 
 
@@ -336,23 +403,27 @@ void CDirectoryTreeCtrl::UpdateParentItems(wxTreeItemId hChild, bool add)
 	wxTreeItemId parent = hChild;
 	while (parent != GetRootItem()) {
 		parent = GetItemParent(parent);
-		CItemData* parent_data = dynamic_cast<CItemData*>(GetItemData(parent));
 		if (add) {
-			parent_data->AddCount();
-			if (parent_data->GetCount()==1) {
-				SetItemImage(parent, IMAGE_FOLDER_SUB_SHARED);
+			if (GetItemImage(parent) == IMAGE_FOLDER_SUB_SHARED) {
+				// parent already marked -> so are all its parents, finished
+				break;
+			} else {
+				SetHasSharedSubdirectory(parent, true);
 			}
 		} else {
-			switch (parent_data->GetCount()) {
-				case 0:
-					break;
-				case 1:
-					SetItemImage(parent, IMAGE_FOLDER);
-				default:
-					parent_data->SubCount();
+			if (GetItemImage(parent) == IMAGE_FOLDER_SUB_SHARED) {
+				// check if now there are still other shared dirs
+				if (HasSharedSubdirectory(GetFullPath(parent))) {
+					// yes, then further parents can stay red
 					break;
+				} else {
+					// no, further parents have to be checked too
+					SetHasSharedSubdirectory(parent, false);
+				}
+			} else {  // should not happen (unmark child of which the parent is already unmarked
+				break;
 			}
 		}
-	};
+	}
 }
 // File_checked_for_headers
