@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -23,7 +23,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#include "updownclient.h"	// Interface declarations
+#include "updownclient.h"		// Needed for CUpDownClient
 
 #include <protocol/Protocols.h>
 #include <protocol/ed2k/Client2Client/TCP.h>
@@ -49,6 +49,7 @@
 #include "Statistics.h"		// Needed for theStats
 #include "Logger.h"
 #include "GuiEvents.h"		// Needed for Notify_*
+#include "UploadQueue.h"	// Needed for CUploadQueue
 
 
 #ifdef __MULE_UNUSED_CODE__
@@ -154,6 +155,7 @@ bool CUpDownClient::AskForDownload()
 	m_bUDPPending = false;
 	m_dwLastAskedTime = ::GetTickCount();
 	SetDownloadState(DS_CONNECTING);
+	SetSentCancelTransfer(0);
 	return TryToConnect();
 }
 
@@ -169,7 +171,7 @@ void CUpDownClient::SendStartupLoadReq()
 	dataStartupLoadReq.WriteHash(m_reqfile->GetFileHash());
 	CPacket* packet = new CPacket(dataStartupLoadReq, OP_EDONKEYPROT, OP_STARTUPLOADREQ);
 	theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-	AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_STARTUPLOADREQ to ") + GetFullIP());	
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_STARTUPLOADREQ to ") + GetFullIP());
 	SendPacket(packet, true, true);
 }
 
@@ -219,16 +221,16 @@ void CUpDownClient::SendFileRequest()
 	dataFileReq.WriteHash(m_reqfile->GetFileHash());
 
 	if (SupportMultiPacket()) {
-		wxString sent_opcodes;
+		DEBUG_ONLY( wxString sent_opcodes; )
 		
 		if (SupportExtMultiPacket()) {
 			dataFileReq.WriteUInt64(m_reqfile->GetFileSize());
 		}
 		
-		AddDebugLogLineM(false, logClient, wxT("Sending file request to client"));
+		AddDebugLogLineN(logClient, wxT("Sending file request to client"));
 		
 		dataFileReq.WriteUInt8(OP_REQUESTFILENAME);
-		sent_opcodes += wxT("|RFNM|");
+		DEBUG_ONLY( sent_opcodes += wxT("|RFNM|"); )
 		// Extended information
 		if (GetExtendedRequestsVersion() > 0) {
 			m_reqfile->WritePartStatus(&dataFileReq);
@@ -237,7 +239,7 @@ void CUpDownClient::SendFileRequest()
 			m_reqfile->WriteCompleteSourcesCount(&dataFileReq);
 		}
 		if (m_reqfile->GetPartCount() > 1) {
-			sent_opcodes += wxT("|RFID|");
+			DEBUG_ONLY( sent_opcodes += wxT("|RFID|"); )
 			dataFileReq.WriteUInt8(OP_SETREQFILEID);
 		}
 		if (IsEmuleClient()) {
@@ -246,25 +248,26 @@ void CUpDownClient::SendFileRequest()
 		}
 		if (IsSourceRequestAllowed()) {
 			if (SupportsSourceExchange2()){
-				sent_opcodes += wxT("|RSRC2|");
+				DEBUG_ONLY( sent_opcodes += wxT("|RSRC2|"); )
 				dataFileReq.WriteUInt8(OP_REQUESTSOURCES2);
 				dataFileReq.WriteUInt8(SOURCEEXCHANGE2_VERSION);
 				const uint16 nOptions = 0; // 16 ... Reserved
 				dataFileReq.WriteUInt16(nOptions);
-			} else{
-				sent_opcodes += wxT("|RSRC|");
+			} else {
+				DEBUG_ONLY( sent_opcodes += wxT("|RSRC|"); )
 				dataFileReq.WriteUInt8(OP_REQUESTSOURCES);
 			}
 			m_reqfile->SetLastAnsweredTimeTimeout();
 			SetLastAskedForSources();
 		}
 		if (IsSupportingAICH()) {
-			sent_opcodes += wxT("|AFHR|");
+			DEBUG_ONLY( sent_opcodes += wxT("|AFHR|"); )
 			dataFileReq.WriteUInt8(OP_AICHFILEHASHREQ);
 		}		
 		CPacket* packet = new CPacket(dataFileReq, OP_EMULEPROT, (SupportExtMultiPacket() ? OP_MULTIPACKET_EXT : OP_MULTIPACKET));
 		theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-		AddDebugLogLineM( false, logLocalClient, wxString::Format(wxT("Local Client: %s "), (SupportExtMultiPacket() ? wxT("OP_MULTIPACKET_EXT (") : wxT("OP_MULTIPACKET (") )) + sent_opcodes + wxT(") to ") + GetFullIP());
+		AddDebugLogLineN(logLocalClient, CFormat(wxT("Local Client: %s (%s) to %s"))
+			% (SupportExtMultiPacket() ? wxT("OP_MULTIPACKET_EXT") : wxT("OP_MULTIPACKET")) % sent_opcodes % GetFullIP());
 		SendPacket(packet, true);
 	} else {
 		//This is extended information
@@ -276,7 +279,7 @@ void CUpDownClient::SendFileRequest()
 		}
 		CPacket* packet = new CPacket(dataFileReq, OP_EDONKEYPROT, OP_REQUESTFILENAME);
 		theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-		AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_REQUESTFILENAME to ") + GetFullIP() );
+		AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_REQUESTFILENAME to ") + GetFullIP() );
 		SendPacket(packet, true);
 	
 		// 26-Jul-2003: removed requesting the file status for files <= PARTSIZE for better compatibility with ed2k protocol (eDonkeyHybrid).
@@ -289,7 +292,7 @@ void CUpDownClient::SendFileRequest()
 			dataSetReqFileID.WriteHash(m_reqfile->GetFileHash());
 			packet = new CPacket(dataSetReqFileID, OP_EDONKEYPROT, OP_SETREQFILEID);
 			theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_SETREQFILEID to ") + GetFullIP());
+			AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_SETREQFILEID to ") + GetFullIP());
 			SendPacket(packet, true);
 		}
 	
@@ -314,7 +317,7 @@ void CUpDownClient::SendFileRequest()
 			packet = new CPacket(packetdata, OP_EMULEPROT, SupportsSourceExchange2() ? OP_REQUESTSOURCES2 : OP_REQUESTSOURCES);
 			
 			theStats::AddUpOverheadSourceExchange(packet->GetPacketSize());
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_REQUESTSOURCES to ") + GetFullIP() );
+			AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_REQUESTSOURCES to ") + GetFullIP() );
 			SendPacket(packet,true,true);
 			SetLastAskedForSources();
 		}
@@ -324,7 +327,7 @@ void CUpDownClient::SendFileRequest()
 			packet = new CPacket(OP_AICHFILEHASHREQ,16,OP_EMULEPROT);
 			packet->Copy16ToDataBuffer((const char *)m_reqfile->GetFileHash().GetHash());
 			theStats::AddUpOverheadOther(packet->GetPacketSize());
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_AICHFILEHASHREQ to ") + GetFullIP());
+			AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_AICHFILEHASHREQ to ") + GetFullIP());
 			SendPacket(packet,true,true);
 		}
 	}
@@ -353,8 +356,7 @@ void CUpDownClient::ProcessFileInfo(const CMemFile* data, const CPartFile* file)
 		m_nPartCount = m_reqfile->GetPartCount();
 		
 		m_reqfile->UpdatePartsFrequency( this, false );	// Decrement
-		m_downPartStatus.clear();
-		m_downPartStatus.resize( m_nPartCount, 1 );
+		m_downPartStatus.setsize( m_nPartCount, 1 );
 		m_reqfile->UpdatePartsFrequency( this, true );	// Increment
 		
 		m_bCompleteSource = true;
@@ -366,13 +368,13 @@ void CUpDownClient::ProcessFileInfo(const CMemFile* data, const CPartFile* file)
 				CPacket* packet = new CPacket(OP_HASHSETREQUEST,16, OP_EDONKEYPROT);
 				packet->Copy16ToDataBuffer((const char *)m_reqfile->GetFileHash().GetHash());
 				theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-				AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_HASHSETREQUEST to ") + GetFullIP());
+				AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_HASHSETREQUEST to ") + GetFullIP());
 				SendPacket(packet,true,true);
 				SetDownloadState(DS_REQHASHSET);
 				m_fHashsetRequesting = 1;
 				m_reqfile->SetHashSetNeeded(false);
 			} else {
-				wxASSERT(0);
+				wxFAIL;
 			}
 		} else {
 			SendStartupLoadReq();
@@ -403,7 +405,7 @@ void CUpDownClient::ProcessFileStatus(bool bUdpPacket, const CMemFile* data, con
 	if (!nED2KPartCount)
 	{
 		m_nPartCount = m_reqfile->GetPartCount();
-		m_downPartStatus.resize( m_nPartCount, 1);
+		m_downPartStatus.setsize( m_nPartCount, 1);
 		bPartsNeeded = true;
 		m_bCompleteSource = true;
 	}
@@ -425,7 +427,7 @@ void CUpDownClient::ProcessFileStatus(bool bUdpPacket, const CMemFile* data, con
 		m_nPartCount = m_reqfile->GetPartCount();
 
 		m_bCompleteSource = false;
-		m_downPartStatus.resize( m_nPartCount, 0 );
+		m_downPartStatus.setsize( m_nPartCount, 0 );
 		uint16 done = 0;
 	
 		try {
@@ -433,10 +435,11 @@ void CUpDownClient::ProcessFileStatus(bool bUdpPacket, const CMemFile* data, con
 				uint8 toread = data->ReadUInt8();
 			
 				for ( uint8 i = 0;i < 8; i++ ) {
-					m_downPartStatus[done] = ((toread>>i)&1)? 1:0;
+					bool status = ((toread>>i)&1)? 1:0;
+					m_downPartStatus.set(done, status);
 				
-					if ( m_downPartStatus[done] ) {
-						if (!m_reqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1)){
+					if (status) {
+						if (!m_reqfile->IsComplete(done)){
 							bPartsNeeded = true;
 							iNeeded++;
 						}
@@ -469,13 +472,13 @@ void CUpDownClient::ProcessFileStatus(bool bUdpPacket, const CMemFile* data, con
 				CPacket* packet = new CPacket(OP_HASHSETREQUEST,16, OP_EDONKEYPROT);
 				packet->Copy16ToDataBuffer((const char *)m_reqfile->GetFileHash().GetHash());
 				theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-				AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_HASHSETREQUEST to ") + GetFullIP());
+				AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_HASHSETREQUEST to ") + GetFullIP());
 				SendPacket(packet, true, true);
 				SetDownloadState(DS_REQHASHSET);
 				m_fHashsetRequesting = 1;
 				m_reqfile->SetHashSetNeeded(false);
 			} else {
-				wxASSERT(0);
+				wxFAIL;
 			}
 		}
 		else {
@@ -531,6 +534,7 @@ void CUpDownClient::SetDownloadState(uint8 byNewState)
 			}
 		}
 		if (byNewState == DS_DOWNLOADING) {
+			msReceivedPrev = GetTickCount();
 			theStats::AddDownloadingSource();
 		} else if (m_nDownloadState == DS_DOWNLOADING) {
 			theStats::RemoveDownloadingSource();
@@ -619,8 +623,8 @@ void CUpDownClient::SendBlockRequests()
 	if (m_DownloadBlocks_list.empty()) {
 		// Barry - instead of getting 3, just get how many is needed
 		uint16 count = m_MaxBlockRequests - m_PendingBlocks_list.size();
-		std::vector<Requested_Block_Struct*> toadd(count);
-		if (m_reqfile->GetNextRequestedBlock(this,&(toadd[0]),&count)) {
+		std::vector<Requested_Block_Struct*> toadd;
+		if (m_reqfile->GetNextRequestedBlock(this, toadd, count)) {
 			for (int i = 0; i != count; i++) {
 				m_DownloadBlocks_list.push_back(toadd[i]);
 			}
@@ -668,12 +672,12 @@ void CUpDownClient::SendBlockRequests()
 		
 		if (slower_client != this) {
 			// Re-request freed blocks.
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_CANCELTRANSFER (faster source eager to transfer) to ") + slower_client->GetFullIP() );
+			AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_CANCELTRANSFER (faster source eager to transfer) to ") + slower_client->GetFullIP() );
 			wxASSERT(m_DownloadBlocks_list.empty());
 			wxASSERT(m_PendingBlocks_list.empty());
 			uint16 count = m_MaxBlockRequests;
-			std::vector<Requested_Block_Struct*> toadd(count);
-			if (m_reqfile->GetNextRequestedBlock(this, &(toadd[0]),&count)) {
+			std::vector<Requested_Block_Struct*> toadd;
+			if (m_reqfile->GetNextRequestedBlock(this, toadd, count)) {
 				for (int i = 0; i != count; i++) {
 					Pending_Block_Struct* pblock = new Pending_Block_Struct;
 					pblock->block = toadd[i];
@@ -685,12 +689,12 @@ void CUpDownClient::SendBlockRequests()
 				}
 			}	else {
 				// WTF, we just freed blocks.
-				wxASSERT(0);
+				wxFAIL;
 				return;
 			}
 		} else {
 			// Drop this one.
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_CANCELTRANSFER (no free blocks) to ") + GetFullIP() );			
+			AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_CANCELTRANSFER (no free blocks) to ") + GetFullIP() );
 			//#warning Kry - Would be nice to swap A4AF here.		
 			return;
 		}
@@ -728,7 +732,8 @@ void CUpDownClient::SendBlockRequests()
 			}
 			
 			packet = new CPacket(data, OP_ED2KV2HEADER, OP_REQUESTPARTS);
-			AddDebugLogLineM( false, logLocalClient, wxString::Format(wxT("Local Client ED2Kv2: OP_REQUESTPARTS(%i) to "),(m_PendingBlocks_list.size()<m_MaxBlockRequests) ? m_PendingBlocks_list.size() : m_MaxBlockRequests) + GetFullIP() );
+			AddDebugLogLineN( logLocalClient, CFormat(wxT("Local Client ED2Kv2: OP_REQUESTPARTS(%i) to %s"))
+				% (m_PendingBlocks_list.size()<m_MaxBlockRequests ? m_PendingBlocks_list.size() : m_MaxBlockRequests) % GetFullIP() );
 
 			break;
 		}
@@ -748,11 +753,11 @@ void CUpDownClient::SendBlockRequests()
 						bHasLongBlocks = true;
 						if (!SupportsLargeFiles()){
 							// Requesting a large block from a client that doesn't support large files?
-							wxASSERT( false );
+							wxFAIL;
 							if (!GetSentCancelTransfer()){
 								CPacket* cancel_packet = new CPacket(OP_CANCELTRANSFER, 0, OP_EDONKEYPROT);
 								theStats::AddUpOverheadFileRequest(cancel_packet->GetPacketSize());
-								AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_CANCELTRANSFER to ") + GetFullIP() );
+								AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_CANCELTRANSFER to ") + GetFullIP() );
 								SendPacket(cancel_packet,true,true);
 								SetSentCancelTransfer(1);
 							}					
@@ -805,18 +810,18 @@ void CUpDownClient::SendBlockRequests()
 				}
 			}	
 			packet = new CPacket(data, (bHasLongBlocks ? OP_EMULEPROT : OP_EDONKEYPROT), (bHasLongBlocks ? (uint8)OP_REQUESTPARTS_I64 : (uint8)OP_REQUESTPARTS));
-			AddDebugLogLineM( false, logLocalClient, wxString::Format(wxT("Local Client: %s to "),(bHasLongBlocks ? wxT("OP_REQUESTPARTS_I64") : wxT("OP_REQUESTPARTS"))) + GetFullIP() );
+			AddDebugLogLineN(logLocalClient, CFormat(wxT("Local Client: %s to %s")) % (bHasLongBlocks ? wxT("OP_REQUESTPARTS_I64") : wxT("OP_REQUESTPARTS")) % GetFullIP());
 			break;
 		}
 		default:
-			wxASSERT(0);
+			wxFAIL;
 	}
 	
 	if (packet) {
 		theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
 		SendPacket(packet, true, true);
 	} else {
-		wxASSERT(0);
+		wxFAIL;
 	}
 }
 
@@ -915,7 +920,7 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 				}
 				
 				if (cur_block->fZStreamError){
-					AddDebugLogLineM( false, logZLib, 
+					AddDebugLogLineN(logZLib,
 						CFormat(wxT("Ignoring %u bytes of block %u-%u because of erroneous zstream state for file: %s"))
 							% (size - header_size) % nStartPos % nEndPos % m_reqfile->GetFileName());
 					m_reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
@@ -923,7 +928,7 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 				}
 		   
 				// Remember this start pos, used to draw part downloading in list
-				m_nLastBlockOffset = nStartPos;
+				m_lastDownloadingPart = nStartPos / PARTSIZE;
 	
 				// Occasionally packets are duplicated, no point writing it twice
 				// This will be 0 in these cases, or the length written otherwise
@@ -933,15 +938,12 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 				if (!packed) {
 					// security sanitize check
 					if (nEndPos > cur_block->block->EndOffset) {
-						AddDebugLogLineM(false, logRemoteClient, CFormat(wxT("Received Blockpacket exceeds requested boundaries (requested end: %u, Part: %u, received end: %u, Part: %u), file: %s remote IP: %s")) % cur_block->block->EndOffset % (uint32)(cur_block->block->EndOffset / PARTSIZE) % nEndPos % (uint32)(nEndPos / PARTSIZE) % m_reqfile->GetFileName() % Uint32toStringIP(GetIP()));
+						AddDebugLogLineN(logRemoteClient, CFormat(wxT("Received Blockpacket exceeds requested boundaries (requested end: %u, Part: %u, received end: %u, Part: %u), file: %s remote IP: %s")) % cur_block->block->EndOffset % (uint32)(cur_block->block->EndOffset / PARTSIZE) % nEndPos % (uint32)(nEndPos / PARTSIZE) % m_reqfile->GetFileName() % Uint32toStringIP(GetIP()));
 						m_reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
 						return;
 					}
 					// Write to disk (will be buffered in part file class)
-					lenWritten = m_reqfile->WriteToBuffer( size - header_size,
-														   (byte*)(packet + header_size),
-														   nStartPos, nEndPos,
-														   cur_block->block);
+					lenWritten = m_reqfile->WriteToBuffer( size - header_size, (byte*)(packet + header_size), nStartPos, nEndPos, cur_block->block, this);
 				} else {
 					// Packed
 					wxASSERT( (long int)size > 0 );
@@ -970,7 +972,7 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 							nEndPos = cur_block->block->StartOffset + cur_block->totalUnzipped - 1;
 	
 							if (nStartPos > cur_block->block->EndOffset || nEndPos > cur_block->block->EndOffset) {
-								AddDebugLogLineM( false, logZLib, 
+								AddDebugLogLineN(logZLib,
 									CFormat(wxT("Corrupted compressed packet for '%s' received (error 666)")) % m_reqfile->GetFileName());
 								m_reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
 							} else {
@@ -979,7 +981,8 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 																	   unzipped,
 																	   nStartPos,
 																	   nEndPos,
-																	   cur_block->block );
+																	   cur_block->block,
+																	   this);
 							}
 						}
 					} else {
@@ -988,7 +991,7 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 							strZipError = wxT(" - ") + wxString::FromAscii(cur_block->zStream->msg);
 						} 
 						
-						AddDebugLogLineM( false, logZLib,
+						AddDebugLogLineN(logZLib,
 							CFormat(wxT("Corrupted compressed packet for '%s' received (error %i): %s"))
 								% m_reqfile->GetFileName() % result % strZipError);
 						
@@ -1048,10 +1051,10 @@ void CUpDownClient::ProcessBlockPacket(const byte* packet, uint32 size, bool pac
 		}
 	} catch (const CEOFException& e) {
 		wxString error = wxString(wxT("Error reading "));
-		if (packed) error += wxString::Format(wxT("packed (LU: %i) "),lenUnzipped);
-		if (packed) error += wxT("largeblocks ");
-		error += wxString::Format(wxT("data packet: RS: %i HS: %i SP: %i EP: %i BS: %i -> "),size,header_size,nStartPos,nEndPos,nBlockSize);
-		AddDebugLogLineM(true, logRemoteClient, error + e.what());
+		if (packed) error += CFormat(wxT("packed (LU: %i) largeblocks ")) % lenUnzipped;
+		error += CFormat(wxT("data packet: RS: %i HS: %i SP: %i EP: %i BS: %i -> "))
+					% size % header_size % nStartPos % nEndPos % nBlockSize;
+		AddDebugLogLineC(logRemoteClient, error + e.what());
 		return;
 	}
 }
@@ -1146,12 +1149,12 @@ int CUpDownClient::unzip(Pending_Block_Struct *block, byte *zipped, uint32 lenZi
 		wxString strZipError;
 		
 		if ( zS->msg ) {
-			strZipError = wxString::Format(wxT(" %d '"), err) + wxString::FromAscii(zS->msg) + wxT("'");
+			strZipError = CFormat(wxT(" %d '%s'")) % err % wxString::FromAscii(zS->msg);
 		} else if (err != Z_OK) {
-			strZipError = wxString::Format(wxT(" %d"), err);
+			strZipError = CFormat(wxT(" %d")) % err;
 		}
 		
-		AddDebugLogLineM(false, logZLib,
+		AddDebugLogLineN(logZLib,
 			CFormat(wxT("Unexpected zip error %s in file '%s'"))
 				% strZipError % (m_reqfile ? m_reqfile->GetFileName() : CPath(wxT("?"))));
 	}
@@ -1164,57 +1167,43 @@ int CUpDownClient::unzip(Pending_Block_Struct *block, byte *zipped, uint32 lenZi
 }
 
 
-// Emilio: rewrite of eMule code to eliminate use of lists for averaging and fix
-// errors in calculation (32-bit rollover and time measurement)  This function 
-// uses a first-order filter with variable time constant (initially very short 
-// to quickly reach the right value without spiking, then gradually approaching 
-// the value of 50 seconds which is equivalent to the original averaging period 
-// used in eMule).  The download rate is measured using actual timestamps.  The 
-// filter-based averaging however uses a simplified algorithm that assumes a 
-// fixed loop time - this does not introduce any measurement error, it simply 
-// makes the degree of smoothing slightly imprecise (the true TC of the filter 
-// varies inversely with the true loop time), which is of no importance here.
+// Speed is now updated only when data was received, calculated as
+// (data received) / (time since last receiption)
+// and slightly filtered (10s average).
+// Result is quite precise now and makes the DownloadRateAdjust workaround obsolete.
 
 float CUpDownClient::CalculateKBpsDown()
 {
-												// -- all timing values are in seconds --
-	const	float tcLoop   =  0.1f;				// _assumed_ Process() loop time = 0.1 sec
-	const	float tcInit   =  0.4f;				// initial filter time constant
-	const	float tcFinal  = 50.0f;				// final filter time constant
-	const	float tcReduce =  5.0f;				// transition from tcInit to tcFinal
-	
-	const	float fInit  = tcLoop/tcInit;		// initial averaging factor
-	const	float fFinal = tcLoop/tcFinal;		// final averaging factor
-	const	float fReduce = std::exp(std::log(fFinal/fInit) / (tcReduce/tcLoop)) * 0.99999;
-	
-	uint32	msCur = ::GetTickCount();
+	const	float tAverage = 10.0;
+	uint32	msCur = GetTickCount();
 
-	if (msReceivedPrev == 0) {  // initialize the averaging filter
-		fDownAvgFilter = fInit;
-		// "kBpsDown =  bytesReceivedCycle/1024.0 / tcLoop"  would be technically correct,
-		// but the first loop often receives a large chunk of data and then produces a spike
-		kBpsDown = /* 0.0 * (1.0-fInit) + */ bytesReceivedCycle/1024.0 / tcLoop * fInit;
-		bytesReceivedCycle = 0;
-	} else if (msCur != msReceivedPrev) {	// (safeguard against divide-by-zero)
-		if (fDownAvgFilter > fFinal) {		// reduce time constant during ramp-up phase
-			fDownAvgFilter *= fReduce;		// this approximates averaging a lengthening list
+	if (bytesReceivedCycle) {
+		float dt = (msCur - msReceivedPrev) / 1000.0; // time since last reception
+		if (dt < 0.01) {	// (safeguard against divide-by-zero)
+			dt = 0.01f;		//  diff should be 100ms actually
 		}
-		kBpsDown = kBpsDown * (1.0 - fDownAvgFilter) 
-					+ (bytesReceivedCycle/1.024)/((float)(msCur-msReceivedPrev)) * fDownAvgFilter;
+		float kBpsDownCur = bytesReceivedCycle / 1024.0 / dt;
+		if (dt >= tAverage) {
+			kBpsDown = kBpsDownCur;
+		} else {
+			kBpsDown = (kBpsDown * (tAverage - dt) + kBpsDownCur * dt) / tAverage;
+		}
+		//AddDebugLogLineN(logLocalClient, CFormat(wxT("CalculateKBpsDown %p kbps %.1f kbpsCur %.1f dt %.3f rcv %d ")) 
+		//			% this % kBpsDown  % kBpsDownCur % dt % bytesReceivedCycle);
 		bytesReceivedCycle = 0;
+		msReceivedPrev = msCur;	
 	}
-	msReceivedPrev = msCur;	
 
 	m_cShowDR++;
 	if (m_cShowDR == 30){
 		m_cShowDR = 0;
 		UpdateDisplayedInfo();
 	}
-	if ((::GetTickCount() - m_dwLastBlockReceived) > DOWNLOADTIMEOUT){
+	if (msCur - m_dwLastBlockReceived > DOWNLOADTIMEOUT) {
 		if (!GetSentCancelTransfer()){
 			CPacket* packet = new CPacket(OP_CANCELTRANSFER, 0, OP_EDONKEYPROT);
 			theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
-			AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_CANCELTRANSFER to ") + GetFullIP() );
+			AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_CANCELTRANSFER to ") + GetFullIP() );
 			SendPacket(packet,true,true);
 			SetSentCancelTransfer(1);
 		}
@@ -1266,7 +1255,7 @@ void CUpDownClient::UDPReaskFNF()
 			}
 		}
 	} else {
-		AddDebugLogLineM( false, logRemoteClient, wxT("UDP ANSWER FNF : ") + GetUserName() + wxT(" - did not remove client because of current download state") );
+		AddDebugLogLineN( logRemoteClient, wxT("UDP ANSWER FNF : ") + GetUserName() + wxT(" - did not remove client because of current download state") );
 	}
 }
 
@@ -1315,7 +1304,7 @@ void CUpDownClient::UDPReaskForDownload()
 		}
 		
 		CPacket* response = new CPacket(data, OP_EMULEPROT, OP_REASKFILEPING);
-		AddDebugLogLineM( false, logClientUDP, wxT("Client UDP socket: send OP_REASKFILEPING") );
+		AddDebugLogLineN( logClientUDP, wxT("Client UDP socket: send OP_REASKFILEPING") );
 		theStats::AddUpOverheadFileRequest(response->GetPacketSize());
 		theApp->clientudp->SendPacket(response,GetConnectIP(),GetUDPPort(), ShouldReceiveCryptUDPPackets(), GetUserHash().GetHash(), false, 0);
 	} else  if (HasLowID() && GetBuddyIP() && GetBuddyPort() && HasValidBuddyID()) {
@@ -1340,32 +1329,34 @@ void CUpDownClient::UDPReaskForDownload()
 		}
 		
 		CPacket* response = new CPacket(data, OP_EMULEPROT, OP_REASKCALLBACKUDP);
-		AddDebugLogLineM( false, logClientUDP, wxT("Client UDP socket: send OP_REASKCALLBACKUDP") );
+		AddDebugLogLineN( logClientUDP, wxT("Client UDP socket: send OP_REASKCALLBACKUDP") );
 		theStats::AddUpOverheadFileRequest(response->GetPacketSize());
 		theApp->clientudp->SendPacket(response, GetBuddyIP(), GetBuddyPort(), false, NULL, true, 0 );
 	}
 }
 
 
-//! Barry - Sets string to show parts downloading, eg NNNYNNNNYYNYN
-wxString CUpDownClient::ShowDownloadingParts() const
+// Get the next part that is requested
+uint16 CUpDownClient::GetNextRequestedPart() const
 {
-	// Initialise to all N's
-	wxString Parts(wxT('N'), m_nPartCount);
+	uint16 part = 0xffff;
 
 	std::list<Pending_Block_Struct*>::const_iterator it = m_PendingBlocks_list.begin();
 	for (; it != m_PendingBlocks_list.end(); ++it) {
-		Parts.SetChar(((*it)->block->StartOffset / PARTSIZE), 'Y');
+		part = (*it)->block->StartOffset / PARTSIZE;
+		if (part != m_lastDownloadingPart) {
+			break;
+		}
 	}
 	
-	return Parts;
+	return part;
 }
 
 
 void CUpDownClient::UpdateDisplayedInfo(bool force)
 {
 	uint32 curTick = ::GetTickCount();
-	if(force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE) {
+	if (force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE) {
 		// Check if we actually need to notify of changes
 		bool update = m_reqfile && m_reqfile->ShowSources();
 		
@@ -1382,13 +1373,48 @@ void CUpDownClient::UpdateDisplayedInfo(bool force)
 	
 		// And finnaly trigger an event if there's any reason
 		if ( update ) {
-			Notify_DownloadCtrlUpdateItem(this);
+			SourceItemType type;
+			switch (GetDownloadState()) {
+				case DS_DOWNLOADING:
+				case DS_ONQUEUE:
+					// We will send A4AF, which will be checked.
+					type = A4AF_SOURCE;
+					break;
+				default:
+					type = UNAVAILABLE_SOURCE;
+					break;
+			}
+			
+			Notify_SourceCtrlUpdateSource(ECID(), type );
+		}
+
+		// Shared files view
+		if (m_uploadingfile && m_uploadingfile->ShowPeers()) {
+			Notify_SharedCtrlRefreshClient(ECID(), AVAILABLE_SOURCE);
 		}
 				
 		m_lastRefreshedDLDisplay = curTick;
 	}
 }
 
+uint8 CUpDownClient::GetObfuscationStatus() const
+{
+	uint8 ret = OBST_UNDEFINED;
+	if (thePrefs::IsClientCryptLayerSupported()) {
+		if (SupportsCryptLayer()) {
+			if ((RequestsCryptLayer() || thePrefs::IsClientCryptLayerRequested()) && HasObfuscatedConnectionBeenEstablished()) {
+				ret = OBST_ENABLED;
+			} else {
+				ret = OBST_SUPPORTED;
+			}
+		} else {
+			ret = OBST_NOT_SUPPORTED;
+		}
+	} else {
+		ret = OBST_DISABLED;
+	}
+	return ret;
+}
 
 // IgnoreNoNeeded = will switch to files of which this source has no needed parts (if no better fiels found)
 // ignoreSuspensions = ignore timelimit for A4Af jumping
@@ -1415,8 +1441,8 @@ bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool ignoreSuspensio
 		A4AFList::iterator it = m_A4AF_list.find( toFile );
 		if ( it != m_A4AF_list.end() ) {
 
-			// We force ignoring of noneeded flag and timestamps
-			if ( IsValidSwapTarget( it, true, true ) ) {
+			// We force ignoring of timestamps
+			if ( IsValidSwapTarget( it, bIgnoreNoNeeded, true ) ) {
 				// Set the target
 				target = it;
 			}
@@ -1455,14 +1481,14 @@ bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool ignoreSuspensio
 	// Try to swap if we found a valid target
 	if ( target != m_A4AF_list.end() ) {
 		
-		// Sainity check, if reqfile doesn't own the source, then something
+		// Sanity check, if reqfile doesn't own the source, then something
 		// is wrong and the swap cannot proceed.
 		if ( m_reqfile->DelSource( this ) ) {
 			CPartFile* SwapTo = target->first;
 			
 			// remove this client from the A4AF list of our new m_reqfile
 			if ( SwapTo->RemoveA4AFSource( this ) ) {
-				Notify_DownloadCtrlRemoveSource(this, SwapTo);
+				Notify_SourceCtrlRemoveSource(ECID(), SwapTo);
 			}
 
 			m_reqfile->RemoveDownloadingSource( this );
@@ -1477,9 +1503,9 @@ bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool ignoreSuspensio
 				// Avoid swapping to this file for a while
 				m_A4AF_list[m_reqfile].timestamp = ::GetTickCount(); 
 							
-				Notify_DownloadCtrlAddSource(m_reqfile, this, A4AF_SOURCE);
+				Notify_SourceCtrlAddSource(m_reqfile, CCLIENTREF(this, wxT("CUpDownClient::SwapToAnotherFile Notify_SourceCtrlAddSource 1")), A4AF_SOURCE);
 			} else {
-				Notify_DownloadCtrlRemoveSource( this, m_reqfile );
+				Notify_SourceCtrlRemoveSource(ECID(), m_reqfile);
 			}
 		
 			SetDownloadState(DS_NONE);
@@ -1494,7 +1520,7 @@ bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool ignoreSuspensio
 
 			SwapTo->AddSource( this );
 		
-			Notify_DownloadCtrlAddSource(SwapTo, this, UNAVAILABLE_SOURCE);
+			Notify_SourceCtrlAddSource(SwapTo, CCLIENTREF(this, wxT("CUpDownClient::SwapToAnotherFile Notify_SourceCtrlAddSource 2")), UNAVAILABLE_SOURCE);
 
 			// Remove the new reqfile from the list of other files
 			m_A4AF_list.erase( target );
@@ -1574,7 +1600,7 @@ void CUpDownClient::SetReqFileAICHHash(CAICHHash* val){
 void CUpDownClient::SendAICHRequest(CPartFile* pForFile, uint16 nPart){
 	CAICHRequestedData request;
 	request.m_nPart = nPart;
-	request.m_pClient = this;
+	request.m_pClient.Link(this CLIENT_DEBUGSTRING("CUpDownClient::SendAICHRequest"));
 	request.m_pPartFile = pForFile;
 	CAICHHashSet::m_liRequestedData.push_back(request);
 	m_fAICHRequested = TRUE;
@@ -1584,7 +1610,7 @@ void CUpDownClient::SendAICHRequest(CPartFile* pForFile, uint16 nPart){
 	pForFile->GetAICHHashset()->GetMasterHash().Write(&data);
 	CPacket* packet = new CPacket(data, OP_EMULEPROT, OP_AICHREQUEST);
 	theStats::AddUpOverheadOther(packet->GetPacketSize());	
-	AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_AICHREQUEST to") + GetFullIP());
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_AICHREQUEST to") + GetFullIP());
 	SafeSendPacket(packet);
 }
 
@@ -1605,25 +1631,25 @@ void CUpDownClient::ProcessAICHAnswer(const byte* packet, uint32 size)
 	CPartFile* pPartFile = theApp->downloadqueue->GetFileByID(hash);
 	CAICHRequestedData request = CAICHHashSet::GetAICHReqDetails(this);
 	uint16 nPart = data.ReadUInt16();
-	if (pPartFile != NULL && request.m_pPartFile == pPartFile && request.m_pClient == this && nPart == request.m_nPart){
+	if (pPartFile != NULL && request.m_pPartFile == pPartFile && request.m_pClient.GetClient() == this && nPart == request.m_nPart){
 		CAICHHash ahMasterHash(&data);
 		if ( (pPartFile->GetAICHHashset()->GetStatus() == AICH_TRUSTED || pPartFile->GetAICHHashset()->GetStatus() == AICH_VERIFIED)
 			 && ahMasterHash == pPartFile->GetAICHHashset()->GetMasterHash())
 		{
 			if(pPartFile->GetAICHHashset()->ReadRecoveryData(request.m_nPart*PARTSIZE, &data)){
 				// finally all checks passed, everythings seem to be fine
-				AddDebugLogLineM( false, logAICHTransfer, wxT("AICH Packet Answer: Succeeded to read and validate received recoverydata"));
+				AddDebugLogLineN(logAICHTransfer, wxT("AICH Packet Answer: Succeeded to read and validate received recoverydata"));
 				CAICHHashSet::RemoveClientAICHRequest(this);
 				pPartFile->AICHRecoveryDataAvailable(request.m_nPart);
 				return;
 			} else {
-				AddDebugLogLineM( false, logAICHTransfer, wxT("AICH Packet Answer: Succeeded to read and validate received recoverydata"));
+				AddDebugLogLineN(logAICHTransfer, wxT("AICH Packet Answer: Succeeded to read and validate received recoverydata"));
 			}
 		} else {
-			AddDebugLogLineM( false, logAICHTransfer, wxT("AICH Packet Answer: Masterhash differs from packethash or hashset has no trusted Masterhash") );
+			AddDebugLogLineN( logAICHTransfer, wxT("AICH Packet Answer: Masterhash differs from packethash or hashset has no trusted Masterhash") );
 		}
 	} else {
-		AddDebugLogLineM( false, logAICHTransfer, wxT("AICH Packet Answer: requested values differ from values in packet") );
+		AddDebugLogLineN( logAICHTransfer, wxT("AICH Packet Answer: requested values differ from values in packet") );
 	}
 
 	CAICHHashSet::ClientAICHRequestFailed(this);
@@ -1652,33 +1678,33 @@ void CUpDownClient::ProcessAICHRequest(const byte* packet, uint32 size)
 			fileResponse.WriteUInt16(nPart);
 			pKnownFile->GetAICHHashset()->GetMasterHash().Write(&fileResponse);
 			if (pKnownFile->GetAICHHashset()->CreatePartRecoveryData(nPart*PARTSIZE, &fileResponse)){
-				AddDebugLogLineM(false, logAICHTransfer,
+				AddDebugLogLineN(logAICHTransfer,
 					CFormat(wxT("AICH Packet Request: Sucessfully created and send recoverydata for '%s' to %s"))
 						% pKnownFile->GetFileName() % GetClientFullInfo());
 				
 				CPacket* packAnswer = new CPacket(fileResponse, OP_EMULEPROT, OP_AICHANSWER);			
 				theStats::AddUpOverheadOther(packAnswer->GetPacketSize());
-				AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_AICHANSWER to") + GetFullIP());
+				AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_AICHANSWER to") + GetFullIP());
 				SafeSendPacket(packAnswer);
 				return;
 			} else {
-				AddDebugLogLineM(false, logAICHTransfer,
+				AddDebugLogLineN(logAICHTransfer,
 					CFormat(wxT("AICH Packet Request: Failed to create recoverydata for '%s' to %s"))
 						% pKnownFile->GetFileName() % GetClientFullInfo());
 			}
 		} else {
-			AddDebugLogLineM(false, logAICHTransfer,
+			AddDebugLogLineN(logAICHTransfer,
 				CFormat(wxT("AICH Packet Request: Failed to create recoverydata - Hashset not ready or requested Hash differs from Masterhash for '%s' to %s"))
 					% pKnownFile->GetFileName() % GetClientFullInfo());
 		}
 	} else {
-		AddDebugLogLineM( false, logAICHTransfer, wxT("AICH Packet Request: Failed to find requested shared file - ") + GetClientFullInfo() );
+		AddDebugLogLineN( logAICHTransfer, wxT("AICH Packet Request: Failed to find requested shared file - ") + GetClientFullInfo() );
 	}
 		
 	CPacket* packAnswer = new CPacket(OP_AICHANSWER, 16, OP_EMULEPROT);
 	packAnswer->Copy16ToDataBuffer(hash.GetHash());
 	theStats::AddUpOverheadOther(packAnswer->GetPacketSize());
-	AddDebugLogLineM( false, logLocalClient, wxT("Local Client: OP_AICHANSWER to") + GetFullIP());
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_AICHANSWER to") + GetFullIP());
 	SafeSendPacket(packAnswer);
 }
 
@@ -1695,7 +1721,7 @@ void CUpDownClient::ProcessAICHFileHash(CMemFile* data, const CPartFile* file){
 		SetReqFileAICHHash(new CAICHHash(ahMasterHash));
 		pPartFile->GetAICHHashset()->UntrustedHashReceived(ahMasterHash, GetConnectIP());
 	} else {
-		AddDebugLogLineM( false, logAICHTransfer, wxT("ProcessAICHFileHash(): PartFile not found or Partfile differs from requested file, ") + GetClientFullInfo() );
+		AddDebugLogLineN( logAICHTransfer, wxT("ProcessAICHFileHash(): PartFile not found or Partfile differs from requested file, ") + GetClientFullInfo() );
 	}
 }
 // File_checked_for_headers
