@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -31,7 +31,6 @@
 #include <common/MenuIDs.h>
 #include <common/Constants.h>
 
-#include <wx/textfile.h>	// Needed for wxTextFile
 #include <wx/utils.h>
 
 #include "Server.h"		// Needed for CServer
@@ -53,7 +52,6 @@
 #include <common/Format.h>	// Needed for CFormat
 #include "IPFilter.h"
 #include <common/FileFunctions.h>	// Needed for CDirIterator
-#include "FileLock.h"		// Needed for CFileLock
 #include "GuiEvents.h"		// Needed for Notify_*
 #include "UserEvents.h"
 #include "MagnetURI.h"		// Needed for CMagnetED2KConverter
@@ -89,6 +87,11 @@ CDownloadQueue::CDownloadQueue()
 	m_dwNextTCPSrcReq = 0;
 	m_cRequestsSentToServer = 0;
 	m_lastDiskCheck = 0;
+	
+	// Static thresholds until dynamic kicks in.
+	m_rareFileThreshold = RARE_FILE;
+	m_commonFileThreshold = 100;
+
 	SetLastKademliaFileRequest();
 }
 
@@ -97,19 +100,17 @@ CDownloadQueue::~CDownloadQueue()
 {
 	if ( !m_filelist.empty() ) {
 		for ( unsigned int i = 0; i < m_filelist.size(); i++ ) {
-			printf("\rSaving PartFile %u of %u", i + 1, (unsigned int)m_filelist.size());
-			fflush(stdout);
+			AddLogLineNS(CFormat(_("Saving PartFile %u of %u")) % (i + 1) % m_filelist.size());
 			delete m_filelist[i];
 		}
-		printf("\nAll PartFiles Saved.\n");
+		AddLogLineNS(_("All PartFiles Saved."));
 	}
 }
 
 
 void CDownloadQueue::LoadMetFiles(const CPath& path)
 {
-	printf("Loading temp files from %s.\n",
-		(const char *)unicode2char(path.GetPrintable()));
+	AddLogLineNS(CFormat(_("Loading temp files from %s.")) % path.GetPrintable());
 	
 	std::vector<CPath> files;
 
@@ -128,7 +129,7 @@ void CDownloadQueue::LoadMetFiles(const CPath& path)
 
 	// Load part-files	
 	for ( size_t i = 0; i < files.size(); i++ ) {
-		printf("\rLoading PartFile %u of %u", (unsigned int)(i + 1), (unsigned int)files.size());
+		AddLogLineNS(CFormat(_("Loading PartFile %u of %u")) % (i + 1) % files.size());
 		fileName = files[i].GetFullName();
 		CPartFile *toadd = new CPartFile();
 		bool result = toadd->LoadPartFile(path, fileName) != 0;
@@ -150,28 +151,25 @@ void CDownloadQueue::LoadMetFiles(const CPath& path)
 					% toadd->GetFileHash().Encode() % fileName;
 			} else {
 				// If result is false, then reading of both the primary and the backup .met failed
-				AddLogLineM(false, 
-					_("ERROR: Failed to load backup file. Search http://forum.amule.org for .part.met recovery solutions."));
+				AddLogLineN(_("ERROR: Failed to load backup file. Search http://forum.amule.org for .part.met recovery solutions."));
 				msg << CFormat(wxT("ERROR: Failed to load PartFile '%s'")) % fileName;
 			}
-			AddDebugLogLineM(true, logPartFile, msg);
-			
-			// Newline so that the error stays visible.
-			printf(": %s\n", (const char*)unicode2char(msg));
+			AddLogLineCS(msg);
 
 			// Delete the partfile object in the end.
 			delete toadd;
 		}
 	}
-	printf("\nAll PartFiles Loaded.\n");
+	AddLogLineNS(_("All PartFiles Loaded."));
 	
 	if ( GetFileCount() == 0 ) {
-		AddLogLineM(false, _("No part files found"));
+		AddLogLineN(_("No part files found"));
 	} else {
-		AddLogLineM(false, wxString::Format(wxPLURAL("Found %u part file", "Found %u part files", GetFileCount()), GetFileCount()) );
+		AddLogLineN(CFormat(wxPLURAL("Found %u part file", "Found %u part files", GetFileCount())) % GetFileCount());
 
 		DoSortByPriority();
 		CheckDiskspace( path );
+		Notify_ShowUpdateCatTabTitles();
 	}
 }
 
@@ -184,6 +182,22 @@ uint16 CDownloadQueue::GetFileCount() const
 }
 
 
+void CDownloadQueue::CopyFileList(std::vector<CPartFile*>& out_list, bool includeCompleted) const
+{
+	wxMutexLocker lock(m_mutex);
+
+	out_list.reserve(m_filelist.size() + includeCompleted ? m_completedDownloads.size() : 0);
+	for (FileQueue::const_iterator it = m_filelist.begin(); it != m_filelist.end(); ++it) {
+		out_list.push_back(*it);
+	}
+	if (includeCompleted) {
+		for (FileList::const_iterator it = m_completedDownloads.begin(); it != m_completedDownloads.end(); ++it) {
+			out_list.push_back(*it);
+		}
+	}
+}
+
+
 CServer* CDownloadQueue::GetUDPServer() const
 {
 	wxMutexLocker lock( m_mutex );
@@ -224,10 +238,10 @@ void CDownloadQueue::AddSearchToDownload(CSearchFile* toadd, uint8 category)
 
 	if (toadd->GetFileSize() > OLD_MAX_FILE_SIZE) {
 		if (!PlatformSpecific::CanFSHandleLargeFiles(thePrefs::GetTempDir())) {
-			AddLogLineM(true, _("Filesystem for Temp directory cannot handle large files."));
+			AddLogLineC(_("Filesystem for Temp directory cannot handle large files."));
 			return;
 		} else if (!PlatformSpecific::CanFSHandleLargeFiles(theApp->glob_prefs->GetCatPath(category))) {
-			AddLogLineM(true, _("Filesystem for Incoming directory cannot handle large files."));
+			AddLogLineC(_("Filesystem for Incoming directory cannot handle large files."));
 			return;
 		}
 	}
@@ -236,7 +250,7 @@ void CDownloadQueue::AddSearchToDownload(CSearchFile* toadd, uint8 category)
 	try {
 		newfile = new CPartFile(toadd);
 	} catch (const CInvalidPacket& WXUNUSED(e)) {
-		AddDebugLogLineM(true, logDownloadQueue, wxT("Search-result contained invalid tags, could not add"));
+		AddDebugLogLineC(logDownloadQueue, wxT("Search-result contained invalid tags, could not add"));
 	}
 	
 	if ( newfile && newfile->GetStatus() != PS_ERROR ) {
@@ -268,14 +282,27 @@ struct SFindBestPF
 {
 	void operator()(CPartFile* file) {
 		// Check if we should filter out other categories
+		int alphaorder = 0;
+
 		if ((m_category != -1) && (file->GetCategory() != m_category)) {
 			return;
 		} else if (file->GetStatus() != PS_PAUSED) {
 			return;
+		} else if (m_alpha && m_result && ((alphaorder = file->GetFileName().GetPrintable().CmpNoCase(m_result->GetFileName().GetPrintable())) > 0)) {
+			return;
 		}
-	
-		if (!m_result || (file->GetDownPriority() > m_result->GetDownPriority())) {
-			m_result = file;				
+
+		if (!m_result) {
+			m_result = file;
+		} else {
+			if (m_alpha && (alphaorder < 0)) {
+				m_result = file;
+			} else if (file->GetDownPriority() > m_result->GetDownPriority()) {
+				// Either not alpha ordered, or they have the same alpha ordering (could happen if they have same name)
+				m_result = file;
+			} else {
+				// Lower priority file
+			}
 		}
 	}
 
@@ -283,13 +310,15 @@ struct SFindBestPF
 	int m_category;
 	//! If any acceptable files are found, this variable store their pointer
 	CPartFile* m_result;
+	//! If we should order alphabetically
+	bool m_alpha;
 };
 
 
 void CDownloadQueue::StartNextFile(CPartFile* oldfile)
 {
 	if ( thePrefs::StartNextFile() ) {
-		SFindBestPF visitor = { -1, NULL };
+		SFindBestPF visitor = { -1, NULL, thePrefs::StartNextFileAlpha() };
 		
 		{
 			wxMutexLocker lock(m_mutex);
@@ -307,6 +336,8 @@ void CDownloadQueue::StartNextFile(CPartFile* oldfile)
 				
 				visitor = std::for_each(m_filelist.begin(), m_filelist.end(), visitor);
 			}	
+
+			// Alpha doesn't need special cases
 		}
 		
 		if (visitor.m_result) {
@@ -333,10 +364,14 @@ void CDownloadQueue::AddDownload(CPartFile* file, bool paused, uint8 category)
 	}
 
 	NotifyObservers( EventType( EventType::INSERTED, file ) );
-
-	file->SetCategory(category);
+	if (category < theApp->glob_prefs->GetCatCount()) {
+		file->SetCategory(category);
+	} else {
+		AddDebugLogLineN( logDownloadQueue, wxT("Tried to add download into invalid category.") );
+	}
 	Notify_DownloadCtrlAddFile( file );
-	AddLogLineM(true, CFormat(_("Downloading %s")) % file->GetFileName() );
+	theApp->searchlist->UpdateSearchFileByHash(file->GetFileHash()); 	// Update file in the search dialog if it's still open
+	AddLogLineC(CFormat(_("Downloading %s")) % file->GetFileName() );
 }
 
 
@@ -344,7 +379,7 @@ bool CDownloadQueue::IsFileExisting( const CMD4Hash& fileid ) const
 {
 	if (CKnownFile* file = theApp->sharedfiles->GetFileByID(fileid)) {
 		if (file->IsPartFile()) {
-			AddLogLineM(true, CFormat( _("You are already trying to download the file '%s'") ) % file->GetFileName());
+			AddLogLineC(CFormat( _("You are already trying to download the file '%s'") ) % file->GetFileName());
 		} else {
 			// Check if the file exists, since otherwise the user is forced to 
 			// manually reload the shares to download a file again.
@@ -356,18 +391,21 @@ bool CDownloadQueue::IsFileExisting( const CMD4Hash& fileid ) const
 				return false;
 			}
 			
-			AddLogLineM(true, CFormat( _("You already have the file '%s'") ) % file->GetFileName());
+			AddLogLineC(CFormat( _("You already have the file '%s'") ) % file->GetFileName());
 		}
 		
 		return true;
 	} else if ((file = GetFileByID(fileid))) {
-		AddLogLineM(true, CFormat( _("You are already trying to download the file %s") ) % file->GetFileName());
+		AddLogLineC(CFormat( _("You are already trying to download the file %s") ) % file->GetFileName());
 		return true;
 	}
 	
 	return false;
 }
 
+#define RARITY_FACTOR 4 // < 25%
+#define NORMALITY_FACTOR 2 // <50%
+// x > NORMALITY_FACTOR -> High availablity.
 
 void CDownloadQueue::Process()
 {
@@ -392,22 +430,83 @@ void CDownloadQueue::Process()
 		uint32 cur_datarate = 0;
 		uint32 cur_udcounter = m_udcounter;
 		
+		std::list<int> m_sourcecountlist;
+
+		bool mustPreventSleep = false;
+
 		for ( uint16 i = 0; i < m_filelist.size(); i++ ) {
 			CPartFile* file = m_filelist[i];
 	
 			CMutexUnlocker unlocker(m_mutex);
 			
-			if ( file->GetStatus() == PS_READY || file->GetStatus() == PS_EMPTY ){
+			uint8 status = file->GetStatus();
+			mustPreventSleep |= !(status == PS_ERROR || status == PS_INSUFFICIENT || status == PS_PAUSED || status == PS_COMPLETE);
+
+			if (status == PS_READY || status == PS_EMPTY ){
 				cur_datarate += file->Process( downspeed, cur_udcounter );
 			} else {
 				//This will make sure we don't keep old sources to paused and stoped files..
 				file->StopPausedFile();
 			}
+
+			if (!file->IsPaused() && !file->IsStopped()) {
+				m_sourcecountlist.push_back(file->GetSourceCount());
+			}
+		}
+
+		if (thePrefs::GetPreventSleepWhileDownloading()) {
+			if ((mustPreventSleep == false) && (theStats::GetSessionSentBytes() < theStats::GetSessionReceivedBytes())) {
+				// I can see right through your clever plan.
+				mustPreventSleep = true;
+			}
+
+			if (mustPreventSleep) {
+				PlatformSpecific::PreventSleepMode();
+			} else {
+				PlatformSpecific::AllowSleepMode();
+			}
+		} else {
+			// Just in case the value changes while we're preventing. Calls to this function are totally inexpensive anwyay
+			PlatformSpecific::AllowSleepMode();
+		}
+
+
+		// Set the source rarity thresholds
+		int nSourceGroups = m_sourcecountlist.size();
+		if (nSourceGroups) {
+			m_sourcecountlist.sort();
+			if (nSourceGroups == 1) {
+				// High anyway.
+				m_rareFileThreshold = m_sourcecountlist.front() + 1;
+				m_commonFileThreshold = m_rareFileThreshold + 1;
+			} else if (nSourceGroups == 2) {
+				// One high, one low (unless they're both 0, then both high)
+				m_rareFileThreshold = (m_sourcecountlist.back() > 0) ? (m_sourcecountlist.back() - 1) : 1;
+				m_commonFileThreshold = m_rareFileThreshold + 1;
+			} else {
+				// More than two, time to do some math.
+
+				// Lower 25% with the current #define values.
+				int rarecutpoint = (nSourceGroups / RARITY_FACTOR); 
+				for (int i = 0; i < rarecutpoint; ++ i) {
+					m_sourcecountlist.pop_front();
+				}
+				m_rareFileThreshold = (m_sourcecountlist.front() > 0) ? (m_sourcecountlist.front() - 1) : 1;
+
+				// 50% of the non-rare ones, with the curent #define values.
+				int commoncutpoint = (nSourceGroups - rarecutpoint) / NORMALITY_FACTOR;
+				for (int i = 0; i < commoncutpoint; ++ i) {
+					m_sourcecountlist.pop_front();
+				}
+				m_commonFileThreshold = (m_sourcecountlist.front() > 0) ? (m_sourcecountlist.front() - 1) : 1;
+			}
+		} else {
+			m_rareFileThreshold = RARE_FILE;
+			m_commonFileThreshold = 100;
 		}
 	
 		m_datarate += cur_datarate;
-	
-	
+
 		if (m_udcounter == 5) {
 			if (theApp->serverconnect->IsUDPSocketAvailable()) {
 				if( (::GetTickCount() - m_lastudpstattime) > UDPSERVERSTATTIME) {
@@ -429,8 +528,6 @@ void CDownloadQueue::Process()
 		}
 	
 		if ( (::GetTickCount() - m_lastsorttime) > 10000 ) {
-			
-			
 			DoSortByPriority();
 		}
 		// Check if any paused files can be resumed
@@ -441,7 +538,7 @@ void CDownloadQueue::Process()
 	
 	// Check for new links once per second.
 	if ((::GetTickCount() - m_nLastED2KLinkCheck) >= 1000) {
-		AddLinksFromFile();
+		theApp->AddLinksFromFile();
 		m_nLastED2KLinkCheck = ::GetTickCount();
 	}
 }
@@ -456,6 +553,12 @@ CPartFile* CDownloadQueue::GetFileByID(const CMD4Hash& filehash) const
 			return m_filelist[ i ];
 		}
 	}
+	// Check completed too so we can execute remote commands (like change cat) on them
+	for (FileList::const_iterator it = m_completedDownloads.begin(); it != m_completedDownloads.end(); ++it) {
+		if ( filehash == (*it)->GetFileHash()) {
+			return *it;
+		}
+	}
 	
 	return NULL;
 }
@@ -469,7 +572,7 @@ CPartFile* CDownloadQueue::GetFileByIndex(unsigned int index)  const
 		return m_filelist[ index ];
 	}
 	
-	wxASSERT( false );
+	wxFAIL;
 	return NULL;
 }
 
@@ -506,7 +609,7 @@ void CDownloadQueue::CheckAndAddSource(CPartFile* sender, CUpDownClient* source)
 	// if we block loopbacks at this point it should prevent us from connecting to ourself
 	if ( source->HasValidHash() ) {
 		if ( source->GetUserHash() == thePrefs::GetUserHash() ) {
-			AddDebugLogLineM( false, logDownloadQueue, wxT("Tried to add source with matching hash to your own.") );
+			AddDebugLogLineN( logDownloadQueue, wxT("Tried to add source with matching hash to your own.") );
 			source->Safe_Delete();
 			return;
 		}
@@ -535,16 +638,16 @@ void CDownloadQueue::CheckAndAddSource(CPartFile* sender, CUpDownClient* source)
 
 		CClientList::SourceList::iterator it = found.begin();
 		for ( ; it != found.end(); it++ ) {
-			CKnownFile* file = (*it)->GetRequestFile();
+			CKnownFile* file = it->GetRequestFile();
 
 			// Only check files on the download-queue
 			if ( file ) {
 				// Is the found source queued for something else?
 				if (  file != sender ) {
 					// Try to add a request for the other file
-					if ( (*it)->AddRequestForAnotherFile(sender)) {
+					if ( it->GetClient()->AddRequestForAnotherFile(sender)) {
 						// Add it to downloadlistctrl
-						Notify_DownloadCtrlAddSource(sender, *it, A4AF_SOURCE);
+						Notify_SourceCtrlAddSource(sender, *it, A4AF_SOURCE);
 					}
 				}
 				
@@ -574,19 +677,13 @@ void CDownloadQueue::CheckAndAddSource(CPartFile* sender, CUpDownClient* source)
 		} else {
 			// Source was known, but reqfile NULL.
 			source->SetRequestFile( sender );
-			if (source->GetSourceFrom() != nSourceFrom) {
-				if (source->GetSourceFrom() != SF_NONE) {
-					theStats::RemoveSourceOrigin(source->GetSourceFrom());
-					theStats::RemoveFoundSource();
-				}
-				source->SetSourceFrom(nSourceFrom);
-			}
+			source->SetSourceFrom(nSourceFrom);
 			sender->AddSource( source );
 			if ( source->GetFileRating() || !source->GetFileComment().IsEmpty() ) {
 				sender->UpdateFileRatingCommentAvail();
 			}
 			
-			Notify_DownloadCtrlAddSource(sender, source, UNAVAILABLE_SOURCE);
+			Notify_SourceCtrlAddSource(sender, CCLIENTREF(source, wxT("CDownloadQueue::CheckAndAddSource Notify_SourceCtrlAddSource 1")), UNAVAILABLE_SOURCE);
 		}
 	} else {
 		// Unknown client, add it to the clients list
@@ -599,7 +696,7 @@ void CDownloadQueue::CheckAndAddSource(CPartFile* sender, CUpDownClient* source)
 			sender->UpdateFileRatingCommentAvail();
 		}
 	
-		Notify_DownloadCtrlAddSource(sender, source, UNAVAILABLE_SOURCE);
+		Notify_SourceCtrlAddSource(sender, CCLIENTREF(source, wxT("CDownloadQueue::CheckAndAddSource Notify_SourceCtrlAddSource 2")), UNAVAILABLE_SOURCE);
 	}
 }
 
@@ -626,15 +723,14 @@ void CDownloadQueue::CheckAndAddKnownSource(CPartFile* sender,CUpDownClient* sou
 	if (!source->HasLowID()) {
 		uint32 nClientIP = wxUINT32_SWAP_ALWAYS(source->GetUserIDHybrid());
 		if (!IsGoodIP(nClientIP, thePrefs::FilterLanIPs())) { // check for 0-IP, localhost and LAN addresses
-			AddDebugLogLineM(false, logIPFilter, wxT("Ignored already known source with IP=%s") + Uint32toStringIP(nClientIP));
+			AddDebugLogLineN(logIPFilter, wxT("Ignored already known source with IP=%s") + Uint32toStringIP(nClientIP));
 			return;
 		}
 	}	
 	
 	// Filter sources which are incompatible with our encryption setting (one requires it, and the other one doesn't supports it)
-	if ( (source->RequiresCryptLayer() && (!thePrefs::IsClientCryptLayerSupported() || !source->HasValidHash())) || (thePrefs::IsClientCryptLayerRequired() && (!source->SupportsCryptLayer() || !source->HasValidHash())))
-	{
-		source->Safe_Delete();
+	if ( (source->RequiresCryptLayer() && (!thePrefs::IsClientCryptLayerSupported() || !source->HasValidHash())) 
+		|| (thePrefs::IsClientCryptLayerRequired() && (!source->SupportsCryptLayer() || !source->HasValidHash()))) {
 		return;
 	}		
 	
@@ -644,7 +740,7 @@ void CDownloadQueue::CheckAndAddKnownSource(CPartFile* sender,CUpDownClient* sou
 	if ( file ) {
 		if ( file != sender ) {
 			if ( source->AddRequestForAnotherFile( sender ) ) {
-				Notify_DownloadCtrlAddSource( sender, source, A4AF_SOURCE );
+				Notify_SourceCtrlAddSource( sender, CCLIENTREF(source, wxT("CDownloadQueue::CheckAndAddKnownSource Notify_SourceCtrlAddSource 1")), A4AF_SOURCE );
 			}
 		}
 	} else {
@@ -656,7 +752,7 @@ void CDownloadQueue::CheckAndAddKnownSource(CPartFile* sender,CUpDownClient* sou
 
 		source->SetSourceFrom(SF_PASSIVE);
 		sender->AddSource( source );
-		Notify_DownloadCtrlAddSource( sender, source, UNAVAILABLE_SOURCE);
+		Notify_SourceCtrlAddSource( sender, CCLIENTREF(source, wxT("CDownloadQueue::CheckAndAddKnownSource Notify_SourceCtrlAddSource 2")), UNAVAILABLE_SOURCE);
 	}
 }
 
@@ -671,6 +767,10 @@ bool CDownloadQueue::RemoveSource(CUpDownClient* toremove, bool	WXUNUSED(updatew
 		
 		// Remove from source-list
 		if ( cur_file->DelSource( toremove ) ) {
+			
+			// Remove from sourcelist widget
+			Notify_SourceCtrlRemoveSource(toremove->ECID(), cur_file);
+			
 			cur_file->RemoveDownloadingSource(toremove);
 			removed = true;
 			if ( bDoStatsUpdate ) {
@@ -689,16 +789,13 @@ bool CDownloadQueue::RemoveSource(CUpDownClient* toremove, bool	WXUNUSED(updatew
 	
 	toremove->SetRequestFile( NULL );
 	toremove->SetDownloadState(DS_NONE);
-
-	// Remove from downloadlist widget
-	Notify_DownloadCtrlRemoveSource(toremove, (CPartFile*)NULL);
 	toremove->ResetFileStatusInfo();
 
 	return removed;
 }
 
 
-void CDownloadQueue::RemoveFile(CPartFile* file)
+void CDownloadQueue::RemoveFile(CPartFile* file, bool keepAsCompleted)
 {
 	RemoveLocalServerRequest( file );
 
@@ -707,6 +804,28 @@ void CDownloadQueue::RemoveFile(CPartFile* file)
 	wxMutexLocker lock( m_mutex );
 
 	EraseValue( m_filelist, file );
+
+	if (keepAsCompleted) {
+		m_completedDownloads.push_back(file);
+	}
+}
+
+
+void CDownloadQueue::ClearCompleted(const ListOfUInts32 & ecids)
+{
+	for (ListOfUInts32::const_iterator it1 = ecids.begin(); it1 != ecids.end(); it1++) {
+		uint32 ecid = *it1;
+		for (FileList::iterator it = m_completedDownloads.begin(); it != m_completedDownloads.end(); it++) {
+			CPartFile * file = *it;
+			if (file->ECID() == ecid) {
+				m_completedDownloads.erase(it);
+				// get a new EC ID so it is resent and cleared in remote gui
+				file->RenewECID();
+				Notify_DownloadCtrlRemoveFile(file);
+				break;
+			}
+		}
+	}
 }
 
 
@@ -715,11 +834,11 @@ CUpDownClient* CDownloadQueue::GetDownloadClientByIP_UDP(uint32 dwIP, uint16 nUD
 	wxMutexLocker lock( m_mutex );
 
 	for ( unsigned int i = 0; i < m_filelist.size(); i++ ) {
-		const CPartFile::SourceSet& set = m_filelist[i]->GetSourceList();
+		const CKnownFile::SourceSet& set = m_filelist[i]->GetSourceList();
 		
-		for ( CPartFile::SourceSet::const_iterator it = set.begin(); it != set.end(); it++ ) {
-			if ( (*it)->GetIP() == dwIP && (*it)->GetUDPPort() == nUDPPort ) {
-				return *it;
+		for ( CKnownFile::SourceSet::const_iterator it = set.begin(); it != set.end(); it++ ) {
+			if ( it->GetIP() == dwIP && it->GetUDPPort() == nUDPPort ) {
+				return it->GetClient();
 			}
 		}
 	}
@@ -791,7 +910,7 @@ bool CDownloadQueue::SendNextUDPPacket()
  			
 			if ( ( status == PS_READY || status == PS_EMPTY ) && file->GetSourceCount() < thePrefs::GetMaxSourcePerFileUDP() ) {
 				if (file->IsLargeFile() && !m_udpserver->SupportsLargeFilesUDP()) {
-					AddDebugLogLineM(false, logDownloadQueue, wxT("UDP Request for sources on a large file ignored: server doesn't support it")); 	
+					AddDebugLogLineN(logDownloadQueue, wxT("UDP Request for sources on a large file ignored: server doesn't support it"));
 				} else {
 					++m_cRequestsSentToServer;
 					hashlist.WriteHash( file->GetFileHash() );
@@ -932,7 +1051,7 @@ void CDownloadQueue::ProcessLocalRequests()
 				if (cur_file->GetStatus() == PS_READY || cur_file->GetStatus() == PS_EMPTY) {
 					uint8 nPriority = cur_file->GetDownPriority();
 					if (nPriority > PR_HIGH) {
-						wxASSERT(0);
+						wxFAIL;
 						nPriority = PR_HIGH;
 					}
 
@@ -945,7 +1064,7 @@ void CDownloadQueue::ProcessLocalRequests()
 				} else {
 					it = m_localServerReqQueue.erase(it);
 					cur_file->SetLocalSrcRequestQueued(false);
-					AddDebugLogLineM( false, logDownloadQueue,
+					AddDebugLogLineN(logDownloadQueue,
 						CFormat(wxT("Local server source request for file '%s' not sent because of status '%s'"))
 							% cur_file->GetFileName() % cur_file->getPartfileStatus());
 				}
@@ -959,9 +1078,9 @@ void CDownloadQueue::ProcessLocalRequests()
 				iFiles++;
 
 				if (!bServerSupportsLargeFiles && cur_file->IsLargeFile()) {
-					AddDebugLogLineM(false, logDownloadQueue, wxT("TCP Request for sources on a large file ignored: server doesn't support it"));
+					AddDebugLogLineN(logDownloadQueue, wxT("TCP Request for sources on a large file ignored: server doesn't support it"));
 				} else {
-					AddDebugLogLineM(false, logDownloadQueue,
+					AddDebugLogLineN(logDownloadQueue,
 						CFormat(wxT("Creating local sources request packet for '%s'")) % cur_file->GetFileName());
 					// create request packet
 					CMemFile data(16 + (cur_file->IsLargeFile() ? 8 : 4));
@@ -992,12 +1111,12 @@ void CDownloadQueue::ProcessLocalRequests()
 		if (iSize > 0) {
 			// create one 'packet' which contains all buffered OP_GETSOURCES ED2K packets to be sent with one TCP frame
 			// server credits: (16+4)*regularfiles + (16+4+8)*largefiles +1
-			CPacket* packet = new CPacket(new byte[iSize], dataTcpFrame.GetLength(), true, false);
+			CScopedPtr<CPacket> packet(new CPacket(new byte[iSize], dataTcpFrame.GetLength(), true, false));
 			dataTcpFrame.Seek(0, wxFromStart);
 			dataTcpFrame.Read(packet->GetPacket(), iSize);
 			uint32 size = packet->GetPacketSize();
-			theApp->serverconnect->SendPacket(packet, true);	// Deletes `packet'.
-			AddDebugLogLineM(false, logDownloadQueue, wxT("Sent local sources request packet."));
+			theApp->serverconnect->SendPacket(packet.release(), true);	// Deletes `packet'.
+			AddDebugLogLineN(logDownloadQueue, wxT("Sent local sources request packet."));
 			theStats::AddUpOverheadServer(size);
 		}
 
@@ -1016,54 +1135,15 @@ void CDownloadQueue::SendLocalSrcRequest(CPartFile* sender)
 }
 
 
-void CDownloadQueue::AddLinksFromFile()
-{
-	const wxString fullPath = theApp->ConfigDir + wxT("ED2KLinks");
-	if (!wxFile::Exists(fullPath)) {
-		return;
-	}
-	
-	// Attempt to lock the ED2KLinks file.
-	CFileLock lock((const char*)unicode2char(fullPath));
-
-	wxTextFile file(fullPath);
-	if ( file.Open() ) {
-		for ( unsigned int i = 0; i < file.GetLineCount(); i++ ) {
-			wxString line = file.GetLine( i ).Strip( wxString::both );
-			
-			if ( !line.IsEmpty() ) {
-				// Special case! used by a secondary running mule to raise this one.
-				if ( line == wxT("RAISE_DIALOG")  ) {
-					Notify_ShowGUI();
-					continue;
-				}
-				
-				AddLink( line );
-			}
-		}
-
-		file.Close();
-	} else {
-		printf("Failed to open ED2KLinks file.\n");
-	}
-	
-	// Delete the file.
-	wxRemoveFile(theApp->ConfigDir +  wxT("ED2KLinks"));
-}
-
-
 void CDownloadQueue::ResetCatParts(uint8 cat)
 {
-	for ( uint16 i = 0; i < GetFileCount(); i++ ) {
-		CPartFile* file = GetFileByIndex( i );
-		
-		if ( file->GetCategory() == cat ) {
-			// Reset the category
-			file->SetCategory( 0 );
-		} else if ( file->GetCategory() > cat ) {
-			// Set to the new position of the original category
-			file->SetCategory( file->GetCategory() - 1 );
-		}
+	for (FileQueue::iterator it = m_filelist.begin(); it != m_filelist.end(); it++) {
+		CPartFile* file = *it;
+		file->RemoveCategory(cat);
+	}
+	for (FileList::iterator it = m_completedDownloads.begin(); it != m_completedDownloads.end(); it++) {
+		CPartFile* file = *it;
+		file->RemoveCategory(cat);
 	}
 }
 
@@ -1311,14 +1391,14 @@ void CDownloadQueue::OnHostnameResolved(uint32 ip)
 }
 
 
-bool CDownloadQueue::AddLink( const wxString& link, int category )
+bool CDownloadQueue::AddLink( const wxString& link, uint8 category )
 {
 	wxString uri(link);
 
 	if (link.compare(0, 7, wxT("magnet:")) == 0) {
 		uri = CMagnetED2KConverter(link);
 		if (uri.empty()) {
-			AddLogLineM(true, CFormat(_("Cannot convert magnet link to eD2k: %s")) % link);
+			AddLogLineC(CFormat(_("Cannot convert magnet link to eD2k: %s")) % link);
 			return false;
 		}
 	}
@@ -1326,13 +1406,13 @@ bool CDownloadQueue::AddLink( const wxString& link, int category )
 	if (uri.compare(0, 7, wxT("ed2k://")) == 0) {
 		return AddED2KLink(uri, category);
 	} else {
-		AddLogLineM(true, CFormat(_("Unknown protocol of link: %s")) % link);
+		AddLogLineC(CFormat(_("Unknown protocol of link: %s")) % link);
 		return false;
 	}
 }
 
 
-bool CDownloadQueue::AddED2KLink( const wxString& link, int category )
+bool CDownloadQueue::AddED2KLink( const wxString& link, uint8 category )
 {
 	wxASSERT( !link.IsEmpty() );
 	wxString URI = link;
@@ -1347,14 +1427,14 @@ bool CDownloadQueue::AddED2KLink( const wxString& link, int category )
 
 		return AddED2KLink( uri.get(), category );
 	} catch ( const wxString& err ) {
-		AddLogLineM( true, CFormat( _("Invalid eD2k link! ERROR: %s")) % err);
+		AddLogLineC(CFormat( _("Invalid eD2k link! ERROR: %s")) % err);
 	}
 	
 	return false;
 }
 
 
-bool CDownloadQueue::AddED2KLink( const CED2KLink* link, int category )
+bool CDownloadQueue::AddED2KLink( const CED2KLink* link, uint8 category )
 {
 	switch ( link->GetKind() ) {
 		case CED2KLink::kFile:
@@ -1373,7 +1453,7 @@ bool CDownloadQueue::AddED2KLink( const CED2KLink* link, int category )
 
 
 
-bool CDownloadQueue::AddED2KLink( const CED2KFileLink* link, int category )
+bool CDownloadQueue::AddED2KLink( const CED2KFileLink* link, uint8 category )
 {
 	CPartFile* file = NULL;
 	if (IsFileExisting(link->GetHashKey())) {
@@ -1384,10 +1464,10 @@ bool CDownloadQueue::AddED2KLink( const CED2KFileLink* link, int category )
 	} else {
 		if (link->GetSize() > OLD_MAX_FILE_SIZE) {
 			if (!PlatformSpecific::CanFSHandleLargeFiles(thePrefs::GetTempDir())) {
-				AddLogLineM(true, _("Filesystem for Temp directory cannot handle large files."));
+				AddLogLineC(_("Filesystem for Temp directory cannot handle large files."));
 				return false;
 			} else if (!PlatformSpecific::CanFSHandleLargeFiles(theApp->glob_prefs->GetCatPath(category))) {
-				AddLogLineM(true, _("Filesystem for Incoming directory cannot handle large files."));
+				AddLogLineC(_("Filesystem for Incoming directory cannot handle large files."));
 				return false;
 			}
 		}
@@ -1460,31 +1540,31 @@ void CDownloadQueue::ObserverAdded( ObserverType* o )
 
 void CDownloadQueue::KademliaSearchFile(uint32_t searchID, const Kademlia::CUInt128* pcontactID, const Kademlia::CUInt128* pbuddyID, uint8_t type, uint32_t ip, uint16_t tcp, uint16_t udp, uint32_t buddyip, uint16_t buddyport, uint8_t byCryptOptions)
 {
-	AddDebugLogLineM(false, logKadSearch, wxString::Format(wxT("Search result sources (type %i)"),type));
+	AddDebugLogLineN(logKadSearch, CFormat(wxT("Search result sources (type %i)")) % type);
 
 	//Safety measure to make sure we are looking for these sources
 	CPartFile* temp = GetFileByKadFileSearchID(searchID);
 	if( !temp ) {
-		AddDebugLogLineM(false, logKadSearch, wxT("This is not the file we're looking for..."));
+		AddDebugLogLineN(logKadSearch, wxT("This is not the file we're looking for..."));
 		return;
 	}
 
 	//Do we need more sources?
 	if(!(!temp->IsStopped() && thePrefs::GetMaxSourcePerFile() > temp->GetSourceCount())) {
-		AddDebugLogLineM(false, logKadSearch, wxT("No more sources needed for this file"));
+		AddDebugLogLineN(logKadSearch, wxT("No more sources needed for this file"));
 		return;
 	}
 
 	uint32_t ED2KID = wxUINT32_SWAP_ALWAYS(ip);
 
 	if (theApp->ipfilter->IsFiltered(ED2KID)) {
-		AddDebugLogLineM(false, logKadSearch, wxT("Source ip got filtered"));
-		AddDebugLogLineM(false, logIPFilter, CFormat(wxT("IPfiltered source IP=%s received from Kademlia")) % Uint32toStringIP(ED2KID));
+		AddDebugLogLineN(logKadSearch, wxT("Source ip got filtered"));
+		AddDebugLogLineN(logIPFilter, CFormat(wxT("IPfiltered source IP=%s received from Kademlia")) % Uint32toStringIP(ED2KID));
 		return;
 	}
 
 	if( (ip == Kademlia::CKademlia::GetIPAddress() || ED2KID == theApp->GetED2KID()) && tcp == thePrefs::GetPort()) {
-		AddDebugLogLineM(false, logKadSearch, wxT("Trying to add myself as source, ignore"));
+		AddDebugLogLineN(logKadSearch, wxT("Trying to add myself as source, ignore"));
 		return;
 	}
 
@@ -1494,12 +1574,12 @@ void CDownloadQueue::KademliaSearchFile(uint32_t searchID, const Kademlia::CUInt
 		case 1: {
 			// NonFirewalled users
 			if(!tcp) {
-				AddDebugLogLineM(false, logKadSearch, CFormat(wxT("Ignored source (IP=%s) received from Kademlia, no tcp port received")) % Uint32toStringIP(ip));
+				AddDebugLogLineN(logKadSearch, CFormat(wxT("Ignored source (IP=%s) received from Kademlia, no tcp port received")) % Uint32toStringIP(ip));
 				return;
 			}
 			if (!IsGoodIP(ED2KID,thePrefs::FilterLanIPs())) {
-				AddDebugLogLineM(false, logKadSearch, CFormat(wxT("%s got filtered")) % Uint32toStringIP(ED2KID));
-				AddDebugLogLineM(false, logIPFilter, CFormat(wxT("Ignored source (IP=%s) received from Kademlia, filtered")) % Uint32toStringIP(ED2KID));
+				AddDebugLogLineN(logKadSearch, CFormat(wxT("%s got filtered")) % Uint32toStringIP(ED2KID));
+				AddDebugLogLineN(logIPFilter, CFormat(wxT("Ignored source (IP=%s) received from Kademlia, filtered")) % Uint32toStringIP(ED2KID));
 				return;
 			}
 			ctemp = new CUpDownClient(tcp, ip, 0, 0, temp, false, true);
@@ -1543,7 +1623,7 @@ void CDownloadQueue::KademliaSearchFile(uint32_t searchID, const Kademlia::CUInt
 			}
 
 			if ((byCryptOptions & 0x08) == 0){
-				AddDebugLogLineM(false, logKadSearch, CFormat(wxT("Received Kad source type 6 (direct callback) which has the direct callback flag not set (%s)")) % Uint32toStringIP(ED2KID));
+				AddDebugLogLineN(logKadSearch, CFormat(wxT("Received Kad source type 6 (direct callback) which has the direct callback flag not set (%s)")) % Uint32toStringIP(ED2KID));
 				break;
 			}
 
@@ -1561,7 +1641,7 @@ void CDownloadQueue::KademliaSearchFile(uint32_t searchID, const Kademlia::CUInt
 		// add encryption settings
 		ctemp->SetConnectOptions(byCryptOptions);
 
-		AddDebugLogLineM(false, logKadSearch, CFormat(wxT("Happily adding a source (%s) type %d")) % Uint32_16toStringIP_Port(ED2KID, ctemp->GetUserPort()) % type);
+		AddDebugLogLineN(logKadSearch, CFormat(wxT("Happily adding a source (%s) type %d")) % Uint32_16toStringIP_Port(ED2KID, ctemp->GetUserPort()) % type);
 		CheckAndAddSource(temp, ctemp);
 	}
 }
