@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2004-2006 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2004-2011 aMule Team ( admin@amule.org / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -22,24 +22,22 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
+#include "ExternalConnector.h"
+
 #ifdef HAVE_CONFIG_H
 	#include "config.h"	// Needed for VERSION and readline detection
 #endif
 
+#include <common/ClientVersion.h>
 #include <common/Format.h>		// Needed for CFormat
-
-#include <cstdio>		// Needed for fprintf(stderr, ...)
-#include <wx/filefn.h>
-#include <wx/intl.h>		// For _()
 #include <wx/tokenzr.h>		// For wxStringTokenizer
 
-#include <unistd.h>	// For getpass() and pause()
 // For readline
 #ifdef HAVE_LIBREADLINE
 	#if defined(HAVE_READLINE_READLINE_H)
-		#include <readline/readline.h>
+		#include <readline/readline.h>  // Do_not_auto_remove
 	#elif defined(HAVE_READLINE_H)
-		#include <readline.h>
+		#include <readline.h> // Do_not_auto_remove
 	#else /* !defined(HAVE_READLINE_H) */
 		extern "C" char *readline (const char*);
 	#endif /* !defined(HAVE_READLINE_H) */
@@ -50,9 +48,9 @@
 // For history
 #ifdef HAVE_READLINE_HISTORY
 	#if defined(HAVE_READLINE_HISTORY_H)
-		#include <readline/history.h>
+		#include <readline/history.h> // Do_not_auto_remove
 	#elif defined(HAVE_HISTORY_H)
-		#include <history.h>
+		#include <history.h> // Do_not_auto_remove
 	#else /* !defined(HAVE_HISTORY_H) */
 		extern "C" void add_history (const char*);
 	#endif /* defined(HAVE_READLINE_HISTORY_H) */
@@ -61,25 +59,18 @@
 #endif /* HAVE_READLINE_HISTORY */
 
 
-#include <ec/ECFileConfig.h>	// Needed for CECFileConfig
-#include <ec/ECPacket.h>		// Needed for CECPacket, CECTag
-#include <ec/ECCodes.h>		// Needed for OPcodes and TAGnames
-
+#include <ec/cpp/ECFileConfig.h>	// Needed for CECFileConfig
 #include <common/MD5Sum.h>
-#include <common/StringFunctions.h>
-
-#include "ExternalConnector.h"
-#include "OtherFunctions.h"
 
+#ifdef _MSC_VER  // silly warnings about deprecated functions
+#pragma warning(disable:4996)
+#endif
 
 //-------------------------------------------------------------------
 
 CCommandTree::~CCommandTree()
 {
-	for (CmdPos_t it = m_subcommands.begin(); it != m_subcommands.end(); ++it) {
-		delete *it;
-	}
-	m_subcommands.clear();
+	DeleteContents(m_subcommands);
 }
 
 
@@ -192,8 +183,7 @@ void CCommandTree::PrintHelpFor(const wxString& command) const
 				}
 			}
 			if (!m_parent) {
-				m_app.Show(_("\nAll commands are case insensitive.\n"
-					   "Type 'help <command>' to get detailed info on <command>.\n"));
+				m_app.Show(CFormat(_("\nAll commands are case insensitive.\nType '%s <command>' to get detailed info on <command>.\n")) % wxT("help"));
 			}
 		}
 	}
@@ -203,21 +193,28 @@ void CCommandTree::PrintHelpFor(const wxString& command) const
 //-------------------------------------------------------------------
 
 CaMuleExternalConnector::CaMuleExternalConnector()
-	: m_commands(*this)
+	: m_configFile(NULL),
+	  m_port(-1),
+	  m_KeepQuiet(false),
+	  m_Verbose(false),
+	  m_interactive(false),
+	  m_commands(*this),
+	  m_ECClient(NULL),
+	  m_InputLine(NULL),
+	  m_NeedsConfigSave(false),
+	  m_locale(NULL),
+	  m_strFullVersion(NULL),
+	  m_strOSDescription(NULL)
 {
-	m_ECClient = NULL;
-	m_KeepQuiet = false;
-	m_InputLine = NULL;
-	m_port = -1;
-	m_NeedsConfigSave = false;
-	m_Verbose = false;
-	m_configFile = NULL;
-	SetAppName(wxT("aMule"));
+	SetAppName(wxT("aMule"));	// Do not change!
 }
 
 CaMuleExternalConnector::~CaMuleExternalConnector()
 {
 	delete m_configFile;
+	delete m_locale;
+	free(m_strFullVersion);
+	free(m_strOSDescription);
 }
 
 void CaMuleExternalConnector::OnInitCommandSet()
@@ -225,8 +222,9 @@ void CaMuleExternalConnector::OnInitCommandSet()
 	m_commands.AddCommand(wxT("Quit"), CMD_ID_QUIT, wxTRANSLATE("Exits from the application."), wxEmptyString);
 	m_commands.AddCommand(wxT("Exit"), CMD_ID_QUIT, wxTRANSLATE("Exits from the application."), wxEmptyString);
 	m_commands.AddCommand(wxT("Help"), CMD_ID_HELP, wxTRANSLATE("Show help."),
-			      wxTRANSLATE("To get help on a command, type 'help <command>'.\n"
-					  "To get the full command list type 'help'.\n"));
+			      /* TRANSLATORS:
+				 Do not translate the word 'help', it is a command to the program! */
+			      wxTRANSLATE("To get help on a command, type 'help <command>'.\nTo get the full command list type 'help'.\n"));
 }
 
 void CaMuleExternalConnector::Show(const wxString &s)
@@ -308,48 +306,44 @@ bool CaMuleExternalConnector::Parse_Command(const wxString& buffer)
 
 void CaMuleExternalConnector::GetCommand(const wxString &prompt, char* buffer, size_t buffer_size)
 {
-	if( !m_KeepQuiet ) {
 #ifdef HAVE_LIBREADLINE
-		if (m_InputLine) {
-			free(m_InputLine);
-			m_InputLine = (char *)NULL;
+		char *text = readline(unicode2char(prompt + wxT("$ ")));
+		if (text && *text && 
+		    (m_InputLine == 0 || strcmp(text,m_InputLine) != 0)) {
+		  add_history (text);
 		}
-		m_InputLine = readline(unicode2char(prompt + wxT("$ ")));
-		if (m_InputLine && *m_InputLine) {
-			add_history (m_InputLine);
-		}
-		const char *text = m_InputLine;
+		if (m_InputLine)
+		  free(m_InputLine);
+		m_InputLine = text;
 #else
 		Show(prompt + wxT("$ "));
-		fflush(stdin);
-		fgets(buffer, buffer_size, stdin);
-		const char *text = buffer;
+		const char *text = fgets(buffer, buffer_size, stdin);	// == buffer if ok, NULL if eof
 #endif /* HAVE_LIBREADLINE */
 		if ( text ) {
 			size_t len = strlen(text);
-			if (len > buffer_size - 2) {
-				len = buffer_size - 2;
+			if (len > buffer_size - 1) {
+				len = buffer_size - 1;
 			}
-			strncpy(buffer, text, len);
-			buffer[len] = '\n';
-			buffer[len + 1] = 0;
+			if (buffer != text) {
+				strncpy(buffer, text, len);
+			}
+			buffer[len] = 0;
 		} else {
 			strncpy(buffer, "quit", buffer_size);
 		}
-	}
 }
 
 void CaMuleExternalConnector::TextShell(const wxString &prompt)
 {
-	char buffer[256];
+	char buffer[2048];
 	wxString buf;
 
 	bool The_End = false;
 	do {
-		GetCommand(prompt, buffer, 256);
+		GetCommand(prompt, buffer, sizeof buffer);
 		buf = char2unicode(buffer);
 		The_End = Parse_Command(buf);
-	} while ((!The_End) && (m_ECClient->IsConnected()));
+	} while ((!The_End) && (m_ECClient->IsSocketConnected()));
 }
 
 void CaMuleExternalConnector::ConnectAndRun(const wxString &ProgName, const wxString& ProgVersion)
@@ -359,19 +353,10 @@ void CaMuleExternalConnector::ConnectAndRun(const wxString &ProgName, const wxSt
 		return;
 	}
 
-	wxString appName =
-		// Find out Application Name
-		#ifdef WEBSERVERDIR
-			wxT("amuleweb")
-		#else
-			wxT("amulecmd")
-		#endif
-	;
-
-	#ifdef CVSDATE
-		Show(CFormat(_("This is %s %s %s\n")) % appName % wxT(VERSION) % wxT(CVSDATE));
+	#ifdef SVNDATE
+		Show(CFormat(_("This is %s %s %s\n")) % wxString::FromAscii(m_appname) % wxT(VERSION) % wxT(SVNDATE));
 	#else
-		Show(CFormat(_("This is %s %s\n")) % appName % wxT(VERSION));
+		Show(CFormat(_("This is %s %s\n")) % wxString::FromAscii(m_appname) % wxT(VERSION));
 	#endif
 
 	// HostName, Port and Password
@@ -380,7 +365,7 @@ void CaMuleExternalConnector::ConnectAndRun(const wxString &ProgName, const wxSt
 		#ifndef __WXMSW__
 			pass_plain = char2unicode(getpass("Enter password for mule connection: "));
 		#else
-			#warning This way, pass enter is not hidden on windows. Bad thing.
+			//#warning This way, pass enter is not hidden on windows. Bad thing.
 			char temp_str[512];
 			fflush(stdin);
 			printf("Enter password for mule connection: \n");
@@ -404,46 +389,42 @@ void CaMuleExternalConnector::ConnectAndRun(const wxString &ProgName, const wxSt
 		// Create the socket
 		Show(_("\nCreating client...\n"));
 		m_ECClient = new CRemoteConnect(NULL);
+		m_ECClient->SetCapabilities(m_ZLIB, true, false);	// ZLIB, UTF8 numbers, notification
 
-		m_ECClient->ConnectToCore(m_host, m_port, wxT("foobar"), m_password.Encode(),
-								 ProgName, ProgVersion);
-		
-		m_ECClient->WaitOnConnect(10);
-
-		if (!m_ECClient->IsConnected()) {
+		// ConnectToCore is blocking since m_ECClient was initialized with NULL
+		if (!m_ECClient->ConnectToCore(m_host, m_port, wxT("foobar"), m_password.Encode(), ProgName, ProgVersion)) {
 			// no connection => close gracefully
-			Show(_("Connection Failed. Unable to connect to the specified host\n"));
+			if (!m_ECClient->GetServerReply().IsEmpty()) {
+					Show(CFormat(wxT("%s\n")) % m_ECClient->GetServerReply());
+			}
+			Show(CFormat(_("Connection Failed. Unable to connect to %s:%d\n")) % m_host % m_port);
 		} else {
 			// Authenticate ourselves
 			// ConnectToCore() already authenticated for us.
 			//m_ECClient->ConnectionEstablished();
 			Show(m_ECClient->GetServerReply()+wxT("\n"));
-			if (m_ECClient->IsConnected()) {
-				ShowGreet();
-				Pre_Shell();
-				if (m_KeepQuiet) {
-					while(true) {
-						#ifndef __WXMSW__
-						pause();
-						#else
-						wxSleep(10);
-						#endif
-					}
-				} else {
-					TextShell(ProgName);
+			if (m_ECClient->IsSocketConnected()) {
+				if (m_interactive) {
+					ShowGreet();
 				}
-				Show(CFormat(_("\nOk, exiting %s...\n")) % ProgName);
+				Pre_Shell();
+				TextShell(ProgName);
 				Post_Shell();
+				if (m_interactive) {
+					Show(CFormat(_("\nOk, exiting %s...\n")) % ProgName);
+				}
 			}
 		}
-		m_ECClient->Destroy();
+		m_ECClient->DestroySocket();
 	} else {
 		Show(_("Cannot connect with an empty password.\nYou must specify a password either in config file\nor on command-line, or enter one when asked.\n\nExiting...\n"));
 	}
 }
 
-void CaMuleExternalConnector::OnInitCmdLine(wxCmdLineParser& parser)
+void CaMuleExternalConnector::OnInitCmdLine(wxCmdLineParser& parser, const char* appname)
 {
+	m_appname = appname;
+
 	parser.AddSwitch(wxEmptyString, wxT("help"),
 		_("Show this help text."),
 		wxCMD_LINE_PARAM_OPTIONAL);
@@ -482,15 +463,7 @@ void CaMuleExternalConnector::OnInitCmdLine(wxCmdLineParser& parser)
 bool CaMuleExternalConnector::OnCmdLineParsed(wxCmdLineParser& parser)
 {
 	if (parser.Found(wxT("version"))) {
-		const char *appName =
-			// Find out Application Name
-			#ifdef WEBSERVERDIR
-				"amuleweb"
-			#else
-				"amulecmd"
-			#endif
-		;
-		printf("%s %s\n", appName, (const char *)unicode2char(GetMuleVersion()));
+		printf("%s %s\n", m_appname, (const char *)unicode2char(GetMuleVersion()));
 		return false;
 	}
 
@@ -521,11 +494,7 @@ bool CaMuleExternalConnector::OnCmdLineParsed(wxCmdLineParser& parser)
 	}
 
 	long port;
-	if (!parser.Found(wxT("port"), &port)) {
-		if (m_port == -1) {
-			m_port = 4712;
-		}
-	} else {
+	if (parser.Found(wxT("port"), &port)) {
 		m_port = port;
 	}
 
@@ -558,7 +527,7 @@ bool CaMuleExternalConnector::OnCmdLineParsed(wxCmdLineParser& parser)
 void CaMuleExternalConnector::LoadAmuleConfig(CECFileConfig& cfg)
 {
 	m_host = wxT("localhost");
-	m_port = cfg.Read(wxT("/ExternalConnect/ECPort"), -1l);
+	m_port = cfg.Read(wxT("/ExternalConnect/ECPort"), 4712l);
 	cfg.ReadHash(wxT("/ExternalConnect/ECPassword"), &m_password);
 	m_language = cfg.Read(wxT("/eMule/Language"), wxEmptyString);
 }
@@ -572,8 +541,9 @@ void CaMuleExternalConnector::LoadConfigFile()
 	if (m_configFile) {
 		m_language = m_configFile->Read(wxT("/Locale"), wxEmptyString);
 		m_host = m_configFile->Read(wxT("/EC/Host"), wxEmptyString);
-		m_port = m_configFile->Read(wxT("/EC/Port"), -1l);
+		m_port = m_configFile->Read(wxT("/EC/Port"), 4712l);
 		m_configFile->ReadHash(wxT("/EC/Password"), &m_password);
+		m_ZLIB = m_configFile->Read(wxT("/EC/ZLIB"), 1l) != 0;
 	}
 }
 
@@ -595,18 +565,44 @@ void CaMuleExternalConnector::SaveConfigFile()
 
 bool CaMuleExternalConnector::OnInit()
 {
+#ifndef __WXMSW__
+	#if wxUSE_ON_FATAL_EXCEPTION
+		// catch fatal exceptions
+		wxHandleFatalExceptions(true);
+	#endif
+#endif
+
+	m_strFullVersion = strdup((const char *)unicode2char(GetMuleVersion()));
+	m_strOSDescription = strdup((const char *)unicode2char(wxGetOsDescription()));
+
+	// Handle uncaught exceptions
+	InstallMuleExceptionHandler();
+
 	bool retval = wxApp::OnInit();
 	OnInitCommandSet();
 	InitCustomLanguages();
-	InitLocale(m_locale, StrLang2wx(m_language));
+	SetLocale(m_language);
 	return retval;
 }
 
+wxString CaMuleExternalConnector::SetLocale(const wxString& language)
+{
+	if (!language.IsEmpty()) {
+		m_language = language;
+		if (m_locale) {
+			delete m_locale;
+		}
+		m_locale = new wxLocale;
+		InitLocale(*m_locale, StrLang2wx(language));
+	}
 
-#if !wxUSE_GUI && defined(__WXMAC__)
+	return m_locale == NULL ? wxString() : m_locale->GetCanonicalName();
+}
 
-#include <wx/apptrait.h>
-#include <wx/stdpaths.h>
+#if !wxUSE_GUI && defined(__WXMAC__) && !wxCHECK_VERSION(2, 9, 0)
+
+#include <wx/apptrait.h> // Do_not_auto_remove
+#include <wx/stdpaths.h> // Do_not_auto_remove
 
 class CaMuleExternalConnectorTraits : public wxConsoleAppTraits
 {
@@ -628,3 +624,45 @@ wxAppTraits* CaMuleExternalConnector::CreateTraits()
 }
 
 #endif
+
+#if wxUSE_ON_FATAL_EXCEPTION
+// Gracefully handle fatal exceptions and print backtrace if possible
+void CaMuleExternalConnector::OnFatalException()
+{
+	/* Print the backtrace */
+	fprintf(stderr, "\n--------------------------------------------------------------------------------\n");	
+	fprintf(stderr, "A fatal error has occurred and %s has crashed.\n", m_appname);
+	fprintf(stderr, "Please assist us in fixing this problem by posting the backtrace below in our\n");
+	fprintf(stderr, "'aMule Crashes' forum and include as much information as possible regarding the\n");
+	fprintf(stderr, "circumstances of this crash. The forum is located here:\n");
+	fprintf(stderr, "    http://forum.amule.org/index.php?board=67.0\n");
+	fprintf(stderr, "If possible, please try to generate a real backtrace of this crash:\n");
+	fprintf(stderr, "    http://wiki.amule.org/index.php/Backtraces\n\n");
+	fprintf(stderr, "----------------------------=| BACKTRACE FOLLOWS: |=----------------------------\n");
+	fprintf(stderr, "Current version is: %s %s\n", m_appname, m_strFullVersion);
+	fprintf(stderr, "Running on: %s\n\n", m_strOSDescription);
+	
+	print_backtrace(1); // 1 == skip this function.
+	
+	fprintf(stderr, "\n--------------------------------------------------------------------------------\n");	
+}
+#endif
+
+#ifdef __WXDEBUG__
+void CaMuleExternalConnector::OnAssertFailure(const wxChar *file, int line, const wxChar *func, const wxChar *cond, const wxChar *msg)
+{
+#if !defined wxUSE_STACKWALKER || !wxUSE_STACKWALKER
+	wxString errmsg = CFormat( wxT("%s:%s:%d: Assertion '%s' failed. %s") ) % file % func % line % cond % ( msg ? msg : wxT("") );
+
+	fprintf(stderr, "Assertion failed: %s\n", (const char*)unicode2char(errmsg));
+
+	// Skip the function-calls directly related to the assert call.
+	fprintf(stderr, "\nBacktrace follows:\n");
+	print_backtrace(3);
+	fprintf(stderr, "\n");
+#else
+	wxApp::OnAssertFailure(file, line, func, cond, msg);
+#endif
+}
+#endif
+// File_checked_for_headers
