@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -28,15 +28,13 @@
 
 
 #include "KnownFile.h"		// Needed for CKnownFile
-#include "CFile.h"		// Needed for CFile
+#include "FileAutoClose.h"	// Needed for CFileAutoClose
 
-#include "OtherStructs.h"	// Needed for Gap_Struct
+#include "OtherStructs.h"	// Needed for Requested_Block_Struct
 #include "DeadSourceList.h"	// Needed for CDeadSourceList
-
-
+#include "GapList.h"
 
 class CSearchFile;
-class CUpDownClient;
 class CMemFile;
 class CFileDataIO;
 class CED2KFileLink;
@@ -44,8 +42,6 @@ class CED2KFileLink;
 //#define BUFFER_SIZE_LIMIT	500000 // Max bytes before forcing a flush
 #define BUFFER_TIME_LIMIT	60000   // Max milliseconds before forcing a flush
 
-#define	STATES_COUNT			13
-
 // Ok, eMule and aMule are building incompatible backup files because 
 // of the different name. aMule was using ".BAK" and eMule ".bak". 
 // This should fix it.
@@ -60,14 +56,6 @@ enum EPartFileFormat {
 	PMT_BADFORMAT
 };
 
-struct PartFileBufferedData
-{
-	byte *data;						// Barry - This is the data to be written
-	uint64 start;					// Barry - This is the start offset of the data
-	uint64 end;						// Barry - This is the end offset of the data
-	Requested_Block_Struct *block;	// Barry - This is the requested block that this data relates to
-};
-
 
 class SFileRating
 {
@@ -89,20 +77,18 @@ class SourcenameItem
 {
 public:
 	wxString	name;
-	long		count;
+	int			count;
 public:
-	SourcenameItem(const wxString &n, long c)
+	SourcenameItem(const wxString &n = EmptyString, int c = 0)
 	:
 	name(n), count(c) {}
 };
 
-typedef std::list<SourcenameItem> SourcenameItemList;
+typedef std::map<uint32,SourcenameItem> SourcenameItemMap;
 
 class CPartFile : public CKnownFile {
 public:
-	typedef std::list<Gap_Struct*> CGapPtrList;
 	typedef std::list<Requested_Block_Struct*> CReqBlockPtrList;
-
 	
 	CPartFile();
 #ifdef CLIENT_GUI
@@ -114,11 +100,14 @@ public:
 	CPartFile(const CED2KFileLink* fileLink);
 	virtual ~CPartFile();
 	
-	bool	CreateFromFile(wxString WXUNUSED(directory), wxString WXUNUSED(filename), void* WXUNUSED(pvProgressParam)) {return false;}// not supported in this class
-	void 	SetPartFileStatus(uint8 newstatus);
 	virtual bool LoadFromFile(const CFileDataIO* WXUNUSED(file)) { return false; }
 	bool	WriteToFile(CFileDataIO* WXUNUSED(file))	{ return false; }
-	bool	IsPartFile() const		{ return !(status == PS_COMPLETE); }
+
+	// virtual functions for CKnownFile and CPartFile:
+	bool	IsPartFile() const		{ return status != PS_COMPLETE; }	// true if not completed
+	bool	IsCompleted() const		{ return status == PS_COMPLETE; }	// true if completed
+	bool	IsCPartFile() const		{ return true; }					// true if it's a CPartFile
+
 	uint32	Process(uint32 reducedownload, uint8 m_icounter);
 	uint8	LoadPartFile(const CPath& in_directory, const CPath& filename, bool from_backup = false, bool getsizeonly = false);
 	bool	SavePartFile(bool Initial = false);
@@ -127,11 +116,12 @@ public:
 	
 	bool    CheckShowItemInGivenCat(int inCategory);
 
-	bool	IsComplete(uint64 start, uint64 end);
+	bool	IsComplete(uint64 start, uint64 end)	{ return m_gaplist.IsComplete(start, end); }
+	bool	IsComplete(uint16 part)			{ return m_gaplist.IsComplete(part); }
 	
 	void	UpdateCompletedInfos();
 
-	bool	GetNextRequestedBlock(CUpDownClient* sender,Requested_Block_Struct** newblocks,uint16* count);
+	bool	GetNextRequestedBlock(CUpDownClient* sender, std::vector<Requested_Block_Struct*>& toadd, uint16& count);
 	void	WritePartStatus(CMemFile* file);
 	void	WriteCompleteSourcesCount(CMemFile* file);
 	static bool 	CanAddSource(uint32 userid, uint16 port, uint32 serverip, uint16 serverport, uint8* pdebug_lowiddropped = NULL, bool ed2kID = true);
@@ -144,9 +134,10 @@ public:
 #endif
 	virtual void	UpdatePartsInfo();
 	const CPath& GetPartMetFileName() const { return m_partmetfilename; }
+	uint16	GetPartMetNumber() const;
 	uint64	GetTransferred() const		{ return transferred; }
 	const CPath& GetFullName() const	{ return m_fullname; }
-	float	GetKBpsDown() const		{ return kBpsDown; }
+	float	GetKBpsDown() const			{ return kBpsDown; }
 	double	GetPercentCompleted() const	{ return percentcompleted; }
 
 #ifndef CLIENT_GUI
@@ -159,7 +150,6 @@ public:
 #endif
 	uint16	GetTransferingSrcCount() const	{ return transferingsrc; }
 	uint16  GetNotCurrentSourcesCount()	const	{ return m_notCurrentSources; };
-	void	SetNotCurrentSourcesCount(uint16 new_count)	{ m_notCurrentSources = new_count; };	
 	uint16	GetValidSourcesCount() const	{ return m_validSources; };
 	
 	uint64	GetNeededSpace();
@@ -172,11 +162,8 @@ public:
 	int	getPartfileStatusRang() const;
 
 	// Barry - Added as replacement for BlockReceived to buffer data before writing to disk
-	uint32	WriteToBuffer(uint32 transize, byte *data, uint64 start, uint64 end, Requested_Block_Struct *block);
-	void	FlushBuffer(bool forcewait=false, bool bForceICH = false, bool bNoAICH = false);	
-
-	// Barry - Is archive recovery in progress
-	volatile bool m_bRecoveringArchive;
+	uint32	WriteToBuffer(uint32 transize, byte *data, uint64 start, uint64 end, Requested_Block_Struct *block, const CUpDownClient* client);
+	void	FlushBuffer(bool fromAICHRecoveryDataAvailable = false);	
 
 	// Barry - Added to prevent list containing deleted blocks on shutdown
 	void	RemoveAllRequestedBlocks(void);
@@ -192,7 +179,7 @@ public:
 	void    AddClientSources(CMemFile* sources, unsigned nSourceFrom, uint8 uClientSXVersion, bool bSourceExchange2, const CUpDownClient* pClient = NULL);
 
 	bool	PreviewAvailable();
-	uint8	GetAvailablePartCount() const	{ return m_availablePartsCount; }
+	uint16	GetAvailablePartCount() const	{ return m_availablePartsCount; }
 	uint32	GetLastAnsweredTime() const	{ return m_ClientSrcAnswered; }
 	void	SetLastAnsweredTime();
 	void	SetLastAnsweredTimeTimeout();
@@ -202,16 +189,14 @@ public:
 	bool	IsStopped() const		{ return this ? m_stopped : true; }
 	bool	IsPaused() const		{ return m_paused; }
 	void	UpdateFileRatingCommentAvail();
-	bool	m_CommentUpdated;
 
 	int	GetCommonFilePenalty();
 	void	UpdateDisplayedInfo(bool force = false);
 	
 	uint8	GetCategory() const { return m_category; }
 	void	SetCategory(uint8 cat);
+	void	RemoveCategory(uint8 cat);
 
-	CFile	m_hpartfile;	//permanent opened handle to avoid write conflicts
-	volatile bool m_bPreviewing;
 	void	SetDownPriority(uint8 newDownPriority, bool bSave = true, bool bRefresh = true);
 	bool	IsAutoDownPriority() const	{ return m_bAutoDownPriority; }
 	void	SetAutoDownPriority(bool flag)	{ m_bAutoDownPriority = flag; }
@@ -260,17 +245,13 @@ public:
 
 	ArrayOfUInts16	m_SrcpartFrequency;
 
-	void	SetShowSources( bool val )	{ m_showSources = val; }
-	bool	ShowSources()				const { return m_showSources; }
-
-	typedef std::set<CUpDownClient*> SourceSet;
-	
 	const SourceSet& GetSourceList()	const { return m_SrcList; }
 	const SourceSet& GetA4AFList()		const { return m_A4AFsrclist; }
+	void	ClearA4AFList()				{ m_A4AFsrclist.clear(); }
 
-	const CGapPtrList		GetGapList() const	{ return m_gaplist; }
 	const CReqBlockPtrList	GetRequestedBlockList() const { return m_requestedblocks_list; }
 
+	const CGapList& GetGapList() const { return m_gaplist; }
 
 	/**
 	 * Adds a source to the list of dead sources.
@@ -295,23 +276,27 @@ public:
 	uint16	GetMaxSourcePerFileSoft() const;
 	uint16	GetMaxSourcePerFileUDP() const;		 
 
-	const FileRatingList &GetRatingAndComments();
+	void GetRatingAndComments(FileRatingList & list) const;
 
 	void	AllocationFinished();
 private:
-	//! A local list of sources that are invalid for this file.
 #ifndef CLIENT_GUI
+	// partfile handle (opened on demand)
+	CFileAutoClose	m_hpartfile;
+	//! A local list of sources that are invalid for this file.
 	CDeadSourceList	m_deadSources;
+
+	class CCorruptionBlackBox* m_CorruptionBlackBox;
 #endif
 
 	uint16	m_notCurrentSources;
-
-	bool	m_showSources;
 	
 	uint32	m_validSources;
 
 	void	AddGap(uint64 start, uint64 end);
+	void	AddGap(uint16 part);
 	void	FillGap(uint64 start, uint64 end);
+	void	FillGap(uint16 part);
 	bool	GetNextEmptyBlockInPart(uint16 partnumber,Requested_Block_Struct* result);
 	bool	IsAlreadyRequested(uint64 start, uint64 end);
 	void	CompleteFile(bool hashingdone);
@@ -322,13 +307,8 @@ private:
 	
 	bool	IsCorruptedPart(uint16 partnumber);
 	
-	uint64	GetTotalGapSizeInPart(uint32 uPart) const;
-
-	uint64	GetTotalGapSizeInRange(uint64 uRangeStart, uint64 uRangeEnd) const;	
-	
 	uint32	m_iLastPausePurge;
 	uint16	m_count;
-	uint16	m_anStates[STATES_COUNT];
 	uint16	transferingsrc;
 	uint64  completedsize;
 	uint64	transferred;
@@ -337,8 +317,9 @@ private:
 	uint64	m_iGainDueToCompression;
 	uint32  m_iTotalPacketsSavedDueToICH;
 	float 	kBpsDown;
-	CPath	m_fullname;
-	CPath	m_partmetfilename;
+	CPath	m_fullname;			// path/name of the met file
+	CPath	m_partmetfilename;	// name of the met file
+	CPath 	m_PartPath; 		// path/name of the partfile
 	bool	m_paused;
 	bool	m_stopped;
 	bool	m_insufficient;
@@ -347,11 +328,11 @@ private:
 	uint8	status;
 	uint32	lastpurgetime;
 	uint32	m_LastNoNeededCheck;
-	CGapPtrList m_gaplist;
+	CGapList m_gaplist;
 	CReqBlockPtrList m_requestedblocks_list;
 	double	percentcompleted;
 	std::list<uint16> m_corrupted_list;
-	uint8	m_availablePartsCount;
+	uint16	m_availablePartsCount;
 	uint32	m_ClientSrcAnswered;
 	bool	m_bPercentUpdated;
 
@@ -359,8 +340,8 @@ private:
 
 	uint32		m_lastRefreshedDLDisplay;
 
-	// Barry - Buffered data to be written
-	std::list<PartFileBufferedData*> m_BufferedData_list;
+	// Buffered data to be written
+	std::list<class PartFileBufferedData*> m_BufferedData_list;
 	
 	uint32 m_nTotalBufferData;
 	uint32 m_nLastBufferFlushTime;
@@ -375,21 +356,27 @@ private:
 	bool		m_hashsetneeded;
 	uint32		m_lastsearchtime;
 	bool		m_localSrcReqQueued;
-	
-	FileRatingList m_FileRatingList;
+
 #ifdef CLIENT_GUI
-	SourcenameItemList m_SourcenameItem_list;
-public:
-	const SourcenameItemList &GetSourcenameItemList() { return m_SourcenameItem_list; }
-	void ClearSourcenameItemList() { m_SourcenameItem_list.clear(); }
-	void AddSourcenameItemList(const wxString &name, long count) { m_SourcenameItem_list.push_back(SourcenameItem(name, count)); }
-#endif
-public:
+	FileRatingList m_FileRatingList;
 	const FileRatingList &GetFileRatingList() { return m_FileRatingList; }
 	void ClearFileRatingList() { m_FileRatingList.clear(); }
 	void AddFileRatingList(const wxString & u, const wxString & f, sint16 r, const wxString & c) { 
 	       m_FileRatingList.push_back(SFileRating(u, f, r, c)); }
 
+	uint32 	m_kbpsDown;
+	uint8   m_iDownPriorityEC;
+	bool	m_isShared;
+	SourcenameItemMap m_SourcenameItemMap;
+
+	ListOfUInts32	m_A4AFClientIDs;
+	ListOfUInts32 & GetA4AFClientIDs()			{ return m_A4AFClientIDs; }
+public:
+	bool	IsShared() const					{ return m_isShared; }
+	SourcenameItemMap &GetSourcenameItemMap()	{ return m_SourcenameItemMap; }
+	PartFileEncoderData m_PartFileEncoderData;
+#endif
+public:
 	bool IsHashSetNeeded() const				{ return m_hashsetneeded; }
 	void SetHashSetNeeded(bool value)			{ m_hashsetneeded = value; }
 	
@@ -399,15 +386,12 @@ public:
 	bool IsLocalSrcRequestQueued() const		{ return m_localSrcReqQueued; }
 	void SetLocalSrcRequestQueued(bool value) 	{ m_localSrcReqQueued = value; }
 
-	void AddA4AFSource(CUpDownClient* src)		{ m_A4AFsrclist.insert(src); }
-	bool RemoveA4AFSource(CUpDownClient* src)	{ return (m_A4AFsrclist.erase(src) > 0); }
+	void AddA4AFSource(CUpDownClient* src)		{ m_A4AFsrclist.insert(CCLIENTREF(src, wxT("A4AFSource"))); }
+	bool RemoveA4AFSource(CUpDownClient* src)	{ return (m_A4AFsrclist.erase(CCLIENTREF(src, wxEmptyString)) > 0); }
 
 	uint32 GetLastSearchTime() const			{ return m_lastsearchtime; }
 	void SetLastSearchTime(uint32 time)			{ m_lastsearchtime = time; }
 	
-
-//	void CleanUpSources( bool noNeeded, bool fullQueue = false, bool highQueue = false );
-
 	void AddDownloadingSource(CUpDownClient* client);
           
 	void RemoveDownloadingSource(CUpDownClient* client);
@@ -423,18 +407,20 @@ public:
 	void LoadSourceSeeds();
 	
 	// Dropping slow sources
-	
-	CUpDownClient* GetSlowerDownloadingClient(uint32 speed, CUpDownClient* caller) ;
+	CUpDownClient* GetSlowerDownloadingClient(uint32 speed, CUpDownClient* caller);
+
+  // Read data for sharing
+	bool ReadData(class CFileArea & area, uint64 offset, uint32 toread);
 
 private:
 	/* downloading sources list */
-	CClientPtrList m_downloadingSourcesList;
+	CClientRefList m_downloadingSourcesList;
 
 	/* Kad Stuff */
 	uint32	m_LastSearchTimeKad;
 	uint8	m_TotalSearchesKad;
 
-friend class CDownQueueRem;
+friend class CKnownFilesRem;
 friend class CPartFileConvert;
 };
 
