@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -74,7 +74,7 @@ void ParsedSearchExpression(const CSearchExpr* pexpr)
 	int iOpOr = 0;
 	int iOpNot = 0;
 
-	for (unsigned int i = 0; i < pexpr->m_aExpr.Count(); i++) {
+	for (unsigned int i = 0; i < pexpr->m_aExpr.GetCount(); i++) {
 		wxString str(pexpr->m_aExpr[i]);
 		if (str == SEARCHOPTOK_AND) {
 			iOpAnd++;
@@ -109,7 +109,7 @@ void ParsedSearchExpression(const CSearchExpr* pexpr)
 	// optimize search expression, if no OR nor NOT specified
 	if (iOpAnd > 0 && iOpOr == 0 && iOpNot == 0) {
 		wxString strAndTerms;
-		for (unsigned int i = 0; i < pexpr->m_aExpr.Count(); i++) {
+		for (unsigned int i = 0; i < pexpr->m_aExpr.GetCount(); i++) {
 			if (pexpr->m_aExpr[i] != SEARCHOPTOK_AND) {
 				// Minor optimization: Because we added the Kad keyword to the boolean search expression,
 				// we remove it here (and only here) again because we know that the entire search expression
@@ -122,7 +122,7 @@ void ParsedSearchExpression(const CSearchExpr* pexpr)
 				}
 			}
 		}
-		wxASSERT( _SearchExpr.m_aExpr.Count() == 0);
+		wxASSERT( _SearchExpr.m_aExpr.GetCount() == 0);
 		_SearchExpr.m_aExpr.Add(strAndTerms);
 	} else {
 		if (pexpr->m_aExpr.GetCount() != 1 || pexpr->m_aExpr[0] != s_strCurKadKeyword)			
@@ -259,7 +259,7 @@ CSearchList::CSearchList()
 
 CSearchList::~CSearchList()
 {
-	StopGlobalSearch();
+	StopSearch();
 
 	while (!m_results.empty()) {
 		RemoveResults(m_results.begin()->first);
@@ -313,10 +313,10 @@ wxString CSearchList::StartNewSearch(uint32* searchID, SearchType type, const CS
 	CMemFilePtr data = CreateSearchData(params, type, supports64bit, packetUsing64bit);
 	
 	if (data.get() == NULL) {
-		wxASSERT(_astrParserErrors.Count());
+		wxASSERT(_astrParserErrors.GetCount());
 		wxString error;
 		
-		for (unsigned int i = 0; i < _astrParserErrors.Count(); ++i) {
+		for (unsigned int i = 0; i < _astrParserErrors.GetCount(); ++i) {
 			error += _astrParserErrors[i] + wxT("\n");
 		}
 		
@@ -331,13 +331,15 @@ wxString CSearchList::StartNewSearch(uint32* searchID, SearchType type, const CS
 			}
 		
 			// searchstring will get tokenized there
-			// The tab must be created with the Kad search ID, so seardhID is updated.
+			// The tab must be created with the Kad search ID, so searchID is updated.
 			Kademlia::CSearch* search = Kademlia::CSearchManager::PrepareFindKeywords(
 										 params.searchString, data->GetLength(), data->GetRawBuffer(), *searchID);
 
 			*searchID = search->GetSearchID();
+			m_currentSearch = *searchID;
+			m_KadSearchFinished = false;
 		} catch (const wxString& what) {
-			AddLogLineM(true, what);
+			AddLogLineC(what);
 			return _("Unexpected error while attempting Kad search: ") + what;				
 		}
 	} else {
@@ -351,6 +353,7 @@ wxString CSearchList::StartNewSearch(uint32* searchID, SearchType type, const CS
 		theApp->serverconnect->SendPacket(searchPacket, (type == LocalSearch));
 
 		if (type == GlobalSearch) {
+			delete m_searchPacket;
 			m_searchPacket = searchPacket;
 			m_64bitSearchPacket = packetUsing64bit;
 			m_searchPacket->SetOpCode(OP_GLOBSEARCHREQ); // will be changed later when actually sending the packet!!
@@ -378,7 +381,12 @@ void CSearchList::LocalSearchEnd()
 
 uint32 CSearchList::GetSearchProgress() const
 {
-	if (m_searchInProgress == false) {
+	if (m_searchType == KadSearch) {
+		// We cannot measure the progress of Kad searches.
+		// But we can tell when they are over.
+		return m_KadSearchFinished ? 0xfffe : 0;
+	}
+	if (m_searchInProgress == false) {	// true only for ED2K search
 		// No search, no progress ;)
 		return 0;
 	}
@@ -390,14 +398,11 @@ uint32 CSearchList::GetSearchProgress() const
 		case GlobalSearch:
 			return 100 - (m_serverQueue.GetRemaining() * 100) 
 					/ theApp->serverlist->GetServerCount();
-
-		case KadSearch:
-			// We cannot measure the progress of Kad searches.
-			return 0;
 		
 		default:
-			wxCHECK(false, 0);
+			wxFAIL;
 	}
+	return 0;
 }
 
 
@@ -435,24 +440,24 @@ void CSearchList::OnGlobalSearchTimer(CTimerEvent& WXUNUSED(evt))
 					extSearchPacket->CopyToDataBuffer(data.GetLength(), m_searchPacket->GetDataBuffer(), m_searchPacket->GetPacketSize());
 					theStats::AddUpOverheadServer(extSearchPacket->GetPacketSize());
 					theApp->serverconnect->SendUDPPacket(extSearchPacket, server, true);
-					AddDebugLogLineM(false, logServerUDP, wxT("Sending OP_GLOBSEARCHREQ3 to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
+					AddDebugLogLineN(logServerUDP, wxT("Sending OP_GLOBSEARCHREQ3 to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
 				} else if (server->GetUDPFlags() & SRV_UDPFLG_EXT_GETFILES) {
 					if (!m_64bitSearchPacket || server->SupportsLargeFilesUDP()) {
 						m_searchPacket->SetOpCode(OP_GLOBSEARCHREQ2);
-						AddDebugLogLineM(false, logServerUDP, wxT("Sending OP_GLOBSEARCHREQ2 to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
+						AddDebugLogLineN(logServerUDP, wxT("Sending OP_GLOBSEARCHREQ2 to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
 						theStats::AddUpOverheadServer(m_searchPacket->GetPacketSize());
 						theApp->serverconnect->SendUDPPacket(m_searchPacket, server, false);
 					} else {
-						AddDebugLogLineM(false, logServerUDP, wxT("Skipped UDP search on server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()) + wxT(": No large file support"));
+						AddDebugLogLineN(logServerUDP, wxT("Skipped UDP search on server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()) + wxT(": No large file support"));
 					}
 				} else {
 					if (!m_64bitSearchPacket || server->SupportsLargeFilesUDP()) {
 						m_searchPacket->SetOpCode(OP_GLOBSEARCHREQ);
-						AddDebugLogLineM(false, logServerUDP, wxT("Sending OP_GLOBSEARCHREQ to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
+						AddDebugLogLineN(logServerUDP, wxT("Sending OP_GLOBSEARCHREQ to server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()));
 						theStats::AddUpOverheadServer(m_searchPacket->GetPacketSize());
 						theApp->serverconnect->SendUDPPacket(m_searchPacket, server, false);
 					} else {
-						AddDebugLogLineM(false, logServerUDP, wxT("Skipped UDP search on server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()) + wxT(": No large file support"));
+						AddDebugLogLineN(logServerUDP, wxT("Skipped UDP search on server ") + Uint32_16toStringIP_Port(server->GetIP(), server->GetPort()) + wxT(": No large file support"));
 					}
 				}
 				CoreNotify_Search_Update_Progress(GetSearchProgress());
@@ -461,7 +466,7 @@ void CSearchList::OnGlobalSearchTimer(CTimerEvent& WXUNUSED(evt))
 		}
 	}	
 	// No more servers left to ask.
-	StopGlobalSearch();
+	StopSearch(true);
 }
 
 
@@ -483,7 +488,7 @@ void CSearchList::ProcessSharedFileList(const byte* in_packet, uint32 size,
 	bool unicoded = (sender->GetUnicodeSupport() != utf8strNone);
 	for (unsigned int i = 0; i != results; ++i){			
 		CSearchFile* toadd = new CSearchFile(packet, unicoded, searchID, 0, 0, directory);
-		if (sender){
+		if (sender) {
 			toadd->SetClientID(sender->GetUserIDHybrid());
 			toadd->SetClientPort(sender->GetUserPort());
 		}
@@ -510,9 +515,9 @@ void CSearchList::ProcessSearchAnswer(const uint8_t* in_packet, uint32_t size, b
 {
 	CMemFile packet(in_packet, size);
 
-       uint32_t results = packet.ReadUInt32();
-       for (; results > 0; --results) {
-               AddToList(new CSearchFile(packet, optUTF8, m_currentSearch, serverIP, serverPort), false);
+	uint32_t results = packet.ReadUInt32();
+	for (; results > 0; --results) {
+		AddToList(new CSearchFile(packet, optUTF8, m_currentSearch, serverIP, serverPort), false);
 	}
 }
 
@@ -528,7 +533,7 @@ bool CSearchList::AddToList(CSearchFile* toadd, bool clientResponse)
 	const uint64 fileSize = toadd->GetFileSize();
 	// If filesize is 0, or file is too large for the network, drop it 
 	if ((fileSize == 0) || (fileSize > MAX_FILE_SIZE)) {
-		AddDebugLogLineM(false, logSearch,
+		AddDebugLogLineN(logSearch,
 				CFormat(wxT("Dropped result with filesize %u: %s"))
 					% fileSize
 					% toadd->GetFileName());
@@ -540,7 +545,7 @@ bool CSearchList::AddToList(CSearchFile* toadd, bool clientResponse)
 	// If the result was not the type the user wanted, drop it.
 	if ((clientResponse == false) && !m_resultType.IsEmpty()) {
 		if (GetFileTypeByName(toadd->GetFileName()) != m_resultType) {
-			AddDebugLogLineM( false, logSearch,
+			AddDebugLogLineN(logSearch,
 				CFormat( wxT("Dropped result type %s != %s, file %s") )
 					% GetFileTypeByName(toadd->GetFileName())
 					% m_resultType
@@ -559,7 +564,7 @@ bool CSearchList::AddToList(CSearchFile* toadd, bool clientResponse)
 		CSearchFile* item = results.at(i);
 		
 		if ((toadd->GetFileHash() == item->GetFileHash()) && (toadd->GetFileSize() == item->GetFileSize())) {
-			AddDebugLogLineM(false, logSearch, CFormat(wxT("Received duplicate results for '%s' : %s")) % item->GetFileName() % item->GetFileHash().Encode());
+			AddDebugLogLineN(logSearch, CFormat(wxT("Received duplicate results for '%s' : %s")) % item->GetFileName() % item->GetFileHash().Encode());
 			// Add the child, possibly updating the parents filename.
 			item->AddChild(toadd);
 			Notify_Search_Update_Sources(item);
@@ -567,7 +572,7 @@ bool CSearchList::AddToList(CSearchFile* toadd, bool clientResponse)
 		}
 	}
 
-	AddDebugLogLineM(false, logSearch,
+	AddDebugLogLineN(logSearch,
 		CFormat(wxT("Added new result '%s' : %s")) 
 			% toadd->GetFileName() % toadd->GetFileHash().Encode());
 	
@@ -609,20 +614,28 @@ void CSearchList::AddFileToDownloadByHash(const CMD4Hash& hash, uint8 cat)
 }
 
 
-void CSearchList::StopGlobalSearch()
+void CSearchList::StopSearch(bool globalOnly)
 {
-	m_searchTimer.Stop();
+	if (m_searchType == GlobalSearch) {
+		m_currentSearch = -1;
+		delete m_searchPacket;
+		m_searchPacket = NULL;
+		m_searchInProgress = false;
 
-	m_currentSearch = -1;
-	delete m_searchPacket;
-	m_searchPacket = NULL;
-	m_searchInProgress = false;
-	
-	CoreNotify_Search_Update_Progress(0xffff);
+		// Order is crucial here: on wx_MSW an additional event can be generated during the stop.
+		// So the packet has to be deleted first, so that OnGlobalSearchTimer() returns immediately
+		// without calling StopGlobalSearch() again.
+		m_searchTimer.Stop();
+
+		CoreNotify_Search_Update_Progress(0xffff);
+	} else if (m_searchType == KadSearch && !globalOnly) {
+		Kademlia::CSearchManager::StopSearch(m_currentSearch, false);
+		m_currentSearch = -1;
+	}
 }
 
 
-CSearchList::CMemFilePtr CSearchList::CreateSearchData(const CSearchParams& params, SearchType type, bool supports64bit, bool& packetUsing64bit)
+CSearchList::CMemFilePtr CSearchList::CreateSearchData(const CSearchParams& params, SearchType WXUNUSED(type), bool supports64bit, bool& packetUsing64bit)
 {
 	// Count the number of used parameters
 	unsigned int parametercount = 0;
@@ -654,35 +667,36 @@ CSearchList::CMemFilePtr CSearchList::CreateSearchData(const CSearchParams& para
 	LexFree();
 	
 #ifdef __DEBUG__
-	printf("Search parsing result for \"%s\": %i\n",
-		(const char*)unicode2UTF8(params.searchString),iParseResult);
+	AddLogLineNS(CFormat(wxT("Search parsing result for \"%s\": %i"))
+		% params.searchString % iParseResult);
 #endif
-	if (_astrParserErrors.Count() > 0) {
-		for (unsigned int i=0; i < _astrParserErrors.Count(); ++i) {
-			printf("Error %u: %s\n",i,(const char*)unicode2UTF8(_astrParserErrors[i]));
+	if (_astrParserErrors.GetCount() > 0) {
+		for (unsigned int i=0; i < _astrParserErrors.GetCount(); ++i) {
+			AddLogLineNS(CFormat(wxT("Error %u: %s\n")) % i % _astrParserErrors[i]);
 		}
 		
 		return CMemFilePtr(NULL);
 	}
 
 	if (iParseResult != 0) {
-		_astrParserErrors.Add(wxString::Format(wxT("Undefined error %i on search expression"),iParseResult));
+		_astrParserErrors.Add(CFormat(wxT("Undefined error %i on search expression")) % iParseResult);
 	
 		return CMemFilePtr(NULL);
 	}
 	
 	#ifdef __DEBUG__
-	printf("Search expression: ");
-	for (unsigned int i = 0; i < _SearchExpr.m_aExpr.Count(); i++){
-		printf("%s ",(const char*)unicode2char(_SearchExpr.m_aExpr[i]));
+	wxString mes(wxT("Search expression:"));
+	for (unsigned int i = 0; i < _SearchExpr.m_aExpr.GetCount(); i++) {
+		mes << wxT(" ") << _SearchExpr.m_aExpr[i];
 	}
-	printf("\nExpression count: %i\n",(int)_SearchExpr.m_aExpr.GetCount());
+	AddLogLineNS(mes);
+	AddLogLineNS(CFormat(wxT("Expression count: %i")) % _SearchExpr.m_aExpr.GetCount());
 	#endif
 
 	parametercount += _SearchExpr.m_aExpr.GetCount();
 	
 	#ifdef __DEBUG__
-	printf("Parameters: %i\n",parametercount);
+	AddLogLineNS(CFormat(wxT("Parameters: %i")) % parametercount);
 	#endif
 	
 	/* Leave the unicode comment there, please... */
@@ -1000,4 +1014,20 @@ void CSearchList::KademliaSearchKeyword(uint32_t searchID, const Kademlia::CUInt
 	AddToList(tempFile);
 }
 
+void CSearchList::UpdateSearchFileByHash(const CMD4Hash& hash)
+{
+	for (ResultMap::iterator it = m_results.begin(); it != m_results.end(); it++) {
+		CSearchResultList& results = it->second;
+		for (size_t i = 0; i < results.size(); ++i) {
+			CSearchFile* item = results.at(i);
+			
+			if (hash == item->GetFileHash()) {
+				// This covers only parent items,
+				// child items have to be updated separately.
+				Notify_Search_Update_Sources(item);
+			}
+		}
+	}
+}
+
 // File_checked_for_headers
