@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -27,17 +27,15 @@
 
 #include <common/MenuIDs.h>
 
-#include "amule.h"		// Needed for theApp
-#include "DownloadQueue.h"	// Needed for CDownloadQueue
+#include "amule.h"			// Needed for theApp
 #include "KnownFileList.h"	// Needed for CKnownFileList
-#include "PartFile.h"		// Needed for CPartFile and CKnownFile
 #include "SearchList.h"		// Needed for CSearchFile
 #include "SearchDlg.h"		// Needed for CSearchDlg
 #include "amuleDlg.h"		// Needed for CamuleDlg
 #include "muuli_wdr.h"		// Needed for clientImages
 #include "Preferences.h"	// Needed for thePrefs
 #include "GuiEvents.h"		// Needed for CoreNotify_Search_Add_Download
-
+#include "MuleColour.h"
 
 BEGIN_EVENT_TABLE(CSearchListCtrl, CMuleListCtrl)
 	EVT_LIST_ITEM_RIGHT_CLICK(-1, CSearchListCtrl::OnRightClick)
@@ -63,7 +61,9 @@ enum SearchListColumns {
 	ID_SEARCH_COL_SIZE,
 	ID_SEARCH_COL_SOURCES,
 	ID_SEARCH_COL_TYPE,
-	ID_SEARCH_COL_FILEID
+	ID_SEARCH_COL_FILEID,
+	ID_SEARCH_COL_STATUS,
+	ID_SEARCH_COL_DIRECTORY
 };
 
 
@@ -84,11 +84,13 @@ m_filterEnabled(false)
 	// Setting the sorter function.
 	SetSortFunc( SortProc );
 
-	InsertColumn( ID_SEARCH_COL_NAME,    _("File Name"), wxLIST_FORMAT_LEFT, 500);
-	InsertColumn( ID_SEARCH_COL_SIZE,    _("Size"),      wxLIST_FORMAT_LEFT, 100);
-	InsertColumn( ID_SEARCH_COL_SOURCES, _("Sources"),   wxLIST_FORMAT_LEFT, 50);
-	InsertColumn( ID_SEARCH_COL_TYPE,    _("Type"),      wxLIST_FORMAT_LEFT, 65);
-	InsertColumn( ID_SEARCH_COL_FILEID,  _("FileID"),    wxLIST_FORMAT_LEFT, 280);
+	InsertColumn( ID_SEARCH_COL_NAME,    _("File Name"), wxLIST_FORMAT_LEFT, 500, wxT("N") );
+	InsertColumn( ID_SEARCH_COL_SIZE,    _("Size"),      wxLIST_FORMAT_LEFT, 100, wxT("Z") );
+	InsertColumn( ID_SEARCH_COL_SOURCES, _("Sources"),   wxLIST_FORMAT_LEFT,  50, wxT("u") );
+	InsertColumn( ID_SEARCH_COL_TYPE,    _("Type"),      wxLIST_FORMAT_LEFT,  65, wxT("Y") );
+	InsertColumn( ID_SEARCH_COL_FILEID,  _("FileID"),    wxLIST_FORMAT_LEFT, 280, wxT("I") );
+	InsertColumn( ID_SEARCH_COL_STATUS,  _("Status"),    wxLIST_FORMAT_LEFT, 100, wxT("S") );
+	InsertColumn( ID_SEARCH_COL_DIRECTORY,  _("Directories"),    wxLIST_FORMAT_LEFT, 280, wxT("D") );  // I would have prefered "Directory" but this is already translated
 
 	m_nResultsID = 0;
 
@@ -111,6 +113,12 @@ m_filterEnabled(false)
 }
 
 
+wxString CSearchListCtrl::GetOldColumnOrder() const
+{
+	return wxT("N,Z,u,Y,I,S");
+}
+
+
 CSearchListCtrl::~CSearchListCtrl()
 {
 	std::list<CSearchListCtrl*>::iterator it = std::find( s_lists.begin(), s_lists.end(), this );
@@ -131,11 +139,12 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	wxCHECK_RET(toshow->GetSearchID() == m_nResultsID, wxT("Wrong search-id for result-list"));
 
 	const wxUIntPtr toshowdata = reinterpret_cast<wxUIntPtr>(toshow);
+	CSearchFile* parent = toshow->GetParent();
 
 	// Check if the result should be shown
 	if (FindItem(-1, toshowdata) != -1) {
 		return;
-	} else if (toshow->GetParent() && !toshow->GetParent()->ShowChildren()) {
+	} else if (parent && !parent->ShowChildren()) {
 		return;
 	} else if (!IsFiltered(toshow)) {
 		if (toshow->HasChildren() && toshow->ShowChildren()) {
@@ -161,13 +170,23 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	}
 
 	// Insert the item before the item found by the search
-	uint32 newid = InsertItem(GetInsertPos(toshowdata), toshow->GetFileName().GetPrintable());
+	long insertPos;
+	if (parent) {
+		insertPos = FindItem(-1, (wxUIntPtr)parent);
+		if (insertPos == -1) {
+			wxFAIL;
+			insertPos = GetItemCount();
+		} else {
+			insertPos++;
+		}
+	} else {
+		insertPos = GetInsertPos(toshowdata);
+	}
+	long newid = InsertItem(insertPos, toshow->GetFileName().GetPrintable());
 
 	// Sanity checks to ensure that results/children are properly positioned.
 #ifdef __WXDEBUG__
 	{
-		CSearchFile* parent = toshow->GetParent();
-
 		if (newid > 0) {
 			CSearchFile* before = (CSearchFile*)GetItemData(newid - 1);
 			wxASSERT(before);
@@ -196,18 +215,21 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	SetItem(newid, ID_SEARCH_COL_SIZE, CastItoXBytes( toshow->GetFileSize() ) );
 
 	// Source count
-       wxString temp = wxString::Format(wxT("%d"), toshow->GetSourceCount());
-       if (toshow->GetCompleteSourceCount()) {
-               temp += wxString::Format(wxT(" (%d)"), toshow->GetCompleteSourceCount());
-       }
-       if (toshow->GetClientsCount()) {
-               temp += wxString::Format(wxT(" [%d]"), toshow->GetClientsCount());
-       }
-#ifdef __DEBUG__
+	wxString temp = CFormat(wxT("%d")) % toshow->GetSourceCount();
+	if (toshow->GetCompleteSourceCount()) {
+		temp += CFormat(wxT(" (%d)")) % toshow->GetCompleteSourceCount();
+	}
+	if (toshow->GetClientsCount()) {
+		temp += CFormat(wxT(" [%d]")) % toshow->GetClientsCount();
+	}
+#if defined(__DEBUG__) && !defined(CLIENT_GUI)
 	if (toshow->GetKadPublishInfo() == 0) {
 		temp += wxT(" | -");
 	} else {
-		temp += wxString::Format(wxT(" | N:%u, P:%u, T:%0.2f"), (toshow->GetKadPublishInfo() & 0xFF000000) >> 24, (toshow->GetKadPublishInfo() & 0x00FF0000) >> 16, (toshow->GetKadPublishInfo() & 0x0000FFFF) / 100.0);
+		temp += CFormat(wxT(" | N:%u, P:%u, T:%0.2f"))
+			% ((toshow->GetKadPublishInfo() & 0xFF000000) >> 24)
+			% ((toshow->GetKadPublishInfo() & 0x00FF0000) >> 16)
+			% ((toshow->GetKadPublishInfo() & 0x0000FFFF) / 100.0);
 	}
 #endif
 	SetItem( newid, ID_SEARCH_COL_SOURCES, temp );
@@ -218,6 +240,12 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	// File-hash
 	SetItem(newid, ID_SEARCH_COL_FILEID, toshow->GetFileHash().Encode() );
 
+	// File status
+	SetItem(newid, ID_SEARCH_COL_STATUS, DetermineStatusPrintable(toshow));
+
+	// Directory where file is located (has a value when search file comes from a "view shared files" request)
+	SetItem(newid, ID_SEARCH_COL_DIRECTORY, toshow->GetDirectory());
+
 	// Set the color of the item
 	UpdateItemColor( newid );
 }
@@ -246,22 +274,27 @@ void CSearchListCtrl::UpdateResult(CSearchFile* toupdate)
 		// Update the filename, which may be changed in case of multiple variants.
 		SetItem(index, ID_SEARCH_COL_NAME, toupdate->GetFileName().GetPrintable());
 
-               wxString temp = wxString::Format(wxT("%d"), toupdate->GetSourceCount());
-               if (toupdate->GetCompleteSourceCount()) {
-                       temp += wxString::Format(wxT(" (%d)"), toupdate->GetCompleteSourceCount());
-               }
-               if (toupdate->GetClientsCount()) {
-                       temp += wxString::Format(wxT(" [%d]"), toupdate->GetClientsCount());
-               }
-#ifdef __DEBUG__
+		wxString temp = CFormat(wxT("%d")) % toupdate->GetSourceCount();
+		if (toupdate->GetCompleteSourceCount()) {
+			temp += CFormat(wxT(" (%d)")) % toupdate->GetCompleteSourceCount();
+		}
+		if (toupdate->GetClientsCount()) {
+			temp += CFormat(wxT(" [%d]")) % toupdate->GetClientsCount();
+		}
+#if defined(__DEBUG__) && !defined(CLIENT_GUI)
 		if (toupdate->GetKadPublishInfo() == 0) {
 			temp += wxT(" | -");
 		} else {
-			temp += wxString::Format(wxT(" | N:%u, P:%u, T:%0.2f"), (toupdate->GetKadPublishInfo() & 0xFF000000) >> 24, (toupdate->GetKadPublishInfo() & 0x00FF0000) >> 16, (toupdate->GetKadPublishInfo() & 0x0000FFFF) / 100.0);
+			temp += CFormat(wxT(" | N:%u, P:%u, T:%0.2f"))
+				% ((toupdate->GetKadPublishInfo() & 0xFF000000) >> 24)
+				% ((toupdate->GetKadPublishInfo() & 0x00FF0000) >> 16)
+				% ((toupdate->GetKadPublishInfo() & 0x0000FFFF) / 100.0);
 		}
 #endif
 		SetItem(index, ID_SEARCH_COL_SOURCES, temp);
 
+		SetItem(index, ID_SEARCH_COL_STATUS, DetermineStatusPrintable(toupdate));
+
 		UpdateItemColor(index);
 
 		// Deletions of items causes rather large amount of flicker, so to
@@ -273,40 +306,46 @@ void CSearchListCtrl::UpdateResult(CSearchFile* toupdate)
 }
 
 
-void CSearchListCtrl::UpdateItemColor( long index )
+void CSearchListCtrl::UpdateItemColor(long index)
 {
 	wxListItem item;
 	item.SetId( index );
 	item.SetColumn( ID_SEARCH_COL_SIZE );
-	item.SetMask( wxLIST_MASK_STATE|wxLIST_MASK_TEXT|wxLIST_MASK_IMAGE|wxLIST_MASK_DATA|wxLIST_MASK_WIDTH|wxLIST_MASK_FORMAT );
+	item.SetMask(
+		wxLIST_MASK_STATE |
+		wxLIST_MASK_TEXT |
+		wxLIST_MASK_IMAGE |
+		wxLIST_MASK_DATA |
+		wxLIST_MASK_WIDTH |
+		wxLIST_MASK_FORMAT);
 
-	if ( GetItem(item) ) {
-		wxColour newcol = SYSCOLOR(wxSYS_COLOUR_WINDOWTEXT);
+	if (GetItem(item)) {
+		CMuleColour newcol(wxSYS_COLOUR_WINDOWTEXT);
 
 		CSearchFile* file = (CSearchFile*)GetItemData(index);
-		CKnownFile* sameFile = theApp->downloadqueue->GetFileByID(file->GetFileHash());
-		if ( !sameFile ) {
-			sameFile = theApp->knownfiles->FindKnownFileByID(file->GetFileHash());
-		}
 
 		int red		= newcol.Red();
 		int green	= newcol.Green();
 		int blue	= newcol.Blue();
 
-		if ( sameFile ) {
-			if ( sameFile->IsPartFile() ) {
-				// File is already being downloaded. Mark as red.
-				red = 255;
-			} else if ( sameFile->GetStatus() == PS_COMPLETE ) {
-				// File has already been downloaded. Mark as green.
-				green = 200;
-			} else {
-				// File has been cancelled or removed. Mark as grey.
-				red = 128;
-				green = 128;
-				blue = 128;
-			}
-		} else {
+		switch (file->GetDownloadStatus()) {
+		case CSearchFile::DOWNLOADED:
+			// File has already been downloaded. Mark as green.
+			green = 255;
+			break;
+		case CSearchFile::QUEUED:
+			// File is downloading.
+		case CSearchFile::QUEUEDCANCELED:
+			// File is downloading and has been canceled before.
+			// Mark as red
+			red = 255;
+			break;
+		case CSearchFile::CANCELED:
+			// File has been canceled. Mark as magenta.
+			red = 255;
+			blue = 255;
+			break;
+		default:
 			// File is new, colour after number of files
 			blue += file->GetSourceCount() * 5;
 			if ( blue > 255 ) {
@@ -323,26 +362,6 @@ void CSearchListCtrl::UpdateItemColor( long index )
 }
 
 
-// Update the colors of all assosiated items, which means parents and/or siblings.
-void CSearchListCtrl::UpdateAllRelativesColor(
-	CSearchFile *file,
-	long index)
-{
-	if ((file->ShowChildren() && file->HasChildren()) ||
-	    file->GetParent()) {
-		CSearchFile *parent = file->GetParent() ?
-			file->GetParent() : file;
-		const CSearchResultList &list = parent->GetChildren();
-		for (size_t j = 0; j < list.size(); ++j) {
-			UpdateItemColor(FindItem(-1, reinterpret_cast<wxUIntPtr>(list.at(j))));
-		}
-		UpdateItemColor(FindItem(-1, reinterpret_cast<wxUIntPtr>(parent)));
-	} else {		
-		UpdateItemColor(index);
-	}
-}
-
-
 void CSearchListCtrl::ShowResults( long ResultsID )
 {
 	DeleteAllItems();
@@ -438,13 +457,8 @@ bool CSearchListCtrl::IsFiltered(const CSearchFile* file)
 	if (m_filterEnabled && m_filter.IsValid()) {
 		result = m_filter.Matches(file->GetFileName().GetPrintable());
 		result = ((result && !m_invert) || (!result && m_invert));
-	
 		if (result && m_filterKnown) {
-			result = !theApp->downloadqueue->GetFileByID(file->GetFileHash());
-
-			if (result) {
-				result = !theApp->knownfiles->FindKnownFileByID(file->GetFileHash());
-			}
+			result = file->GetDownloadStatus() == CSearchFile::NEW;
 		}
 	}
 
@@ -530,12 +544,44 @@ int CSearchListCtrl::SortProc(wxUIntPtr item1, wxUIntPtr item2, long sortData)
 		// Sort by file-hash
 		case ID_SEARCH_COL_FILEID:
 			result = CmpAny(file2->GetFileHash(), file1->GetFileHash());
+			break;
+		
+		// Sort by file status
+		case ID_SEARCH_COL_STATUS:
+			result = CmpAny(DetermineStatusPrintable(file2), DetermineStatusPrintable(file1));
+			break;
+
+		// Sort by directory
+		case ID_SEARCH_COL_DIRECTORY:
+			result = CmpAny(file1->GetDirectory(), file2->GetDirectory());
+			if (result == 0) {	// if equal sort by name
+				result = CmpAny(file1->GetFileName(), file2->GetFileName());
+			}
+			break;
 	}
 
 	return modifier * result;
 }
 
 
+void CSearchListCtrl::SetSorting(unsigned column, unsigned order)
+{
+	Freeze();
+	// First collapse all parent items
+	// Backward order means our index won't be influenced by items getting collapsed.
+	for (int i = GetItemCount(); i--;) {
+		CSearchFile* file = ((CSearchFile*)GetItemData(i));
+		if (file->ShowChildren()) {
+			ShowChildren(file, false);
+		}
+	}
+
+	// Then do the sorting
+	CMuleListCtrl::SetSorting(column, order);
+	Thaw();
+}
+
+
 void CSearchListCtrl::SyncLists( CSearchListCtrl* src, CSearchListCtrl* dst )
 {
 	wxCHECK_RET(src && dst, wxT("NULL argument in SyncLists"));
@@ -557,12 +603,12 @@ void CSearchListCtrl::SyncLists( CSearchListCtrl* src, CSearchListCtrl* dst )
 }
 
 
-void CSearchListCtrl::SyncOtherLists( CSearchListCtrl* src )
+void CSearchListCtrl::SyncOtherLists(CSearchListCtrl *src)
 {
 	std::list<CSearchListCtrl*>::iterator it;
 
-	for ( it = s_lists.begin(); it != s_lists.end(); ++it ) {
-		if ( (*it) != src ) {
+	for (it = s_lists.begin(); it != s_lists.end(); ++it) {
+		if ((*it) != src) {
 			SyncLists( src, *it );
 		}
 	}
@@ -587,10 +633,13 @@ void CSearchListCtrl::OnRightClick(wxListEvent& event)
 		
 		menu.Append(MP_MENU_CATS, _("Download in category"), cats);
 		menu.AppendSeparator();
-/* Commented out while it's gone
-		menu.Append(MP_RAZORSTATS, _("Get Razorback 2's stats for this file"));
-		menu.AppendSeparator();
-*/
+
+		const wxString & statsServer = thePrefs::GetStatsServerName();
+		if (!statsServer.IsEmpty()) {
+			menu.Append(MP_RAZORSTATS, CFormat(_("Get %s for this file")) % statsServer);
+			menu.AppendSeparator();
+		}
+
 		menu.Append(MP_SEARCHRELATED, _("Search related files (eD2k, local server)"));
 		menu.AppendSeparator();
 
@@ -635,7 +684,7 @@ void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& WXUNUSED(event) )
 	
 	long index = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
 	
-	while( index != -1 ) {
+	while (index != -1) {
 		CSearchFile* file = (CSearchFile*)GetItemData( index );
 
 		URIs += theApp->CreateED2kLink( file ) + wxT("\n");
@@ -643,7 +692,7 @@ void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& WXUNUSED(event) )
 		index = GetNextItem( index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
 	}
 
-	if ( !URIs.IsEmpty() ) {
+	if (!URIs.IsEmpty()) {
 		theApp->CopyTextToClipboard( URIs.RemoveLast() );
 	}
 }
@@ -652,24 +701,24 @@ void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& WXUNUSED(event) )
 void CSearchListCtrl::OnRazorStatsCheck( wxCommandEvent& WXUNUSED(event) )
 {
 	int item = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
-	if ( item == -1 ) {
+	if (item == -1) {
 		return;
 	}
 
 	CSearchFile* file = (CSearchFile*)GetItemData( item );
-	theApp->amuledlg->LaunchUrl(wxT("http://stats.razorback2.com/ed2khistory?ed2k=") + file->GetFileHash().Encode());
+	theApp->amuledlg->LaunchUrl(thePrefs::GetStatsServerURL() + file->GetFileHash().Encode());
 }
 
 
 void CSearchListCtrl::OnRelatedSearch( wxCommandEvent& WXUNUSED(event) )
 {
 	int item = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
-	if ( item == -1 ) {
+	if (item == -1) {
 		return;
 	}
 
 	CSearchFile* file = (CSearchFile*)GetItemData( item );
-	theApp->searchlist->StopGlobalSearch();
+	theApp->searchlist->StopSearch(true);
 	theApp->amuledlg->m_searchwnd->ResetControls();
 	CastByID( IDC_SEARCHNAME, theApp->amuledlg->m_searchwnd, wxTextCtrl )->
 		SetValue(wxT("related::") + file->GetFileHash().Encode());
@@ -679,16 +728,14 @@ void CSearchListCtrl::OnRelatedSearch( wxCommandEvent& WXUNUSED(event) )
 
 void CSearchListCtrl::OnMarkAsKnown( wxCommandEvent& WXUNUSED(event) )
 {
-	int index = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
-	if (index == -1) {
-		return;
+#ifndef CLIENT_GUI
+	long index = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	while (index > -1) {
+		CSearchFile *searchFile = (CSearchFile *)GetItemData(index);
+		CKnownFile *knownFile(new CKnownFile(*searchFile));
+		theApp->knownfiles->SafeAddKFile(knownFile);
+		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-
-#ifndef CLIENT_GUI	
-	CSearchFile *searchFile = (CSearchFile *)GetItemData(index);
-	CKnownFile *knownFile(new CKnownFile(*searchFile));
-	theApp->knownfiles->SafeAddKFile(knownFile);
-	UpdateAllRelativesColor(searchFile, index);
 #endif
 }
 
@@ -718,12 +765,11 @@ void CSearchListCtrl::OnItemActivated(wxListEvent& event)
 
 bool CSearchListCtrl::AltSortAllowed(unsigned column) const
 {
-	switch ( column ) {
-		case ID_SEARCH_COL_SOURCES:
-			return true;
-		
-		default:
-			return false;
+	switch (column) {
+	case ID_SEARCH_COL_SOURCES:
+		return true;
+	default:
+		return false;
 	}
 }
 
@@ -742,19 +788,20 @@ void CSearchListCtrl::DownloadSelected(int category)
 		}		
 	}
 	
+	// Process all selections
 	long index = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	while (index > -1) {
 		CSearchFile* file = (CSearchFile*)GetItemData(index);
 		CoreNotify_Search_Add_Download(file, category);
-		UpdateAllRelativesColor(file, index);
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
+	// Listcontrol gets updated by notification when download is started
 }
 
 
 const wxBrush& GetBrush(wxSystemColour index)
 {
-	return *wxTheBrushList->FindOrCreateBrush(SYSCOLOR(index));
+	return CMuleColour(index).GetBrush();
 }
 
 
@@ -767,19 +814,19 @@ void CSearchListCtrl::OnDrawItem(
 	if (highlighted) {
 		if (GetFocus()) {
 			dc->SetBackground(GetBrush(wxSYS_COLOUR_HIGHLIGHT));
-			dc->SetTextForeground(SYSCOLOR(wxSYS_COLOUR_HIGHLIGHTTEXT));
+			dc->SetTextForeground(CMuleColour(wxSYS_COLOUR_HIGHLIGHTTEXT));
 		} else {
 			dc->SetBackground(GetBrush(wxSYS_COLOUR_BTNSHADOW));
-			dc->SetTextForeground(SYSCOLOR(wxSYS_COLOUR_HIGHLIGHTTEXT));
+			dc->SetTextForeground(CMuleColour(wxSYS_COLOUR_HIGHLIGHTTEXT));
 		}
 	} else {
 		dc->SetBackground(GetBrush(wxSYS_COLOUR_LISTBOX));
-		dc->SetTextForeground(SYSCOLOR(wxSYS_COLOUR_WINDOWTEXT));
+		dc->SetTextForeground(CMuleColour(wxSYS_COLOUR_WINDOWTEXT));
 	}
 
 	// Define the border of the drawn area
-	if ( highlighted ) {
-		dc->SetPen(wxPen(BLEND(dc->GetBackground().GetColour(), 65)));
+	if (highlighted) {
+		dc->SetPen(*(wxThePenList->FindOrCreatePen(CMuleColour(dc->GetBackground().GetColour()).Blend(65), 1, wxSOLID)));
 	} else {
 		dc->SetPen(*wxTRANSPARENT_PEN);
 		dc->SetTextForeground(GetItemTextColour(item));
@@ -857,7 +904,7 @@ void CSearchListCtrl::OnDrawItem(
 	}
 
 	// Draw tree last so it draws over selected and focus (looks better)
-	if ( tree_show ) {
+	if (tree_show) {
 		// Gather some information
 		const bool notLast = (item + 1 < GetItemCount());
 		const bool notFirst = (item != 0);
@@ -885,7 +932,7 @@ void CSearchListCtrl::OnDrawItem(
 				// Draw empty circle
 				dc->SetBrush(*wxTRANSPARENT_BRUSH);
 			} else {
-				dc->SetBrush(GetItemTextColour(item));
+				dc->SetBrush(*(wxTheBrushList->FindOrCreateBrush(GetItemTextColour(item))));
 			}
 
 			dc->DrawCircle( treeCenter, middle, 3 );
@@ -949,4 +996,25 @@ wxString CSearchListCtrl::GetTTSText(unsigned item) const
 {
 	return GetItemText(item);
 }
+
+
+wxString CSearchListCtrl::DetermineStatusPrintable(CSearchFile *toshow)
+{
+	switch (toshow->GetDownloadStatus()) {
+	case CSearchFile::DOWNLOADED:
+		// File has already been downloaded.
+		return _("Downloaded");
+	case CSearchFile::QUEUED:
+		// File is downloading.
+	case CSearchFile::QUEUEDCANCELED:
+		// File is downloading and has been canceled before.
+		return _("Queued");
+	case CSearchFile::CANCELED:
+		// File has been canceled.
+		return _("Canceled");
+	default:
+		// File is new.
+		return _("New");
+	}
+}
 // File_checked_for_headers
