@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -26,6 +26,7 @@
 #ifndef SHAREDFILELIST_H
 #define SHAREDFILELIST_H
 
+#include <list>
 #include <map>
 #include <wx/thread.h>		// Needed for wxMutex
 
@@ -38,13 +39,15 @@ class CKnownFile;
 class CMemFile;
 class CMD4Hash;
 class CServer;
-class CUpDownClient;
 class CPublishKeywordList;
 class CPath;
 class CAICHHash;
+class CThreadTask;
 
 
 typedef std::map<CMD4Hash,CKnownFile*> CKnownFileMap;
+typedef std::map<wxString, CPath> StringPathMap;
+typedef std::list<CPath> PathList;
 
 class CSharedFileList {
 public:
@@ -56,18 +59,34 @@ public:
 	CKnownFile*	GetFileByID(const CMD4Hash& filehash);
 	short	GetFilePriorityByID(const CMD4Hash& filehash);
 	const CKnownFile* GetFileByIndex(unsigned int index) const;
-	void	CreateOfferedFilePacket(CKnownFile* cur_file, CMemFile* files, CServer* pServer, CUpDownClient* pClient);
 	size_t	GetCount()	{ wxMutexLocker lock(list_mut); return m_Files_map.size(); }
 	size_t  GetFileCount()	{ wxMutexLocker lock(list_mut); return m_Files_map.size(); }
-	void	CopyFileList(std::vector<CKnownFile*>& out_list);
+	void	CopyFileList(std::vector<CKnownFile*>& out_list) const;
 	void	UpdateItem(CKnownFile* toupdate);
-	unsigned	AddFilesFromDirectory(const CPath& directory);
 	void    GetSharedFilesByDirectory(const wxString& directory, CKnownFilePtrList& list);
 	void	ClearED2KPublishInfo();
 	void	RepublishFile(CKnownFile* pFile);
 	void	Process();
 	void	PublishNextTurn()	{ m_lastPublishED2KFlag = true; }
 	bool	RenameFile(CKnownFile* pFile, const CPath& newName);
+
+	/**
+	 * Returns the name of a folder visible to the public.
+	 *
+	 * @param dir The full path to a shared directory.
+	 * @return The name of the shared directory that will be visible to the public.
+	 *
+	 * This function is used to hide sensitive data considering the directory structure of the client.
+	 * The returned public name consists of only subdirectories that are shared.
+	 * Example: /ed2k/shared/games/tetris -> "games/tetris" if /ed2k/shared are not marked as shared
+	 */
+	wxString		GetPublicSharedDirName(const CPath& dir);
+	const CPath*	GetDirForPublicSharedDirName(const wxString& strSharedDir) const;
+	
+	/**
+	 * Returns true, if the specified path points to a shared directory or single shared file.
+	 */
+	bool			IsShared(const CPath& path) const;
 	
 	/* Kad Stuff */
 	void	Publish();
@@ -83,7 +102,10 @@ public:
 	void CheckAICHHashes(const std::list<CAICHHash>& hashes);
 	
 private:
+	typedef std::list<CThreadTask *> TaskList;
+
 	bool	AddFile(CKnownFile* pFile);
+	unsigned	AddFilesFromDirectory(const CPath& directory, TaskList & hashTasks);
 	void	FindSharedFiles();
 	bool	reloading;
 	
@@ -96,7 +118,8 @@ private:
 	CKnownFileMap		m_Files_map;
 	mutable wxMutex		list_mut;
 
-	
+	StringPathMap m_PublicSharedDirNames;  //! used for mapping strings to shared directories
+
 	/* Kad Stuff */
 	CPublishKeywordList* m_keywords;
 	unsigned int m_currFileSrc;
