@@ -1,9 +1,9 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
-// Copyright (C) 2005-2006Dvai Tams ( gonosztopi@amule.org )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2005-2011 Dévai Tamás ( gonosztopi@amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -18,37 +18,39 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #include "Statistics.h"		// Interface declarations
-#include <ec/ECCodes.h>		// Needed for EC tagnames
-#include <ec/ECPacket.h>		// Needed for CECTag
-#include "OPCodes.h"
 
-#include <wx/intl.h>	// Needed for i18n
+#include <protocol/ed2k/ClientSoftware.h>
 
-#ifndef EC_REMOTE
-	#include "GetTickCount.h"	// Needed for GetTickCount64()
+#include <ec/cpp/ECTag.h>		// Needed for CECTag
+
+#ifndef CLIENT_GUI
 	#ifndef AMULE_DAEMON
 		#include <common/Format.h>		// Needed for CFormat
-		#include "OtherFunctions.h"	// Needed for CastItoSpeed()
 	#endif
+	#include "CFile.h"		// Needed for CFile access
+	#include <common/Path.h>	// Needed for JoinPaths
+	#include <wx/config.h>		// Needed for wxConfig
 	#include "DataToText.h"		// Needed for GetSoftName()
 	#include "Preferences.h"	// Needed for thePrefs
-	#include "amule.h"		// Needed for theApp
 	#include "ListenSocket.h"	// (tree, GetAverageConnections)
 	#include "ServerList.h"		// Needed for CServerList (tree)
 	#include <cmath>		// Needed for std::floor
 	#include "updownclient.h"	// Needed for CUpDownClient
 #else
+	#include "GetTickCount.h"	// Needed for GetTickCount64()
 	#include "Preferences.h"
-	#include <ec/RemoteConnect.h>		// Needed for CRemoteConnect
+	#include <ec/cpp/RemoteConnect.h>		// Needed for CRemoteConnect
 #endif
 
+#include "amule.h"		// Needed for theApp
+#include <wx/intl.h>
 
 #ifdef __BSD__
 	// glibc -> bsd libc
@@ -58,7 +60,7 @@
 #endif /* __BSD__ */
 
 
-#ifndef EC_REMOTE
+#ifndef CLIENT_GUI
 
 /*----- CPreciseRateCounter -----*/
 
@@ -121,7 +123,7 @@ void CStatTreeItemRateCounter::AddECValues(CECTag* tag) const
 #ifndef AMULE_DAEMON
 wxString CStatTreeItemPeakConnections::GetDisplayString() const
 {
-	return wxString::Format(wxGetTranslation(m_label), theStats::GetPeakConnections());
+	return CFormat(wxGetTranslation(m_label)) % theStats::GetPeakConnections();
 }
 #endif
 
@@ -141,17 +143,17 @@ CPreciseRateCounter*		CStatistics::s_downOverheadRate;
 CStatTreeItemRateCounter*	CStatistics::s_uploadrate;
 CStatTreeItemRateCounter*	CStatistics::s_downloadrate;
 
-#else /* EC_REMOTE */
+#else /* CLIENT_GUI */
 
 uint64	CStatistics::s_start_time;
 uint64	CStatistics::s_statData[sdTotalItems];
 
-#endif /* !EC_REMOTE / EC_REMOTE */
+#endif /* !CLIENT_GUI / CLIENT_GUI */
 
 // Tree root
 CStatTreeItemBase*		CStatistics::s_statTree;
 
-#ifndef EC_REMOTE
+#ifndef CLIENT_GUI
 // Uptime
 CStatTreeItemTimer*		CStatistics::s_uptime;
 
@@ -162,6 +164,7 @@ CStatTreeItemPackets*		CStatistics::s_fileReqUpOverhead;
 CStatTreeItemPackets*		CStatistics::s_sourceXchgUpOverhead;
 CStatTreeItemPackets*		CStatistics::s_serverUpOverhead;
 CStatTreeItemPackets*		CStatistics::s_kadUpOverhead;
+CStatTreeItemCounter*		CStatistics::s_cryptUpOverhead;
 CStatTreeItemNativeCounter*	CStatistics::s_activeUploads;
 CStatTreeItemNativeCounter*	CStatistics::s_waitingUploads;
 CStatTreeItemCounter*		CStatistics::s_totalSuccUploads;
@@ -175,6 +178,7 @@ CStatTreeItemPackets*		CStatistics::s_fileReqDownOverhead;
 CStatTreeItemPackets*		CStatistics::s_sourceXchgDownOverhead;
 CStatTreeItemPackets*		CStatistics::s_serverDownOverhead;
 CStatTreeItemPackets*		CStatistics::s_kadDownOverhead;
+CStatTreeItemCounter*		CStatistics::s_cryptDownOverhead;
 CStatTreeItemNativeCounter*	CStatistics::s_foundSources;
 CStatTreeItemNativeCounter*	CStatistics::s_activeDownloads;
 
@@ -214,8 +218,14 @@ CStatTreeItemCounter*		CStatistics::s_numberOfShared;
 CStatTreeItemCounter*		CStatistics::s_sizeOfShare;
 
 // Kad
-uint64						CStatistics::s_kadNodesTotal;
-uint16						CStatistics::s_kadNodesCur;
+uint64_t			CStatistics::s_kadNodesTotal;
+uint16_t			CStatistics::s_kadNodesCur;
+
+// Totals
+uint64_t			CStatistics::s_totalSent;
+uint64_t			CStatistics::s_totalReceived;
+
+bool				CStatistics::s_statsNeedSave;
 
 
 CStatistics::CStatistics()
@@ -228,11 +238,11 @@ CStatistics::CStatistics()
 	// Init graphs
 
 	average_minutes = thePrefs::GetStatsAverageMinutes();
-	
+
 	HR hr = {0.0, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0};
 	hrInit = hr;
 	nHistRanges = 7;	// =ceil(log(max_update_delay)/log(2))
-	nPointsPerRange = GetPointsPerRange(); 
+	nPointsPerRange = GetPointsPerRange();
 	bitsHistClockMask = (1 << (nHistRanges-1)) - 1;
 	aposRecycle = new listPOS[nHistRanges];
 	listPOS *ppos = aposRecycle+nHistRanges-1;
@@ -241,7 +251,7 @@ CStatistics::CStatistics()
 		*ppos-- = --listHR.end();
 		for (int j=nPointsPerRange; j>1; --j)
 			listHR.push_back(hr);
-	}	
+	}
 
 	// Init rate counters outside the tree
 
@@ -252,6 +262,10 @@ CStatistics::CStatistics()
 
 	InitStatsTree();
 	s_uptime->SetStartTime(start_time);
+
+	// Load saved statistics
+	Load();
+	s_statsNeedSave = false;
 }
 
 
@@ -272,6 +286,74 @@ CStatistics::~CStatistics()
 }
 
 
+uint64_t ReadUInt64FromCfg(wxConfigBase* cfg, const wxString& key)
+{
+	wxString buffer;
+
+	cfg->Read(key, &buffer, wxT("0"));
+
+	uint64 tmp = 0;
+	for (unsigned int i = 0; i < buffer.Length(); ++i) {
+		if ((buffer[i] >= wxChar('0')) &&(buffer[i] <= wxChar('9'))) {
+			tmp = tmp * 10 + (buffer[i] - wxChar('0'));
+		} else {
+			tmp = 0;
+			break;
+		}
+	}
+
+	return tmp;
+}
+
+void CStatistics::Load()
+{
+	CFile f;
+
+	s_totalSent = 0;
+	s_totalReceived = 0;
+	if (f.Open(JoinPaths(theApp->ConfigDir, wxT("statistics.dat")))) {
+		uint8_t version = f.ReadUInt8();
+		if (version == 0) {
+			s_totalSent = f.ReadUInt64();
+			s_totalReceived = f.ReadUInt64();
+		}
+	}
+
+	// Load old values from config
+	bool cfgChanged = false;
+	wxConfigBase* cfg = wxConfigBase::Get();
+	if (cfg->HasEntry(wxT("/Statistics/TotalUploadedBytes"))) {
+		s_totalSent += ReadUInt64FromCfg(cfg, wxT("/Statistics/TotalUploadedBytes"));
+		cfg->DeleteEntry(wxT("/Statistics/TotalUploadedBytes"));
+		cfgChanged = true;
+	}
+	if (cfg->HasEntry(wxT("/Statistics/TotalDownloadedBytes"))) {
+		s_totalReceived += ReadUInt64FromCfg(cfg, wxT("/Statistics/TotalDownloadedBytes"));
+		cfg->DeleteEntry(wxT("/Statistics/TotalDownloadedBytes"));
+		cfgChanged = true;
+	}
+	if (cfgChanged) {
+		cfg->Flush();
+		s_statsNeedSave = s_totalSent > 0 || s_totalReceived > 0;
+		Save();
+	}
+}
+
+
+void CStatistics::Save()
+{
+	if (s_statsNeedSave) {
+		CFile f;
+
+		if (f.Open(JoinPaths(theApp->ConfigDir, wxT("statistics.dat")), CFile::write)) {
+			f.WriteUInt8(0);	/* version */
+			f.WriteUInt64(s_totalSent);
+			f.WriteUInt64(s_totalReceived);
+		}
+		s_statsNeedSave = false;
+	}
+}
+
 void CStatistics::CalculateRates()
 {
 	uint64_t now = GetTickCount64();
@@ -282,24 +364,25 @@ void CStatistics::CalculateRates()
 }
 
 
+
 /* ------------------------------- GRAPHS ---------------------------- */
 
 /*
 History List
 
-  The basic idea here is that we want to keep as much history as we can without paying 
-a high price in terms of memory space.  Because we keep the history for display purposes, 
-we can take advantage of the fact that when the period shown in the graphs is long 
-then each pixel represents a long period.  So as the most recent history we keep one 
-window full of points at a resolution of 1 second, the next window full at 2 seconds, 
-the next at 4 seconds and so on, up to the maximum desired.  This way there is always 
-at least one sample point per pixel for any update delay set by the user, and the 
+  The basic idea here is that we want to keep as much history as we can without paying
+a high price in terms of memory space.  Because we keep the history for display purposes,
+we can take advantage of the fact that when the period shown in the graphs is long
+then each pixel represents a long period.  So as the most recent history we keep one
+window full of points at a resolution of 1 second, the next window full at 2 seconds,
+the next at 4 seconds and so on, up to the maximum desired.  This way there is always
+at least one sample point per pixel for any update delay set by the user, and the
 memory required grows with the *log* of the total time period covered.
-  The history is kept in a doubly-linked list, with the most recent snapshot at the tail.  
-The number of nodes in the list is fixed, and there are no calls to RemoveHead() and 
-AddTail() which would add overhead and contribute to memory fragmentation.  Instead, 
-every second when a new point gets recorded, one of the existing nodes is recycled; 
-it is disjoined from its present place, put at the tail of the list, and then gets 
+  The history is kept in a doubly-linked list, with the most recent snapshot at the tail.
+The number of nodes in the list is fixed, and there are no calls to RemoveHead() and
+AddTail() which would add overhead and contribute to memory fragmentation.  Instead,
+every second when a new point gets recorded, one of the existing nodes is recycled;
+it is disjoined from its present place, put at the tail of the list, and then gets
 filled with new data.   [Emilio Sandoz]
   This unfortunately does not work with stl classes, as none of them supports moving
 a node to another place, so we have to erase and re-add nodes.
@@ -307,32 +390,32 @@ a node to another place, so we have to erase and re-add nodes.
 
 void CStatistics::RecordHistory()
 {	// First we query and compute some values, then we store them in the history list
-	
+
 	// A few comments about the use of double and float in computations:
 	// Even on a hi-res screen our graphs will have 10 bits of resolution at most,
 	// so the 24 bits resolution of a float on 32 bit Intel processors is more than
-	// enough for all displayed values.  Rate computations however, and especially 
-	// running average computations, use differences (delta bytes/ delta time), and 
-	// for long uptimes the difference between two timestamps can lose too much 
+	// enough for all displayed values.  Rate computations however, and especially
+	// running average computations, use differences (delta bytes/ delta time), and
+	// for long uptimes the difference between two timestamps can lose too much
 	// accuracy because the large mantissa causes less significant bits to be dropped
-	// (same for the difference between two cumulative byte counts).  [We don't store 
+	// (same for the difference between two cumulative byte counts).  [We don't store
 	// these values as integers because they will be used in floating point calculations,
-	// and we want to perform the conversion only once).  Therefore timestamps and 
+	// and we want to perform the conversion only once).  Therefore timestamps and
 	// Kbyte counts are stored in the history as doubles, while computed values use
 	// float (to save space and execution time).
 
 /*
 	Store values; first determine the node to be recycled (using the bits in iClock)
-	
+
     oldest records ----------------- listHR ------------------ youngest records
-	
+
 	O-[Range 2^n sec]-O- ... -O-[Range 4 sec]-O-[Range 2 sec]-O-[Range 1 sec]-O
 	|                 |       |               |               > every 2 secs -^
 	|                 |  ...  |                >--------------- every 4 secs -^
 	|                 |       >------------------------ recycle every 8 secs -^
 	|                 |                                                 ...
-	|                 >-the node at this position is recycled every 2^n secs -^	
-	>-------------------(ditto for the oldest node at the head of the list) --^	
+	|                 >-the node at this position is recycled every 2^n secs -^
+	>-------------------(ditto for the oldest node at the head of the list) --^
 	^                                                         ^
     aposRecycle[nHistRanges-1]               ...              aposRecycle[0]  Tail
 */
@@ -343,7 +426,7 @@ void CStatistics::RecordHistory()
 	if (bits <= bitsHistClockMask) {
 		ppos = aposRecycle;
 		while ((bits /= 2) != 0)  // count to the highest bit that was just toggled to 1
-			++ppos;	
+			++ppos;
 		// recycle one node and jump over the next to move it to the next higher range
 		listHR.push_back(**ppos);
 		*ppos = ++listHR.erase(*ppos);
@@ -353,7 +436,7 @@ void CStatistics::RecordHistory()
 		listHR.push_back(**ppos);
 		*ppos = listHR.erase(*ppos);
 	}
-	
+
 	// now save the latest data point in this node
  	listPOS phr = --listHR.end();
 	phr->kBytesSent = GetSessionSentBytes() / 1024.0;
@@ -364,7 +447,7 @@ void CStatistics::RecordHistory()
 	phr->cntConnections = GetActiveConnections();
 	phr->cntDownloads = GetDownloadingSources();
 	phr->sTimestamp = GetUptimeMillis() / 1000.0;
-	
+
 	s_kadNodesTotal += s_kadNodesCur;
 	phr->kadNodesTotal = s_kadNodesTotal;
 	phr->kadNodesCur = s_kadNodesCur;
@@ -375,32 +458,37 @@ unsigned CStatistics::GetHistory(	// Assemble arrays of sample points for a grap
 	unsigned cntPoints,		// number of sample points to assemble
 	double sStep,			// time difference between sample points
 	double sFinal,			// latest allowed timestamp
-	float** ppf,			// an array of pointers to arrays of floats for the result
+	const std::vector<float *> &ppf,// an array of pointers to arrays of floats for the result
 	StatsGraphType which_graph)	// the graph which will receive the points
-{	
-	if (sStep==0.0 || cntPoints==0)
+{
+	if (sStep==0.0 || cntPoints==0) {
 		return(0);
-	float		*pf1 = *ppf;
-	float		*pf2 = *(ppf+1);
-	float		*pf3 = *(ppf+2);
-	unsigned	cntFilled = 0;
-	listRPOS	pos = listHR.rbegin();
+	}
 
-	// start of list should be an integer multiple of the sampling period for samples 
+	float *pf1 = ppf[0];
+	float *pf2 = ppf[1];
+	float *pf3 = ppf[2];
+	unsigned cntFilled = 0;
+	listRPOS pos = listHR.rbegin();
+
+	// start of list should be an integer multiple of the sampling period for samples
 	// to be consistent when the graphs are resized horizontally
 	double	sTarget;
-	if (sFinal >= 0.0)
+	if (sFinal >= 0.0) {
 		sTarget = sFinal;
-	else
-		sTarget = (sStep==1.0 ? pos->sTimestamp : std::floor(pos->sTimestamp/sStep) * sStep); 
+	} else {
+		sTarget = sStep==1.0 ?
+			pos->sTimestamp :
+			std::floor(pos->sTimestamp/sStep) * sStep;
+	}
 
-	HR	**ahr = NULL, **pphr = NULL;
-	bool	bRateGraph = (which_graph != GRAPH_CONN);	// rate graph or connections graph?
+	HR **ahr = NULL, **pphr = NULL;
+	bool bRateGraph = (which_graph != GRAPH_CONN);	// rate graph or connections graph?
 	if (bRateGraph) {
 		ahr = new HR* [cntPoints];
 		pphr = ahr;
 	}
-	
+
 	while (pos != listHR.rend()) {
 		if (pos->sTimestamp > sTarget) {
 			++pos;
@@ -413,23 +501,27 @@ unsigned CStatistics::GetHistory(	// Assemble arrays of sample points for a grap
 			*pf2++ = (float)pos->cntConnections;
 			*pf3++ = (float)pos->cntDownloads;
 		}
-		if (++cntFilled  == cntPoints)		// enough points 
+		if (++cntFilled  == cntPoints) {	// enough points
 			break;
-		if (pos->sTimestamp == 0.0)		// reached beginning of uptime
+		}
+		if (pos->sTimestamp == 0.0) {		// reached beginning of uptime
 			break;
+		}
 		if ((sTarget -= sStep) <= 0.0) {	// don't overshoot the beginning
-			if (bRateGraph)
+			if (bRateGraph) {
 				*pphr++ = &hrInit;
-			else
+			} else {
 				*pf1++ = *pf2++ = *pf3++ = 0.0;
+			}
 			++cntFilled;
 			break;
 		}
 	}
 
 	if (bRateGraph) {
-		if  (cntFilled > 0)
+		if  (cntFilled > 0) {
 			ComputeAverages(pphr, pos, cntFilled, sStep, ppf, which_graph);
+		}
 		delete[] ahr;
 	}
 
@@ -442,7 +534,7 @@ unsigned CStatistics::GetHistoryForWeb(  // Assemble arrays of sample points for
 	double sStep,			// time difference between sample points
 	double *sStart,			// earliest allowed timestamp
 	uint32 **graphData)		// a pointer to a pointer that will point to the graph data array
-{	
+{
 	if (*sStart < 0.0) {
 		*sStart = 0.0;
 	}
@@ -452,7 +544,7 @@ unsigned CStatistics::GetHistoryForWeb(  // Assemble arrays of sample points for
 	listRPOS	pos = listHR.rbegin();
 	double		LastTimeStamp = pos->sTimestamp;
 	double		sTarget = LastTimeStamp;
-	
+
 	HR	**pphr = new HR *[cntPoints];
 
 	while (pos != listHR.rend()) {
@@ -461,7 +553,7 @@ unsigned CStatistics::GetHistoryForWeb(  // Assemble arrays of sample points for
 			continue;
 		}
 		pphr[cntFilled] = &(*pos);
-		if (++cntFilled  == cntPoints)		// enough points 
+		if (++cntFilled  == cntPoints)		// enough points
 			break;
 		if (pos->sTimestamp <= *sStart)		// reached beginning of requested time
 			break;
@@ -500,13 +592,13 @@ unsigned CStatistics::GetHistoryForWeb(  // Assemble arrays of sample points for
 
 
 void CStatistics::ComputeAverages(
-	HR		**pphr,			// pointer to (end of) array of assembled history records
-	listRPOS	pos,			// position in history list from which to backtrack
-	unsigned	cntFilled,		// number of points in the sample data
-	double		sStep,			// time difference between two samples
-	float		**ppf,			// an array of pointers to arrays of floats with sample data
-	StatsGraphType	which_graph)		// the graph which will receive the points
-{	
+	HR		**pphr,		// pointer to (end of) array of assembled history records
+	listRPOS	pos,		// position in history list from which to backtrack
+	unsigned	cntFilled,	// number of points in the sample data
+	double		sStep,		// time difference between two samples
+	const std::vector<float *> &ppf,// an array of pointers to arrays of floats with sample data
+	StatsGraphType	which_graph)	// the graph which will receive the points
+{
 	double		sTarget, kValueRun;
 	uint64 		avgTime = average_minutes * 60;
 	unsigned	nBtPoints = (unsigned)(avgTime / sStep);
@@ -519,7 +611,7 @@ void CStatistics::ComputeAverages(
 		default:
 			wxCHECK_RET(false, wxT("ComputeAverages called with unsupported graph type."));
 	}
-	
+
 	runningAvg->m_timespan = avgTime * 1000;
 	runningAvg->m_tick_history.clear();
 	runningAvg->m_byte_history.clear();
@@ -531,21 +623,27 @@ void CStatistics::ComputeAverages(
 	} else {
 		sTarget = std::max(0.0, pos->sTimestamp - sStep);
 	}
-	
+
 	while (nBtPoints--) {
 		while (pos != listHR.rend() && pos->sTimestamp > sTarget) ++pos;	// find next history record
 		if (pos != listHR.rend()) {
 			runningAvg->m_tick_history.push_front((uint64)(pos->sTimestamp * 1000.0));
-			
+
 			uint32 value = 0;
 			switch (which_graph) {
-				case GRAPH_DOWN:	value = (uint32)(pos->kBpsDownCur * 1024.0);	break;
-				case GRAPH_UP:		value = (uint32)(pos->kBpsUpCur * 1024.0);		break;
-				case GRAPH_KAD:		value = (uint32)(pos->kadNodesCur * 1024.0);	break;
-				default:
-					wxCHECK_RET(false, wxT("ComputeAverages called with unsupported graph type."));		
-			}		
-			
+			case GRAPH_DOWN:
+				value = (uint32)(pos->kBpsDownCur * 1024.0);
+				break;
+			case GRAPH_UP:
+				value = (uint32)(pos->kBpsUpCur * 1024.0);
+				break;
+			case GRAPH_KAD:
+				value = (uint32)(pos->kadNodesCur * 1024.0);
+				break;
+			default:
+				wxCHECK_RET(false, wxT("ComputeAverages called with unsupported graph type."));
+			}
+
 			runningAvg->m_byte_history.push_front(value);
 			runningAvg->m_total += value;
 		} else {
@@ -557,9 +655,9 @@ void CStatistics::ComputeAverages(
 	};
 
 	// now compute averages in returned arrays, starting with the earliest values
-	float	*pf1 = *ppf++ + cntFilled - 1;	// holds session avg
-	float	*pf2 = *ppf++ + cntFilled - 1;	// holds running avg
-	float	*pf3 = *ppf + cntFilled - 1;	// holds current rate
+	float *pf1 = ppf[0] + cntFilled - 1;	// holds session avg
+	float *pf2 = ppf[1] + cntFilled - 1;	// holds running avg
+	float *pf3 = ppf[2] + cntFilled - 1;	// holds current rate
 
 	for (int cnt=cntFilled; cnt>0; cnt--, pf1--, pf2--, pf3--) {
 		HR *phr = *(--pphr);
@@ -573,7 +671,7 @@ void CStatistics::ComputeAverages(
 			kValueRun = phr->kadNodesTotal;
 			*pf3 = phr->kadNodesCur;
 		}
-		
+
 		*pf1 = kValueRun / phr->sTimestamp;
 		(*runningAvg) += (uint32)(*pf3 * 1024.0);
 		runningAvg->CalculateRate((uint64)(phr->sTimestamp * 1000.0));
@@ -612,7 +710,7 @@ GraphUpdateInfo CStatistics::GetPointsForUpdate()
 	update.kadnodes[0] = phr->kadNodesTotal / phr->sTimestamp;
 	update.kadnodes[1] = m_graphRunningAvgKad.GetRate() / 1024.0;
 	update.kadnodes[2] = phr->kadNodesCur;
-		
+
 	return update;
 }
 
@@ -631,7 +729,7 @@ void CStatistics::InitStatsTree()
 	tmpRoot1 = s_statTree->AddChild(new CStatTreeItemBase(wxTRANSLATE("Transfer"), stSortChildren));
 
 	tmpRoot2 = tmpRoot1->AddChild(new CStatTreeItemBase(wxTRANSLATE("Uploads")), 2);
-	s_sessionUpload = (CStatTreeItemUlDlCounter*)tmpRoot2->AddChild(new CStatTreeItemUlDlCounter(wxTRANSLATE("Uploaded Data (Session (Total)): %s"), thePrefs::GetTotalUploaded, stSortChildren | stSortByValue));
+	s_sessionUpload = (CStatTreeItemUlDlCounter*)tmpRoot2->AddChild(new CStatTreeItemUlDlCounter(wxTRANSLATE("Uploaded Data (Session (Total)): %s"), theStats::GetTotalSentBytes, stSortChildren | stSortByValue));
 	// Children will be added on-the-fly
 	s_totalUpOverhead = (CStatTreeItemPacketTotals*)tmpRoot2->AddChild(new CStatTreeItemPacketTotals(wxTRANSLATE("Total Overhead (Packets): %s")));
 	s_fileReqUpOverhead = (CStatTreeItemPackets*)tmpRoot2->AddChild(new CStatTreeItemPackets(wxTRANSLATE("File Request Overhead (Packets): %s")));
@@ -642,6 +740,8 @@ void CStatistics::InitStatsTree()
 	s_totalUpOverhead->AddPacketCounter(s_serverUpOverhead);
 	s_kadUpOverhead = (CStatTreeItemPackets*)tmpRoot2->AddChild(new CStatTreeItemPackets(wxTRANSLATE("Kad Overhead (Packets): %s")));
 	s_totalUpOverhead->AddPacketCounter(s_kadUpOverhead);
+	s_cryptUpOverhead = (CStatTreeItemCounter*)tmpRoot2->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Crypt overhead (UDP): %s")));
+	s_cryptUpOverhead->SetDisplayMode(dmBytes);
 	s_activeUploads = (CStatTreeItemNativeCounter*)tmpRoot2->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Active Uploads: %s")));
 	s_waitingUploads = (CStatTreeItemNativeCounter*)tmpRoot2->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Waiting Uploads: %s")));
 	s_totalSuccUploads = (CStatTreeItemCounter*)tmpRoot2->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Total successful upload sessions: %s")));
@@ -650,7 +750,7 @@ void CStatistics::InitStatsTree()
 	tmpRoot2->AddChild(new CStatTreeItemAverage(wxTRANSLATE("Average upload time: %s"), s_totalUploadTime, s_totalSuccUploads, dmTime));
 
 	tmpRoot2 = tmpRoot1->AddChild(new CStatTreeItemBase(wxTRANSLATE("Downloads")), 1);
-	s_sessionDownload = (CStatTreeItemUlDlCounter*)tmpRoot2->AddChild(new CStatTreeItemUlDlCounter(wxTRANSLATE("Downloaded Data (Session (Total)): %s"), thePrefs::GetTotalDownloaded, stSortChildren | stSortByValue));
+	s_sessionDownload = (CStatTreeItemUlDlCounter*)tmpRoot2->AddChild(new CStatTreeItemUlDlCounter(wxTRANSLATE("Downloaded Data (Session (Total)): %s"), theStats::GetTotalReceivedBytes, stSortChildren | stSortByValue));
 	// Children will be added on-the-fly
 	s_totalDownOverhead = (CStatTreeItemPacketTotals*)tmpRoot2->AddChild(new CStatTreeItemPacketTotals(wxTRANSLATE("Total Overhead (Packets): %s")));
 	s_fileReqDownOverhead = (CStatTreeItemPackets*)tmpRoot2->AddChild(new CStatTreeItemPackets(wxTRANSLATE("File Request Overhead (Packets): %s")));
@@ -661,16 +761,18 @@ void CStatistics::InitStatsTree()
 	s_totalDownOverhead->AddPacketCounter(s_serverDownOverhead);
 	s_kadDownOverhead = (CStatTreeItemPackets*)tmpRoot2->AddChild(new CStatTreeItemPackets(wxTRANSLATE("Kad Overhead (Packets): %s")));
 	s_totalDownOverhead->AddPacketCounter(s_kadDownOverhead);
+	s_cryptDownOverhead = (CStatTreeItemCounter*)tmpRoot2->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Crypt overhead (UDP): %s")));
+	s_cryptDownOverhead->SetDisplayMode(dmBytes);
 	s_foundSources = (CStatTreeItemNativeCounter*)tmpRoot2->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Found Sources: %s"), stSortChildren | stSortByValue));
 	s_activeDownloads = (CStatTreeItemNativeCounter*)tmpRoot2->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Active Downloads (chunks): %s")));
 
-	tmpRoot1->AddChild(new CStatTreeItemRatio(wxTRANSLATE("Session UL:DL Ratio (Total): %s"), s_sessionUpload, s_sessionDownload), 3);
+	tmpRoot1->AddChild(new CStatTreeItemRatio(wxTRANSLATE("Session UL:DL Ratio (Total): %s"), s_sessionUpload, s_sessionDownload, theStats::GetTotalSentBytes, theStats::GetTotalReceivedBytes), 3);
 
 	tmpRoot1 = s_statTree->AddChild(new CStatTreeItemBase(wxTRANSLATE("Connection")));
-	tmpRoot1->AddChild(new CStatTreeItemAverageSpeed(wxTRANSLATE("Average Downloadrate (Session): %s"), s_sessionDownload, s_uptime));
-	tmpRoot1->AddChild(new CStatTreeItemAverageSpeed(wxTRANSLATE("Average Uploadrate (Session): %s"), s_sessionUpload, s_uptime));
-	s_downloadrate = (CStatTreeItemRateCounter*)tmpRoot1->AddChild(new CStatTreeItemRateCounter(wxTRANSLATE("Max Downloadrate (Session): %s"), true, 30000));
-	s_uploadrate = (CStatTreeItemRateCounter*)tmpRoot1->AddChild(new CStatTreeItemRateCounter(wxTRANSLATE("Max Uploadrate (Session): %s"), true, 30000));
+	tmpRoot1->AddChild(new CStatTreeItemAverageSpeed(wxTRANSLATE("Average download rate (Session): %s"), s_sessionDownload, s_uptime));
+	tmpRoot1->AddChild(new CStatTreeItemAverageSpeed(wxTRANSLATE("Average upload rate (Session): %s"), s_sessionUpload, s_uptime));
+	s_downloadrate = (CStatTreeItemRateCounter*)tmpRoot1->AddChild(new CStatTreeItemRateCounter(wxTRANSLATE("Max download rate (Session): %s"), true, 30000));
+	s_uploadrate = (CStatTreeItemRateCounter*)tmpRoot1->AddChild(new CStatTreeItemRateCounter(wxTRANSLATE("Max upload rate (Session): %s"), true, 30000));
 	s_reconnects = (CStatTreeItemReconnects*)tmpRoot1->AddChild(new CStatTreeItemReconnects(wxTRANSLATE("Reconnects: %i")));
 	s_sinceFirstTransfer = (CStatTreeItemTimer*)tmpRoot1->AddChild(new CStatTreeItemTimer(wxTRANSLATE("Time Since First Transfer: %s"), stHideIfZero));
 	s_sinceConnected = (CStatTreeItemTimer*)tmpRoot1->AddChild(new CStatTreeItemTimer(wxTRANSLATE("Connected To Server Since: %s")));
@@ -681,14 +783,14 @@ void CStatistics::InitStatsTree()
 	tmpRoot1->AddChild(new CStatTreeItemPeakConnections(wxTRANSLATE("Peak Connections (estimate): %i")));
 
 	s_clients = (CStatTreeItemHiddenCounter*)s_statTree->AddChild(new CStatTreeItemHiddenCounter(wxTRANSLATE("Clients"), stSortChildren | stSortByValue));
-	s_unknown = (CStatTreeItemCounter*)s_clients->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Unknown") wxT(": %s")), 6);
+	s_unknown = (CStatTreeItemCounter*)s_clients->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Unknown: %s")), 6);
 	//s_lowID = (CStatTreeItem*)s_clients->AddChild(new CStatTreeItem(wxTRANSLATE("LowID: %u (%.2f%% Total %.2f%% Known)")), 5);
 	//s_secIdentOnOff = (CStatTreeItem*)s_clients->AddChild(new CStatTreeItem(wxTRANSLATE("SecIdent On/Off: %u (%.2f%%) : %u (%.2f%%)")), 4);
 #ifdef __DEBUG__
 	s_hasSocket = (CStatTreeItemNativeCounter*)s_clients->AddChild(new CStatTreeItemNativeCounter(wxT("HasSocket: %s")), 3);
 #endif
-	s_filtered = (CStatTreeItemNativeCounter*)s_clients->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Filtered") wxT(": %s")), 2);
-	s_banned = (CStatTreeItemNativeCounter*)s_clients->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Banned") wxT(": %s")), 1);
+	s_filtered = (CStatTreeItemNativeCounter*)s_clients->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Filtered: %s")), 2);
+	s_banned = (CStatTreeItemNativeCounter*)s_clients->AddChild(new CStatTreeItemNativeCounter(wxTRANSLATE("Banned: %s")), 1);
 	s_clients->AddChild(new CStatTreeItemTotalClients(wxTRANSLATE("Total: %i Known: %i"), s_clients, s_unknown), 0x80000000);
 
 	// TODO: Use counters?
@@ -709,7 +811,7 @@ void CStatistics::InitStatsTree()
 	s_numberOfShared = (CStatTreeItemCounter*)tmpRoot1->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Number of Shared Files: %s")));
 	s_sizeOfShare = (CStatTreeItemCounter*)tmpRoot1->AddChild(new CStatTreeItemCounter(wxTRANSLATE("Total size of Shared Files: %s")));
 	s_sizeOfShare->SetDisplayMode(dmBytes);
-	tmpRoot1->AddChild(new CStatTreeItemAverage(wxTRANSLATE("Average filesize: %s"), s_sizeOfShare, s_numberOfShared, dmBytes));
+	tmpRoot1->AddChild(new CStatTreeItemAverage(wxTRANSLATE("Average file size: %s"), s_sizeOfShare, s_numberOfShared, dmBytes));
 }
 
 
@@ -722,12 +824,7 @@ void CStatistics::UpdateStatsTree()
 	s_foundSources->ReSortChildren();
 	// TODO: sort OS_Info subtrees.
 
-	s_avgConnections->SetValue(theApp.listensocket->GetAverageConnections());
-
-#if 0
-	(*cli13) = wxString::Format(_("LowID: %u (%.2f%% Total %.2f%% Known)"),#lowid , (#total>0)?((double)100*#lowid/#total):0, (double)100*#knownLowID/#known);
-	(*cli14) = wxString::Format(_("SecIdent On/Off: %u (%.2f%%) : %u (%.2f%%)"), #secOn , ((#eMule+#aMule)>0)?((double)100*#secOn / (#eMule+#aMule)):0, #secOff, ((#eMule+#aMule)>0)?((double)100*#secOff /(#eMule+#aMule) ):0);
-#endif
+	s_avgConnections->SetValue(theApp->listensocket->GetAverageConnections());
 
 	// get serverstats
 	// TODO: make these realtime, too
@@ -737,7 +834,7 @@ void CStatistics::UpdateStatsTree()
 	uint32 servtuser;
 	uint32 servtfile;
 	float servocc;
-	theApp.serverlist->GetStatus(servfail, servuser, servfile, servtuser, servtfile, servocc);
+	theApp->serverlist->GetStatus(servfail, servuser, servfile, servtuser, servtfile, servocc);
 	s_workingServers->SetValue((uint64)((*s_totalServers)-servfail));
 	s_failedServers->SetValue((uint64)servfail);
 	s_usersOnWorking->SetValue((uint64)servuser);
@@ -816,7 +913,7 @@ void CStatistics::AddUploadToSoft(uint8 SoftType, uint32 bytes)
 
 inline bool SupportsOSInfo(unsigned clientSoft)
 {
-	return (clientSoft == SO_AMULE) || (clientSoft == SO_HYDRANODE);
+	return (clientSoft == SO_AMULE) || (clientSoft == SO_HYDRANODE) || (clientSoft == SO_NEW2_MLDONKEY);
 }
 
 // Do some random black magic to strings to get a relatively unique number for them.
@@ -830,7 +927,7 @@ uint32 GetIdFromString(const wxString& str)
                 id ^= old_id;
                 id -= old_id;
         }
-	return ((id >> 1) + id | 0x00000100) & 0x7fffffff;
+	return (((id >> 1) + id) | 0x00000100) & 0x7fffffff;
 }
 
 void CStatistics::AddKnownClient(CUpDownClient *pClient)
@@ -911,15 +1008,16 @@ void CStatistics::RemoveKnownClient(uint32 clientSoft, uint32 clientVersion, con
 	}
 }
 
-#else /* EC_REMOTE (CLIENT_GUI) */
+#else /* CLIENT_GUI */
 
-CStatistics::CStatistics(CRemoteConnect* conn)
-	: m_conn(conn)
+CStatistics::CStatistics(CRemoteConnect &conn)
+:
+m_conn(conn)
 {
 	s_start_time = GetTickCount64();
 
 	// Init Tree
-	s_statTree = new CStatTreeItemBase(_("Statistics"));
+	s_statTree = new CStatTreeItemBase(_("Statistics"), 0);
 
 	// Clear stat data container
 	for (int i = 0; i < sdTotalItems; ++i) {
@@ -936,30 +1034,62 @@ CStatistics::~CStatistics()
 
 void CStatistics::UpdateStats(const CECPacket* stats)
 {
-	s_statData[sdUpload] = stats->GetTagByNameSafe(EC_TAG_STATS_UL_SPEED)->GetInt32Data();
-	s_statData[sdUpOverhead] = stats->GetTagByNameSafe(EC_TAG_STATS_UP_OVERHEAD)->GetInt32Data();
-	s_statData[sdDownload] = stats->GetTagByNameSafe(EC_TAG_STATS_DL_SPEED)->GetInt32Data();
-	s_statData[sdDownOverhead] = stats->GetTagByNameSafe(EC_TAG_STATS_DOWN_OVERHEAD)->GetInt32Data();
-	s_statData[sdWaitingClients] = stats->GetTagByNameSafe(EC_TAG_STATS_UL_QUEUE_LEN)->GetInt32Data();
-	s_statData[sdBannedClients] = stats->GetTagByNameSafe(EC_TAG_STATS_BANNED_COUNT)->GetInt32Data();
-};
+	s_statData[sdUpload] = stats->GetTagByNameSafe(EC_TAG_STATS_UL_SPEED)->GetInt();
+	s_statData[sdUpOverhead] = stats->GetTagByNameSafe(EC_TAG_STATS_UP_OVERHEAD)->GetInt();
+	s_statData[sdDownload] = stats->GetTagByNameSafe(EC_TAG_STATS_DL_SPEED)->GetInt();
+	s_statData[sdDownOverhead] = stats->GetTagByNameSafe(EC_TAG_STATS_DOWN_OVERHEAD)->GetInt();
+	s_statData[sdWaitingClients] = stats->GetTagByNameSafe(EC_TAG_STATS_UL_QUEUE_LEN)->GetInt();
+	s_statData[sdBannedClients] = stats->GetTagByNameSafe(EC_TAG_STATS_BANNED_COUNT)->GetInt();
+	s_statData[sdED2KUsers] = stats->GetTagByNameSafe(EC_TAG_STATS_ED2K_USERS)->GetInt();
+	s_statData[sdKadUsers] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_USERS)->GetInt();
+	s_statData[sdED2KFiles] = stats->GetTagByNameSafe(EC_TAG_STATS_ED2K_FILES)->GetInt();
+	s_statData[sdKadFiles] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_FILES)->GetInt();
+	s_statData[sdKadFirewalledUDP] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_FIREWALLED_UDP)->GetInt();
+	s_statData[sdKadIndexedSources] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_INDEXED_SOURCES)->GetInt();
+	s_statData[sdKadIndexedKeywords] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_INDEXED_KEYWORDS)->GetInt();
+	s_statData[sdKadIndexedNotes] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_INDEXED_NOTES)->GetInt();
+	s_statData[sdKadIndexedLoad] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_INDEXED_LOAD)->GetInt();
+	s_statData[sdKadIPAdress] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_IP_ADRESS)->GetInt();
+	s_statData[sdBuddyStatus] = stats->GetTagByNameSafe(EC_TAG_STATS_BUDDY_STATUS)->GetInt();
+	s_statData[sdBuddyIP] = stats->GetTagByNameSafe(EC_TAG_STATS_BUDDY_IP)->GetInt();
+	s_statData[sdBuddyPort] = stats->GetTagByNameSafe(EC_TAG_STATS_BUDDY_PORT)->GetInt();
+	s_statData[sdKadInLanMode] = stats->GetTagByNameSafe(EC_TAG_STATS_KAD_IN_LAN_MODE)->GetInt();
+	s_statData[sdTotalSentBytes] = stats->GetTagByNameSafe(EC_TAG_STATS_TOTAL_SENT_BYTES)->GetInt();
+	s_statData[sdTotalReceivedBytes] = stats->GetTagByNameSafe(EC_TAG_STATS_TOTAL_RECEIVED_BYTES)->GetInt();
+	s_statData[sdSharedFileCount] = stats->GetTagByNameSafe(EC_TAG_STATS_SHARED_FILE_COUNT)->GetInt();
+
+	const CECTag * LoggerTag = stats->GetTagByName(EC_TAG_STATS_LOGGER_MESSAGE);
+	if (LoggerTag) {
+		for (CECTag::const_iterator it = LoggerTag->begin(); it != LoggerTag->end(); it++) {
+			theApp->AddRemoteLogLine(it->GetStringData());
+		}
+	}
+}
 
 
 void CStatistics::UpdateStatsTree()
 {
-	CECPacket request(EC_OP_GET_STATSTREE);
-	if (thePrefs::GetMaxClientVersions() != 0) {
-		request.AddTag(CECTag(EC_TAG_STATTREE_CAPPING, (uint8)thePrefs::GetMaxClientVersions()));
-	}
-	const CECPacket* reply = m_conn->SendRecvPacket(&request);
-	if (reply) {
-		const CECTag* treeRoot = reply->GetTagByName(EC_TAG_STATTREE_NODE);
-		if (treeRoot) {
-			delete s_statTree;
-			s_statTree = new CStatTreeItemBase(treeRoot);
-		}
-	}
-	delete reply;
 }
 
-#endif /* !EC_REMOTE */
+
+void CStatistics::RebuildStatTreeRemote(const CECTag * tag)
+{
+	delete s_statTree;
+	s_statTree = new CStatTreeItemBase(tag);
+}
+
+
+uint64 CStatistics::GetUptimeMillis()
+{
+	return GetTickCount64() - s_start_time;
+}
+
+
+uint64 CStatistics::GetUptimeSeconds()
+{
+	return (GetTickCount64() - s_start_time) / 1000;
+}
+
+#endif /* !CLIENT_GUI */
+
+// File_checked_for_headers
