@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -26,10 +26,15 @@
 
 #include "Tag.h"				// Interface declarations
 
+#include <common/Format.h>		// Needed for WXLONGLONGFMTSPEC
 
 #include "SafeFile.h"		// Needed for CFileDataIO
 #include "MD4Hash.h"			// Needed for CMD4Hash
 
+#ifdef __SUNPRO_CC
+#define __FUNCTION__ __FILE__+__LINE__
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 // CTag
 
@@ -73,7 +78,7 @@ CTag::CTag(const CTag& rTag)
 		m_pData = new unsigned char[rTag.GetBsobSize()];
 		memcpy(m_pData, rTag.GetBsob(), rTag.GetBsobSize());
 	} else {
-		wxASSERT(0);
+		wxFAIL;
 		m_uVal = 0;
 	}
 }
@@ -173,7 +178,7 @@ CTag::CTag(const CFileDataIO& data, bool bOptUTF8)
 				} else {
 					// Since we cannot determine the length of this tag, we
 					// simply have to abort reading the file.
-					throw CInvalidPacket(wxString::Format(wxT("Unknown tag type encounted %x, cannot proceed!"),m_uType));
+					throw CInvalidPacket(CFormat(wxT("Unknown tag type encounted %x, cannot proceed!")) % m_uType);
 				}
 		}
 	} catch (...) {
@@ -230,7 +235,7 @@ CTag &CTag::operator=(const CTag &rhs)
 			m_pData = p;
 			memcpy(m_pData, rhs.GetBsob(), rhs.GetBsobSize());
 		} else {
-			wxASSERT(0);
+			wxFAIL;
 			m_uVal = 0;
 		}
 	}
@@ -379,7 +384,7 @@ bool CTag::WriteNewEd2kTag(CFileDataIO* data, EUtf8Str eStrEncode) const
 				data->WriteString(*m_pstrVal,eStrEncode,0); 
 			} else {
 				printf("%s; Unknown tag: type=0x%02X\n", __FUNCTION__, uType);
-				wxASSERT(0);
+				wxFAIL;
 				return false;
 			}
 			break;
@@ -412,14 +417,14 @@ wxString CTag::GetFullInfo() const
 	if (!m_Name.IsEmpty()) {
 		// Special case: Kad tags, and some ED2k tags ...
 		if (m_Name.Length() == 1) {
-			strTag = wxString::Format(wxT("0x%02X"), m_Name[0]);
+			strTag = CFormat(wxT("0x%02X")) % (unsigned)m_Name[0];
 		} else {
 			strTag = wxT('\"');
 			strTag += m_Name;
 			strTag += wxT('\"');
 		}
 	} else {
-		strTag = wxString::Format(wxT("0x%02X"), m_uName);
+		strTag = CFormat(wxT("0x%02X")) % m_uName;
 	}
 	strTag += wxT("=");
 	if (m_uType == TAGTYPE_STRING) {
@@ -427,24 +432,24 @@ wxString CTag::GetFullInfo() const
 		strTag += *m_pstrVal;
 		strTag += wxT("\"");
 	} else if (m_uType >= TAGTYPE_STR1 && m_uType <= TAGTYPE_STR16) {
-		strTag += wxString::Format(wxT("(Str%u)\""), m_uType - TAGTYPE_STR1 + 1)
+		strTag += CFormat(wxT("(Str%u)\"")) % (m_uType - TAGTYPE_STR1 + 1)
 					+  *m_pstrVal + wxT("\"");
 	} else if (m_uType == TAGTYPE_UINT64) {
-		strTag += wxString::Format(wxT("(Int64)%") wxLongLongFmtSpec wxT("u"), m_uVal);
+		strTag += CFormat(wxT("(Int64)%u")) % m_uVal;
 	} else if (m_uType == TAGTYPE_UINT32) {
-		strTag += wxString::Format(wxT("(Int32)%u"), (unsigned)m_uVal);
+		strTag += CFormat(wxT("(Int32)%u")) % m_uVal;
 	} else if (m_uType == TAGTYPE_UINT16) {
-		strTag += wxString::Format(wxT("(Int16)%u"), (unsigned)m_uVal);
+		strTag += CFormat(wxT("(Int16)%u")) % m_uVal;
 	} else if (m_uType == TAGTYPE_UINT8) {
-		strTag += wxString::Format(wxT("(Int8)%u"), (unsigned)m_uVal);
+		strTag += CFormat(wxT("(Int8)%u")) % m_uVal;
 	} else if (m_uType == TAGTYPE_FLOAT32) {
-		strTag += wxString::Format(wxT("(Float32)%f"), m_fVal);
+		strTag += CFormat(wxT("(Float32)%f")) % m_fVal;
 	} else if (m_uType == TAGTYPE_BLOB) {
-		strTag += wxString::Format(wxT("(Blob)%u"), m_nSize);
+		strTag += CFormat(wxT("(Blob)%u")) % m_nSize;
 	} else if (m_uType == TAGTYPE_BSOB) {
-		strTag += wxString::Format(wxT("(Bsob)%u"), m_nSize);
+		strTag += CFormat(wxT("(Bsob)%u")) % m_nSize;
 	} else {
-		strTag += wxString::Format(wxT("Type=%u"), m_uType);
+		strTag += CFormat(wxT("Type=%u")) % m_uType;
 	}
 	return strTag;
 }
@@ -463,10 +468,6 @@ CTagHash::CTagHash(uint8 name, const CMD4Hash& value)
 
 void deleteTagPtrListEntries(TagPtrList* taglist)
 {
-	TagPtrList::const_iterator it;
-	for (it = taglist->begin(); it != taglist->end(); it++) {
-		delete *it;
-	}
-	taglist->clear();
+	DeleteContents(*taglist);
 }
 // File_checked_for_headers
