@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -23,33 +23,29 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
+#include "updownclient.h"		// Needed for CUpDownClient
+
+#include <protocol/Protocols.h>
+#include <protocol/ed2k/Client2Client/TCP.h>
+
 #include <zlib.h>
-#include <cmath>		// Needed for std::exp
 
-#include "OtherFunctions.h"	// Needed for nstrdup
-
-#include "ClientUDPSocket.h"	// Needed for CClientUDPSocket
 #include "ClientCredits.h"	// Needed for CClientCredits
 #include "Packet.h"		// Needed for CPacket
 #include "MemFile.h"		// Needed for CMemFile
 #include "UploadQueue.h"	// Needed for CUploadQueue
 #include "DownloadQueue.h"	// Needed for CDownloadQueue
-#include "Preferences.h"	// Needed for CPreferences
-#include "OtherStructs.h"	// Needed for Requested_Block_Struct
-#include "ServerConnect.h"		// Needed for CServerConnect
 #include "PartFile.h"		// Needed for PR_POWERSHARE
-#include "KnownFile.h"		// Needed for CKnownFile
-#include "KnownFileList.h"		// Needed for CKnownFileLists
 #include "ClientTCPSocket.h"	// Needed for CClientTCPSocket
 #include "SharedFileList.h"	// Needed for CSharedFileList
 #include "amule.h"		// Needed for theApp
-#include "OPCodes.h"		// Needed for PARTSIZE
-#include "BarShader.h"		// Needed for CBarShader
-#include "updownclient.h"	// Needed for CUpDownClient
 #include "ClientList.h"
 #include "Statistics.h"		// Needed for theStats
 #include "Logger.h"
-#include <common/Format.h>
+#include "ScopedPtr.h"		// Needed for CScopedArray
+#include "GuiEvents.h"		// Needed for Notify_*
+#include "FileArea.h"		// Needed for CFileArea
+
 
 //	members of CUpDownClient
 //	which are mainly used for uploading functions 
@@ -61,7 +57,7 @@ void CUpDownClient::SetUploadState(uint8 eNewState)
 			// Reset upload data rate computation
 			m_nUpDatarate = 0;
 			m_nSumForAvgUpDataRate = 0;
-			m_AvarageUDR_list.RemoveAll();
+			m_AvarageUDR_list.clear();
 		}
 		if (eNewState == US_UPLOADING) {
 			m_fSentOutOfPartReqs = 0;
@@ -73,8 +69,7 @@ void CUpDownClient::SetUploadState(uint8 eNewState)
 	}
 }
 
-#ifndef CLIENT_GUI
-uint32 CUpDownClient::GetScore(bool sysvalue, bool isdownloading, bool onlybasevalue) const
+uint32 CUpDownClient::CalculateScoreInternal()
 {
 	//TODO: complete this (friends, uploadspeed, amuleuser etc etc)
 	if (m_Username.IsEmpty()) {
@@ -103,67 +98,52 @@ uint32 CUpDownClient::GetScore(bool sysvalue, bool isdownloading, bool onlybasev
 	if (IsBanned())
 		return 0;
 
-	if (sysvalue && HasLowID() && !IsConnected()){
+	// score applies only to waiting clients, not to downloading clients
+	if (IsDownloading()) {
 		return 0;
-	}	
+	}
 
-	// TODO coded by tecxx & herbert, one yet unsolved problem here:
+	// calculate score, based on waitingtime and other factors
+	float fBaseValue = (float)(::GetTickCount()-GetWaitStartTime())/1000;
+	
+	fBaseValue *= GetScoreRatio();	// credits
+	
+	// Take file upload priority into account
+	//
+	// One yet unsolved problem here:
 	// sometimes a client asks for 2 files and there is no way to decide, which file the 
 	// client finally gets. so it could happen that he is queued first because of a 
 	// high prio file, but then asks for something completely different.
-	int filepriority = 10; // standard
-	if(pFile != NULL){ 
-		switch(pFile->GetUpPriority()) {
-		        case PR_POWERSHARE: //added for powershare (deltaHF)
-				filepriority = 2500; 
-				break; //end
-			case PR_VERYHIGH:
-				filepriority = 18;
-				break;
-			case PR_HIGH:
-				filepriority = 9;
-				break;
-			case PR_LOW:
-				filepriority = 6;
-				break;
-			case PR_VERYLOW:
-				filepriority = 2;
-				break;
-			case PR_NORMAL:
-			default:
-				filepriority = 7;
-				break;
-		}
+	float filepriority;
+	switch (pFile->GetUpPriority()) {
+		case PR_POWERSHARE:
+			filepriority = 250.0f; 
+			break;
+		case PR_VERYHIGH:
+			filepriority = 1.8f;
+			break;
+		case PR_HIGH:
+			filepriority = 0.9f;
+			break;
+		case PR_LOW:
+			filepriority = 0.6f;
+			break;
+		case PR_VERYLOW:
+			filepriority = 0.2f;
+			break;
+		case PR_NORMAL:
+		default:
+			filepriority = 0.7f;
+			break;
 	}
-	// calculate score, based on waitingtime and other factors
-	float fBaseValue;
-	if (onlybasevalue) {
-		fBaseValue = 100;
-	} else if (!isdownloading) {
-		fBaseValue = (float)(::GetTickCount()-GetWaitStartTime())/1000;
-	} else {
-		// we dont want one client to download forever
-		// the first 15 min downloadtime counts as 15 min waitingtime and you get
-		// a 15 min bonus while you are in the first 15 min :)
-		// (to avoid 20 sec downloads) after this the score won't raise anymore 
-		fBaseValue = (float)(m_dwUploadTime-GetWaitStartTime());
-		wxASSERT( m_dwUploadTime > GetWaitStartTime()); // Obviously
-		fBaseValue += (float)(::GetTickCount() - m_dwUploadTime > 900000)? 900000:1800000;
-		fBaseValue /= 1000;
-	}
-	
-	float modif = GetScoreRatio();
-	fBaseValue *= modif;
-	
-	if (!onlybasevalue) {
-		fBaseValue *= (float(filepriority)/10.0f);
-	}
-	if( (IsEmuleClient() || GetClientSoft() < 10) && m_byEmuleVersion <= 0x19) {
+	fBaseValue *= filepriority;
+
+	if ( (IsEmuleClient() || GetClientSoft() < 10) && m_byEmuleVersion <= 0x19) {
 		fBaseValue *= 0.5f;
 	}
 	return (uint32)fBaseValue;
 }
-#endif
+
 
 // Checks if it is next requested block from another chunk of the actual file or from another file 
 // 
@@ -175,17 +155,17 @@ bool CUpDownClient::IsDifferentPartBlock() const // [Tarod 12/22/2002]
 	bool different_part = false;
 	
 	// Check if we have good lists and proceed to check for different chunks
-	if (!m_BlockRequests_queue.IsEmpty() && !m_DoneBlocks_list.IsEmpty())
+	if ((!m_BlockRequests_queue.empty()) && !m_DoneBlocks_list.empty())
 	{
 		Requested_Block_Struct* last_done_block = NULL;
 		Requested_Block_Struct* next_requested_block = NULL;
-		uint32 last_done_part = 0xffffffff;
-		uint32 next_requested_part = 0xffffffff;
+		uint64 last_done_part = 0xffffffff;
+		uint64 next_requested_part = 0xffffffff;
 			
 			
 		// Get last block and next pending
-		last_done_block = (Requested_Block_Struct*)m_DoneBlocks_list.GetHead();
-		next_requested_block = (Requested_Block_Struct*)m_BlockRequests_queue.GetHead(); 
+		last_done_block = m_DoneBlocks_list.front();
+		next_requested_block = m_BlockRequests_queue.front();
 			
 		// Calculate corresponding parts to blocks
 		last_done_part = last_done_block->StartOffset / PARTSIZE;
@@ -194,12 +174,12 @@ bool CUpDownClient::IsDifferentPartBlock() const // [Tarod 12/22/2002]
 		// Test is we are asking same file and same part
 		if ( last_done_part != next_requested_part) { 
 			different_part = true;
-			AddDebugLogLineM(false, logClient, wxT("Session ended due to new chunk."));
+			AddDebugLogLineN(logClient, wxT("Session ended due to new chunk."));
 		}
 	
 		if (md4cmp(last_done_block->FileID, next_requested_block->FileID) != 0) { 
 			different_part = true;
-			AddDebugLogLineM(false, logClient, wxT("Session ended due to different file."));
+			AddDebugLogLineN(logClient, wxT("Session ended due to different file."));
 		}
 	} 
 
@@ -209,140 +189,110 @@ bool CUpDownClient::IsDifferentPartBlock() const // [Tarod 12/22/2002]
 
 void CUpDownClient::CreateNextBlockPackage()
 {
-    // See if we can do an early return. There may be no new blocks to load from disk and add to buffer, or buffer may be large enough allready.
-    if(m_BlockRequests_queue.IsEmpty() || // There are no new blocks requested
-       m_addedPayloadQueueSession > GetQueueSessionPayloadUp() && m_addedPayloadQueueSession-GetQueueSessionPayloadUp() > 50*1024) { // the buffered data is large enough allready
-        return;
-    }
-
-    CFile file;
-	byte* filedata = 0;
-	wxString fullname;
 	try {
-        // Buffer new data if current buffer is less than 100 KBytes
-        while (!m_BlockRequests_queue.IsEmpty() &&
-               (m_addedPayloadQueueSession <= GetQueueSessionPayloadUp() || m_addedPayloadQueueSession-GetQueueSessionPayloadUp() < 100*1024)) {
+		// Buffer new data if current buffer is less than 100 KBytes
+		while (!m_BlockRequests_queue.empty()
+			   && m_addedPayloadQueueSession - m_nCurQueueSessionPayloadUp < 100*1024) {
 
-			Requested_Block_Struct* currentblock = m_BlockRequests_queue.GetHead();
-			CKnownFile* srcfile = theApp.sharedfiles->GetFileByID(CMD4Hash(currentblock->FileID));
+			Requested_Block_Struct* currentblock = m_BlockRequests_queue.front();
+			CKnownFile* srcfile = theApp->sharedfiles->GetFileByID(CMD4Hash(currentblock->FileID));
 			
 			if (!srcfile) {
 				throw wxString(wxT("requested file not found"));
 			}
 
-			if (srcfile->IsPartFile() && ((CPartFile*)srcfile)->GetStatus() != PS_COMPLETE) {
-				#warning This seems a good idea from eMule. We must import this.
-				#if 0
-				// Do not access a part file, if it is currently moved into the incoming directory.
-				// Because the moving of part file into the incoming directory may take a noticable 
-				// amount of time, we can not wait for 'm_FileCompleteMutex' and block the main thread.
-				if (!((CPartFile*)srcfile)->m_FileCompleteMutex.Lock(0)){ // just do a quick test of the mutex's state and return if it's locked.
-					return;
-				}
-				lockFile.m_pObject = &((CPartFile*)srcfile)->m_FileCompleteMutex;
-				// If it's a part file which we are uploading the file remains locked until we've read the
-				// current block. This way the file completion thread can not (try to) "move" the file into
-				// the incoming directory.
-				#endif
+			// Check if this know file is a CPartFile. 
+			// For completed part files IsPartFile() returns false, so they are 
+			// correctly treated as plain CKnownFile.
+			CPartFile* srcPartFile = srcfile->IsPartFile() ? (CPartFile*)srcfile : NULL;
 
-				fullname = ((CPartFile*)srcfile)->GetFullName();
-				fullname.Truncate(fullname.Length()-4);
-			} else {
-				fullname = JoinPaths(srcfile->GetFilePath(), srcfile->GetFileName());
-			}
-		
-			uint32 togo;
-			if (currentblock->StartOffset > currentblock->EndOffset){
-				togo = currentblock->EndOffset + (srcfile->GetFileSize() - currentblock->StartOffset);
-			} else {
-				togo = currentblock->EndOffset - currentblock->StartOffset;
-				
-				if (srcfile->IsPartFile() && !((CPartFile*)srcfile)->IsComplete(currentblock->StartOffset,currentblock->EndOffset-1)) {
-					throw wxString(wxT("Asked for incomplete block "));
-				}
+			// THIS EndOffset points BEHIND the last byte requested
+			// (other than the offsets used in the PartFile code)
+			if (currentblock->EndOffset > srcfile->GetFileSize()) {
+				throw wxString(CFormat(wxT("Asked for data up to %d beyond end of file (%d)"))
+									% currentblock->EndOffset % srcfile->GetFileSize());
+			} else if (currentblock->StartOffset > currentblock->EndOffset) { 
+				throw wxString(CFormat(wxT("Asked for invalid block (start %d > end %d)"))
+									% currentblock->StartOffset % currentblock->EndOffset);
 			}
 
+			uint64 togo = currentblock->EndOffset - currentblock->StartOffset;
+
 			if (togo > EMBLOCKSIZE * 3) {
-				throw wxString(wxT("Client requested too large of a block."));
-			}
-			
-			if (!srcfile->IsPartFile()){
-				if ( !file.Open(fullname,CFile::read) ) {
-					// The file was most likely moved/deleted. However it is likely that the
-					// same is true for other files, so we recheck all shared files. 
-					AddLogLineM( false, CFormat( _("Failed to open file (%s), removing from list of shared files.") ) % srcfile->GetFileName() );
-					theApp.sharedfiles->RemoveFile(srcfile);
-					
-					throw wxString(wxT("Failed to open requested file: Removing from list of shared files!"));
-				}			
-			
-				file.Seek(currentblock->StartOffset, wxFromStart);
-				
-				filedata = new byte[togo+500];
-				file.Read(filedata, togo);
-				file.Close();
-			} else {
-				CPartFile* partfile = (CPartFile*)srcfile;
-				partfile->m_hpartfile.Seek(currentblock->StartOffset);
-				
-				filedata = new byte[togo+500];
-				partfile->m_hpartfile.Read(filedata, togo); 
-				// Partfile should NOT be closed!!!
+				throw wxString(CFormat(wxT("Client requested too large block (%d > %d)"))
+									% togo % (EMBLOCKSIZE * 3));
 			}
 
-			#warning Part of the above import.
-			#if 0
-			if (lockFile.m_pObject){
-				lockFile.m_pObject->Unlock(); // Unlock the (part) file as soon as we are done with accessing it.
-				lockFile.m_pObject = NULL;
+			CFileArea area;
+			if (srcPartFile) {
+				if (!srcPartFile->IsComplete(currentblock->StartOffset,currentblock->EndOffset-1)) {
+					throw wxString(CFormat(wxT("Asked for incomplete block (%d - %d)"))
+									% currentblock->StartOffset % (currentblock->EndOffset-1));
+				}
+				if (!srcPartFile->ReadData(area, currentblock->StartOffset, togo)) {
+					throw wxString(wxT("Failed to read from requested partfile"));
+				}
+			} else {
+				CFileAutoClose file;
+				CPath fullname = srcfile->GetFilePath().JoinPaths(srcfile->GetFileName());
+				if ( !file.Open(fullname, CFile::read) ) {
+					// The file was most likely moved/deleted. So remove it from the list of shared files.
+					AddLogLineN(CFormat( _("Failed to open file (%s), removing from list of shared files.") ) % srcfile->GetFileName() );
+					theApp->sharedfiles->RemoveFile(srcfile);
+					
+					throw wxString(wxT("Failed to open requested file: Removing from list of shared files!"));
+				}
+				area.ReadAt(file, currentblock->StartOffset, togo);
 			}
-			#endif
-	
+			area.CheckError();
+
 			SetUploadFileID(srcfile);
 
 			// check extention to decide whether to compress or not
 			if (m_byDataCompVer == 1 && GetFiletype(srcfile->GetFileName()) != ftArchive) {
-				CreatePackedPackets(filedata,togo,currentblock);
+				CreatePackedPackets(area.GetBuffer(), togo, currentblock);
 			} else {
-				CreateStandartPackets(filedata,togo,currentblock);
+				CreateStandardPackets(area.GetBuffer(), togo, currentblock);
 			}
 			
 			// file statistic
 			srcfile->statistic.AddTransferred(togo);
 
-            m_addedPayloadQueueSession += togo;
+			m_addedPayloadQueueSession += togo;
 
-			m_DoneBlocks_list.AddHead(m_BlockRequests_queue.RemoveHead());
-			delete[] filedata;
-			filedata = NULL;
+			Requested_Block_Struct* block = m_BlockRequests_queue.front();
+			
+			m_BlockRequests_queue.pop_front();			
+			m_DoneBlocks_list.push_front(block);
 		}
 
 		return;
-	} catch (const wxString& error) {
-		AddDebugLogLineM(false, logClient, wxT("Client '") + GetUserName() + wxT("' caused error while creating packet (") + error + wxT(") - disconnecting client"));
+	} catch (const wxString& DEBUG_ONLY(error)) {
+		AddDebugLogLineN(logClient, 
+			CFormat(wxT("Client '%s' (%s) caused error while creating packet (%s) - disconnecting client"))
+				% GetUserName() % GetFullIP() % error);
 	} catch (const CIOFailureException& error) {
-		AddDebugLogLineM(true, logClient, wxT("IO failure while reading requested file: ") + error.what());
-	} catch (const CEOFException& error) {
-		AddDebugLogLineM(true, logClient, GetClientFullInfo() + wxT(" requested file-data at an invalid position - disconnecting"));
+		AddDebugLogLineC(logClient, wxT("IO failure while reading requested file: ") + error.what());
+	} catch (const CEOFException& WXUNUSED(error)) {
+		AddDebugLogLineC(logClient, GetClientFullInfo() + wxT(" requested file-data at an invalid position - disconnecting"));
 	}
 	
 	// Error occured.	
-	theApp.uploadqueue->RemoveFromUploadQueue(this);
-	delete[] filedata;
+	theApp->uploadqueue->RemoveFromUploadQueue(this);
 }
 
 
-void CUpDownClient::CreateStandartPackets(const byte* buffer, uint32 togo, Requested_Block_Struct* currentblock)
+void CUpDownClient::CreateStandardPackets(const byte* buffer, uint32 togo, Requested_Block_Struct* currentblock)
 {
 	uint32 nPacketSize;
 
-	CMemFile memfile((byte*)buffer, togo);
+	CMemFile memfile(buffer, togo);
 	if (togo > 10240) {
 		nPacketSize = togo/(uint32)(togo/10240);
 	} else {
 		nPacketSize = togo;
 	}
-
+	
 	while (togo){
 		if (togo < nPacketSize*2) {
 			nPacketSize = togo;
@@ -351,18 +301,30 @@ void CUpDownClient::CreateStandartPackets(const byte* buffer, uint32 togo, Reque
 		wxASSERT(nPacketSize);
 		togo -= nPacketSize;
 		
-		CMemFile data(nPacketSize+24);
+		uint64 endpos = (currentblock->EndOffset - togo);
+		uint64 startpos = endpos - nPacketSize;
+		
+		bool bLargeBlocks = (startpos > 0xFFFFFFFF) || (endpos > 0xFFFFFFFF);
+		
+		CMemFile data(nPacketSize + 16 + 2 * (bLargeBlocks ? 8 :4));
 		data.WriteHash(GetUploadFileID());
-		data.WriteUInt32((currentblock->EndOffset - togo) - nPacketSize);
-		data.WriteUInt32((currentblock->EndOffset - togo));
+		if (bLargeBlocks) {
+			data.WriteUInt64(startpos);
+			data.WriteUInt64(endpos);
+		} else {
+			data.WriteUInt32(startpos);
+			data.WriteUInt32(endpos);
+		}
 		char *tempbuf = new char[nPacketSize];
 		memfile.Read(tempbuf, nPacketSize);
 		data.Write(tempbuf, nPacketSize);
 		delete [] tempbuf;
-		CPacket* packet = new CPacket(&data,OP_EDONKEYPROT,OP_SENDINGPART);
-	
-		theStats::AddUpOverheadFileRequest(24);
+		CPacket* packet = new CPacket(data, (bLargeBlocks ? OP_EMULEPROT : OP_EDONKEYPROT), (bLargeBlocks ? (uint8)OP_SENDINGPART_I64 : (uint8)OP_SENDINGPART));	
+		theStats::AddUpOverheadFileRequest(16 + 2 * (bLargeBlocks ? 8 :4));
 		theStats::AddUploadToSoft(GetClientSoft(), nPacketSize);
+		AddDebugLogLineN(logLocalClient, 
+			CFormat(wxT("Local Client: %s to %s"))
+				% (bLargeBlocks ? wxT("OP_SENDINGPART_I64") : wxT("OP_SENDINGPART")) % GetFullIP() );
 		m_socket->SendPacket(packet,true,false, nPacketSize);
 	}
 }
@@ -370,16 +332,15 @@ void CUpDownClient::CreateStandartPackets(const byte* buffer, uint32 togo, Reque
 
 void CUpDownClient::CreatePackedPackets(const byte* buffer, uint32 togo, Requested_Block_Struct* currentblock)
 {
-	byte* output = new byte[togo+300];
 	uLongf newsize = togo+300;
-	uint16 result = compress2(output, &newsize, buffer, togo,9);
+	CScopedArray<byte> output(newsize);
+	uint16 result = compress2(output.get(), &newsize, buffer, togo, 9);
 	if (result != Z_OK || togo <= newsize){
-		delete[] output;
-		CreateStandartPackets(buffer, togo, currentblock);
+		CreateStandardPackets(buffer, togo, currentblock);
 		return;
 	}
 	
-	CMemFile memfile(output,newsize);
+	CMemFile memfile(output.get(), newsize);
 	
 	uint32 totalPayloadSize = 0;
 	uint32 oldSize = togo;
@@ -397,15 +358,21 @@ void CUpDownClient::CreatePackedPackets(const byte* buffer, uint32 togo, Request
 		}
 		togo -= nPacketSize;
 
-		CMemFile data(nPacketSize+24);
+		bool isLargeBlock = (currentblock->StartOffset > 0xFFFFFFFF) || (currentblock->EndOffset > 0xFFFFFFFF);
+		
+		CMemFile data(nPacketSize + 16 + (isLargeBlock ? 12 : 8));
 		data.WriteHash(GetUploadFileID());
-		data.WriteUInt32(currentblock->StartOffset);
+		if (isLargeBlock) {
+			data.WriteUInt64(currentblock->StartOffset);
+		} else {
+			data.WriteUInt32(currentblock->StartOffset);
+		}
 		data.WriteUInt32(newsize);			
 		char *tempbuf = new char[nPacketSize];
 		memfile.Read(tempbuf, nPacketSize);
 		data.Write(tempbuf,nPacketSize);
 		delete [] tempbuf;
-		CPacket* packet = new CPacket(&data, OP_EMULEPROT, OP_COMPRESSEDPART);
+		CPacket* packet = new CPacket(data, OP_EMULEPROT, (isLargeBlock ? OP_COMPRESSEDPART_I64 : OP_COMPRESSEDPART));
 	
 		// approximate payload size
 		uint32 payloadSize = nPacketSize*oldSize/newsize;
@@ -419,9 +386,11 @@ void CUpDownClient::CreatePackedPackets(const byte* buffer, uint32 togo, Request
 		// put packet directly on socket
 		theStats::AddUpOverheadFileRequest(24);
 		theStats::AddUploadToSoft(GetClientSoft(), nPacketSize);
+		AddDebugLogLineN(logLocalClient, 
+			CFormat(wxT("Local Client: %s to %s"))
+				% (isLargeBlock ? wxT("OP_COMPRESSEDPART_I64") : wxT("OP_COMPRESSEDPART")) % GetFullIP() );
 		m_socket->SendPacket(packet,true,false, payloadSize);			
 	}
-	delete[] output;
 }
 
 
@@ -444,7 +413,7 @@ void CUpDownClient::ProcessExtendedInfo(const CMemFile *data, CKnownFile *tempre
 	
 	uint16 nED2KUpPartCount = data->ReadUInt16();
 	if (!nED2KUpPartCount) {
-		m_upPartStatus.resize( tempreqfile->GetPartCount(), 0 );
+		m_upPartStatus.setsize( tempreqfile->GetPartCount(), 0 );
 	} else {
 		if (tempreqfile->GetED2KPartCount() != nED2KUpPartCount) {
 			// We already checked if we are talking about the same file.. So if we get here, something really strange happened!
@@ -452,14 +421,14 @@ void CUpDownClient::ProcessExtendedInfo(const CMemFile *data, CKnownFile *tempre
 			return;
 		}
 	
-		m_upPartStatus.resize( tempreqfile->GetPartCount(), 0 );
+		m_upPartStatus.setsize( tempreqfile->GetPartCount(), 0 );
 	
 		try {
 			uint16 done = 0;
 			while (done != m_upPartStatus.size()) {
 				uint8 toread = data->ReadUInt8();
 				for (sint32 i = 0;i != 8;i++){
-					m_upPartStatus[done] = (toread>>i)&1;
+					m_upPartStatus.set(done, (toread>>i)&1);
 					//	We may want to use this for another feature..
 					//	if (m_upPartStatus[done] && !tempreqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
 					// bPartsNeeded = true;
@@ -488,7 +457,7 @@ void CUpDownClient::ProcessExtendedInfo(const CMemFile *data, CKnownFile *tempre
 	
 	m_uploadingfile->UpdateUpPartsFrequency( this, true ); // Increment
 	
-	Notify_QlistRefreshClient(this);
+	Notify_SharedCtrlRefreshClient(ECID(), AVAILABLE_SOURCE);
 }
 
 
@@ -507,8 +476,7 @@ void CUpDownClient::SetUploadFileID(CKnownFile* newreqfile)
 		
 		if (m_requpfileid != newreqfile->GetFileHash()) {
 			m_requpfileid = newreqfile->GetFileHash();
-			m_upPartStatus.clear();
-			m_upPartStatus.resize( newreqfile->GetPartCount(), 0 );
+			m_upPartStatus.setsize( newreqfile->GetPartCount(), 0 );
 		} else {
 			// this is the same file we already had assigned. Only update data.
 			newreqfile->UpdateUpPartsFrequency(this, true); // Increment
@@ -527,27 +495,35 @@ void CUpDownClient::SetUploadFileID(CKnownFile* newreqfile)
 void CUpDownClient::AddReqBlock(Requested_Block_Struct* reqblock)
 {
 	if (GetUploadState() != US_UPLOADING) {
-		AddDebugLogLineM(false, logRemoteClient, wxT("UploadClient: Client tried to add requested block when not in upload slot! Prevented requested blocks from being added."));
+		AddDebugLogLineN(logRemoteClient, wxT("UploadClient: Client tried to add requested block when not in upload slot! Prevented requested blocks from being added."));
 		delete reqblock;
 		return;
 	}
 	
-	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0;m_DoneBlocks_list.GetNext(pos)){
-		if (reqblock->StartOffset == m_DoneBlocks_list.GetAt(pos)->StartOffset && reqblock->EndOffset == m_DoneBlocks_list.GetAt(pos)->EndOffset){
-			delete reqblock;
-			return;
+	{
+		std::list<Requested_Block_Struct*>::iterator it = m_DoneBlocks_list.begin();
+		for (; it != m_DoneBlocks_list.end(); ++it) {
+			if (reqblock->StartOffset == (*it)->StartOffset && reqblock->EndOffset == (*it)->EndOffset) {
+				delete reqblock;
+				return;
+			}
 		}
 	}
-	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0;m_BlockRequests_queue.GetNext(pos)){
-		if (reqblock->StartOffset == m_BlockRequests_queue.GetAt(pos)->StartOffset && reqblock->EndOffset == m_BlockRequests_queue.GetAt(pos)->EndOffset){
-			delete reqblock;
-			return;
-		}
-	}
-	m_BlockRequests_queue.AddTail(reqblock);
 
+	{
+		std::list<Requested_Block_Struct*>::iterator it = m_BlockRequests_queue.begin();
+		for (; it != m_BlockRequests_queue.end(); ++it) {
+			if (reqblock->StartOffset == (*it)->StartOffset && reqblock->EndOffset == (*it)->EndOffset) {
+				delete reqblock;
+				return;
+			}
+		}
+	}
+	
+	m_BlockRequests_queue.push_back(reqblock);
 }
 
+
 uint32 CUpDownClient::GetWaitStartTime() const
 {
 	uint32 dwResult = 0;
@@ -581,6 +557,20 @@ void CUpDownClient::ClearWaitStartTime()
 }
 
 
+void CUpDownClient::ResetSessionUp()
+{
+	m_nCurSessionUp = m_nTransferredUp;
+	m_addedPayloadQueueSession = 0;
+	m_nCurQueueSessionPayloadUp = 0;
+	// If upload was resumed there can be a remaining payload in the socket
+	// causing (prepared - sent) getting negative. So reset the counter here.
+	if (m_socket) {
+		CEMSocket* s = m_socket;
+		s->GetSentPayloadSinceLastCallAndReset();
+	}
+}
+
+
 uint32 CUpDownClient::SendBlockData()
 {
     uint32 curTick = ::GetTickCount();
@@ -600,13 +590,13 @@ uint32 CUpDownClient::SendBlockData()
 //		thePrefs.Add2SessionTransferData(GetClientSoft(), uUpStatsPort, true, true, sentBytesPartFile, (IsFriend() && GetFriendSlot()));
 
 		m_nTransferredUp += sentBytesCompleteFile + sentBytesPartFile;
-        credits->AddUploaded(sentBytesCompleteFile + sentBytesPartFile, GetIP(), theApp.CryptoAvailable());
+        credits->AddUploaded(sentBytesCompleteFile + sentBytesPartFile, GetIP(), theApp->CryptoAvailable());
 
         sentBytesPayload = s->GetSentPayloadSinceLastCallAndReset();
         m_nCurQueueSessionPayloadUp += sentBytesPayload;
 
-        if (theApp.uploadqueue->CheckForTimeOver(this)) {
-            theApp.uploadqueue->RemoveFromUploadQueue(this, true);
+        if (theApp->uploadqueue->CheckForTimeOver(this)) {
+            theApp->uploadqueue->RemoveFromUploadQueue(this);
 			SendOutOfPartReqsAndAddToWaitingQueue();
         } else {
             // read blocks from file and put on socket
@@ -615,24 +605,25 @@ uint32 CUpDownClient::SendBlockData()
     }
 
     if(sentBytesCompleteFile + sentBytesPartFile > 0 ||
-        m_AvarageUDR_list.GetCount() == 0 || (curTick - m_AvarageUDR_list.GetTail().timestamp) > 1*1000) {
+        m_AvarageUDR_list.empty() || (curTick - m_AvarageUDR_list.back().timestamp) > 1*1000) {
         // Store how much data we've transferred this round,
         // to be able to calculate average speed later
         // keep sum of all values in list up to date
         TransferredData newitem = {sentBytesCompleteFile + sentBytesPartFile, curTick};
-        m_AvarageUDR_list.AddTail(newitem);
+        m_AvarageUDR_list.push_back(newitem);
         m_nSumForAvgUpDataRate += sentBytesCompleteFile + sentBytesPartFile;
     }
 
     // remove to old values in list
-    while (m_AvarageUDR_list.GetCount() > 0 && (curTick - m_AvarageUDR_list.GetHead().timestamp) > 10*1000) {
+    while ((!m_AvarageUDR_list.empty()) && (curTick - m_AvarageUDR_list.front().timestamp) > 10*1000) {
         // keep sum of all values in list up to date
-        m_nSumForAvgUpDataRate -= m_AvarageUDR_list.RemoveHead().datalen;
+        m_nSumForAvgUpDataRate -= m_AvarageUDR_list.front().datalen;
+		m_AvarageUDR_list.pop_front();
     }
 
     // Calculate average speed for this slot
-    if(m_AvarageUDR_list.GetCount() > 0 && (curTick - m_AvarageUDR_list.GetHead().timestamp) > 0 && GetUpStartTimeDelay() > 2*1000) {
-        m_nUpDatarate = ((uint64)m_nSumForAvgUpDataRate*1000) / (curTick-m_AvarageUDR_list.GetHead().timestamp);
+    if ((!m_AvarageUDR_list.empty()) && (curTick - m_AvarageUDR_list.front().timestamp) > 0 && GetUpStartTimeDelay() > 2*1000) {
+        m_nUpDatarate = ((uint64)m_nSumForAvgUpDataRate*1000) / (curTick-m_AvarageUDR_list.front().timestamp);
     } else {
         // not enough values to calculate trustworthy speed. Use -1 to tell this
         m_nUpDatarate = 0; //-1;
@@ -642,7 +633,7 @@ uint32 CUpDownClient::SendBlockData()
 	m_cSendblock++;
 	if (m_cSendblock == 30){
 		m_cSendblock = 0;
-		Notify_UploadCtrlRefreshClient(this);
+		Notify_SharedCtrlRefreshClient(ECID(), AVAILABLE_SOURCE);
 	}
 
     return sentBytesCompleteFile + sentBytesPartFile;
@@ -660,14 +651,13 @@ void CUpDownClient::SendOutOfPartReqsAndAddToWaitingQueue()
 	//the downloader didn't send any request blocks. Then the connection times out..
 	//I did some tests with eDonkey also and it seems to work well with them also..
 	
-	AddDebugLogLineM(false, logLocalClient, wxT("Local Client: Sending OP_OUTOFPARTREQS"));
-	
 	// Send this inmediately, don't queue.
-	CPacket* pPacket = new CPacket(OP_OUTOFPARTREQS, 0);
+	CPacket* pPacket = new CPacket(OP_OUTOFPARTREQS, 0, OP_EDONKEYPROT);
 	theStats::AddUpOverheadFileRequest(pPacket->GetPacketSize());
+	AddDebugLogLineN( logLocalClient, wxT("Local Client: OP_OUTOFPARTREQS to ") + GetFullIP() );
 	SendPacket(pPacket, true, true);
 	
-	theApp.uploadqueue->AddClientToQueue(this);
+	theApp->uploadqueue->AddClientToQueue(this);
 }
 
 
@@ -685,12 +675,12 @@ void CUpDownClient::FlushSendBlocks()
 
 void CUpDownClient::SendHashsetPacket(const CMD4Hash& forfileid)
 {
-	CKnownFile* file = theApp.sharedfiles->GetFileByID( forfileid );
+	CKnownFile* file = theApp->sharedfiles->GetFileByID( forfileid );
 	bool from_dq = false;
 	if ( !file ) {
 		from_dq = true;
-		if ((file = theApp.downloadqueue->GetFileByID(forfileid)) == NULL) {
-			AddLogLineM(false, CFormat( _("Hashset requested for unknown file: %s") ) % forfileid.Encode() );
+		if ((file = theApp->downloadqueue->GetFileByID(forfileid)) == NULL) {
+			AddLogLineN(CFormat( _("Hashset requested for unknown file: %s") ) % forfileid.Encode() );
 		
 			return;
 		}
@@ -698,13 +688,13 @@ void CUpDownClient::SendHashsetPacket(const CMD4Hash& forfileid)
 	
 	if ( !file->GetHashCount() ) {
 		if (from_dq) {
-			AddDebugLogLineM(false, logRemoteClient, wxT("Requested hashset could not be found"));	
+			AddDebugLogLineN(logRemoteClient, wxT("Requested hashset could not be found"));
 			return;
 		} else {
-			file = theApp.downloadqueue->GetFileByID(forfileid);
+			file = theApp->downloadqueue->GetFileByID(forfileid);
 			if (!(file && file->GetHashCount())) {
-				AddDebugLogLineM(false, logRemoteClient, wxT("Requested hashset could not be found"));	
-				return;				
+				AddDebugLogLineN(logRemoteClient, wxT("Requested hashset could not be found"));
+				return;
 			}
 		}
 	}	
@@ -716,9 +706,9 @@ void CUpDownClient::SendHashsetPacket(const CMD4Hash& forfileid)
 	for (int i = 0; i != parts; i++) {
 		data.WriteHash(file->GetPartHash(i));
 	}
-	CPacket* packet = new CPacket(&data);	
-	packet->SetOpCode(OP_HASHSETANSWER);
+	CPacket* packet = new CPacket(data, OP_EDONKEYPROT, OP_HASHSETANSWER);	
 	theStats::AddUpOverheadFileRequest(packet->GetPacketSize());
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_HASHSETANSWER to ") + GetFullIP());
 	SendPacket(packet,true,true);
 }
 
@@ -726,24 +716,16 @@ void CUpDownClient::SendHashsetPacket(const CMD4Hash& forfileid)
 void CUpDownClient::ClearUploadBlockRequests()
 {
 	FlushSendBlocks();
-	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0; ) {
-		delete m_BlockRequests_queue.GetNext(pos);
-	}
-	m_BlockRequests_queue.RemoveAll();
-	
-	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0; ) {
-		delete m_DoneBlocks_list.GetNext(pos);
-	}
-	m_DoneBlocks_list.RemoveAll();
+	DeleteContents(m_BlockRequests_queue);
+	DeleteContents(m_DoneBlocks_list);
 }
 
-
 void CUpDownClient::SendRankingInfo(){
 	if (!ExtProtocolAvailable()) {
 		return;
 	}
 
-	uint16 nRank = theApp.uploadqueue->GetWaitingPosition(this);
+	uint16 nRank = GetUploadQueueWaitingPosition();
 	if (!nRank) {
 		return;
 	}
@@ -753,11 +735,9 @@ void CUpDownClient::SendRankingInfo(){
 	// Kry: what are these zero bytes for. are they really correct?
 	// Kry - Well, eMule does like that. I guess they're ok.
 	data.WriteUInt32(0); data.WriteUInt32(0); data.WriteUInt16(0);
-	AddDebugLogLineM(false, logLocalClient, wxT("Local Client: OP_QUEUERANKING "));
-	CPacket* packet = new CPacket(&data,OP_EMULEPROT);
-	packet->SetOpCode(OP_QUEUERANKING);
-	
+	CPacket* packet = new CPacket(data, OP_EMULEPROT, OP_QUEUERANKING);
 	theStats::AddUpOverheadOther(packet->GetPacketSize());
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_QUEUERANKING to ") + GetFullIP());
 	SendPacket(packet,true,true);
 }
 
@@ -769,8 +749,8 @@ void CUpDownClient::SendCommentInfo(CKnownFile* file)
 	}
 	m_bCommentDirty = false;
 
-	// We used to limit the comment to 50 before, now we do not anymore.
-	const wxString& desc = file->GetFileComment();
+	// Truncate to max len.
+	wxString desc = file->GetFileComment().Left(MAXFILECOMMENTLEN);
 	uint8 rating = file->GetFileRating();
 	
 	if ( file->GetFileRating() == 0 && desc.IsEmpty() ) {
@@ -781,39 +761,35 @@ void CUpDownClient::SendCommentInfo(CKnownFile* file)
 	data.WriteUInt8(rating);
 	data.WriteString(desc, GetUnicodeSupport(), 4 /* size it's uint32 */);
 	
-	AddDebugLogLineM(false, logLocalClient, wxT("Local Client: OP_FILEDESC"));	
-	CPacket* packet = new CPacket(&data,OP_EMULEPROT);
-	packet->SetOpCode(OP_FILEDESC);
+	CPacket* packet = new CPacket(data, OP_EMULEPROT, OP_FILEDESC);
 	theStats::AddUpOverheadOther(packet->GetPacketSize());
+	AddDebugLogLineN(logLocalClient, wxT("Local Client: OP_FILEDESC to ") + GetFullIP());
 	SendPacket(packet,true);
 }
 
 void  CUpDownClient::UnBan(){
 	m_Aggressiveness = 0;
 	
-	theApp.clientlist->AddTrackClient(this);
-	theApp.clientlist->RemoveBannedClient( GetIP() );
+	theApp->clientlist->AddTrackClient(this);
+	theApp->clientlist->RemoveBannedClient( GetIP() );
 	SetUploadState(US_NONE);
 	ClearWaitStartTime();
-	
-	Notify_ShowQueueCount(theStats::GetWaitingUserCount());
 }
 
 void CUpDownClient::Ban(){
-	theApp.clientlist->AddTrackClient(this);
-	theApp.clientlist->AddBannedClient( GetIP() );
+	theApp->clientlist->AddTrackClient(this);
+	theApp->clientlist->AddBannedClient( GetIP() );
 	
-	AddDebugLogLineM( false, logClient, wxT("Client '") + GetUserName() + wxT("' seems to be an aggressive client and is banned from the uploadqueue"));
+	AddDebugLogLineN(logClient, wxT("Client '") + GetUserName() + wxT("' seems to be an aggressive client and is banned from the uploadqueue"));
 	
 	SetUploadState(US_BANNED);
 	
-	Notify_ShowQueueCount(theStats::GetWaitingUserCount());
-	Notify_QlistRefreshClient(this);
+	Notify_SharedCtrlRefreshClient(ECID(), UNAVAILABLE_SOURCE);
 }
 
 bool CUpDownClient::IsBanned() const
 {
-	return ( (theApp.clientlist->IsBannedClient(GetIP()) ) && m_nDownloadState != DS_DOWNLOADING);
+	return ( (theApp->clientlist->IsBannedClient(GetIP()) ) && m_nDownloadState != DS_DOWNLOADING);
 }
 
 void CUpDownClient::CheckForAggressive()
@@ -832,7 +808,7 @@ void CUpDownClient::CheckForAggressive()
 		
 		// Is the client EVIL?
 		if ( m_Aggressiveness >= 10 && (!IsBanned() && m_nDownloadState != DS_DOWNLOADING )) {
-			AddDebugLogLineM( false, logClient, CFormat( wxT("Aggressive client banned (score: %d): %s -- %s -- %s") ) 
+			AddDebugLogLineN(logClient, CFormat( wxT("Aggressive client banned (score: %d): %s -- %s -- %s") )
 				% m_Aggressiveness
 				% m_Username
 				% m_strModVersion
@@ -852,10 +828,91 @@ void CUpDownClient::CheckForAggressive()
 void CUpDownClient::SetUploadFileID(const CMD4Hash& new_id)
 {
 	// Update the uploading file found 
-	CKnownFile* uploadingfile = theApp.sharedfiles->GetFileByID(new_id);
+	CKnownFile* uploadingfile = theApp->sharedfiles->GetFileByID(new_id);
 	if ( !uploadingfile ) {
 		// Can this really happen?
-		uploadingfile = theApp.downloadqueue->GetFileByID(new_id);
+		uploadingfile = theApp->downloadqueue->GetFileByID(new_id);
 	}
 	SetUploadFileID(uploadingfile); // This will update queue count on old and new file.
 }
+
+void CUpDownClient::ProcessRequestPartsPacket(const byte* pachPacket, uint32 nSize, bool largeblocks) {
+	
+	CMemFile data(pachPacket, nSize);
+	
+	CMD4Hash reqfilehash = data.ReadHash();
+	
+	uint64 auStartOffsets[3];
+	uint64 auEndOffsets[3];
+	
+	if (largeblocks) {
+		auStartOffsets[0] = data.ReadUInt64();
+		auStartOffsets[1] = data.ReadUInt64();
+		auStartOffsets[2] = data.ReadUInt64();
+		
+		auEndOffsets[0] = data.ReadUInt64();
+		auEndOffsets[1] = data.ReadUInt64();
+		auEndOffsets[2] = data.ReadUInt64();
+	} else {
+		auStartOffsets[0] = data.ReadUInt32();
+		auStartOffsets[1] = data.ReadUInt32();
+		auStartOffsets[2] = data.ReadUInt32();
+		
+		auEndOffsets[0] = data.ReadUInt32();
+		auEndOffsets[1] = data.ReadUInt32();
+		auEndOffsets[2] = data.ReadUInt32();		
+	}
+	
+	for (unsigned int i = 0; i < itemsof(auStartOffsets); i++) {
+		AddDebugLogLineN(logClient,
+			CFormat(wxT("Client %s requests %d File block %d-%d (%d bytes):"))
+				% GetFullIP() % i % auStartOffsets[i] % auEndOffsets[i] 
+				% (auEndOffsets[i] - auStartOffsets[i]));
+		if (auEndOffsets[i] > auStartOffsets[i]) {
+			Requested_Block_Struct* reqblock = new Requested_Block_Struct;
+			reqblock->StartOffset = auStartOffsets[i];
+			reqblock->EndOffset = auEndOffsets[i];
+			md4cpy(reqblock->FileID, reqfilehash.GetHash());
+			reqblock->transferred = 0;
+			AddReqBlock(reqblock);
+		} else {
+			if (auEndOffsets[i] != 0 || auStartOffsets[i] != 0) {
+				AddDebugLogLineN(logClient, wxT("Client request is invalid!"));
+			}
+		}
+	}	
+}
+
+void CUpDownClient::ProcessRequestPartsPacketv2(const CMemFile& data) {
+	
+	CMD4Hash reqfilehash = data.ReadHash();
+	
+	uint8 numblocks = data.ReadUInt8();
+	
+	for (int i = 0; i < numblocks; i++) {		
+		Requested_Block_Struct* reqblock = new Requested_Block_Struct;
+		try {
+			reqblock->StartOffset = data.GetIntTagValue();
+			// We have to do +1, because the block matching uses that.
+			reqblock->EndOffset = data.GetIntTagValue() + 1;
+			if ((reqblock->StartOffset || reqblock->EndOffset) && (reqblock->StartOffset > reqblock->EndOffset)) {
+				AddDebugLogLineN(logClient, CFormat(wxT("Client %s request is invalid! %d / %d"))
+					% GetFullIP() % reqblock->StartOffset % reqblock->EndOffset);
+				throw wxString(wxT("Client request is invalid!"));
+			}
+			
+			AddDebugLogLineN(logClient,
+				CFormat(wxT("Client %s requests %d File block %d-%d (%d bytes):"))
+					% GetFullIP() % i % reqblock->StartOffset % reqblock->EndOffset
+					% (reqblock->EndOffset - reqblock->StartOffset));
+			
+			md4cpy(reqblock->FileID, reqfilehash.GetHash());
+			reqblock->transferred = 0;
+			AddReqBlock(reqblock);
+		} catch (...) {
+			delete reqblock;
+			throw;
+		}
+	}
+}
+// File_checked_for_headers
