@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -113,7 +113,7 @@ public:
 		wxSize dlg_size = wxSize(DEFAULT_SIZE_X,DEFAULT_SIZE_Y));
 	~CamuleDlg();
 
-	void AddLogLine(bool addtostatusbar, const wxString& line);
+	void AddLogLine(const wxString& line);
 	void AddServerMessageLine(wxString& message);
 	void ResetLog(int id);
 	
@@ -183,9 +183,10 @@ public:
 	
 	void DoNetworkRearrange();
 	
-#ifdef ENABLE_IP2COUNTRY	
 	CIP2Country*		m_IP2Country;
-#endif	
+	void IP2CountryDownloadFinished(uint32 result);
+	void EnableIP2Country();
+
 	wxWindow*		m_activewnd;
 	CTransferWnd*		m_transferwnd;
 	CServerWnd*		m_serverwnd;
@@ -206,9 +207,7 @@ protected:
 	void OnToolBarButton(wxCommandEvent& ev);
 	void OnAboutButton(wxCommandEvent& ev);
 	void OnPrefButton(wxCommandEvent& ev);
-#ifndef CLIENT_GUI	
 	void OnImportButton(wxCommandEvent& ev);
-#endif
 	void OnMinimize(wxIconizeEvent& evt);
 	void OnBnClickedFast(wxCommandEvent& evt);
 	void OnGUITimer(wxTimerEvent& evt);
@@ -228,6 +227,7 @@ private:
 	uint32 m_last_iconizing;
 	wxFileName m_skinFileName;
 	std::vector<wxString> m_clientSkinNames;
+	bool m_GeoIPavailable;
 
 	WX_DECLARE_STRING_HASH_MAP(wxZipEntry*, ZipCatalog);
 	ZipCatalog::iterator it;
