@@ -1,9 +1,9 @@
 //
 // This file is part of aMule Project
 //
-// Copyright (c) 2004-2008 Angel Vidal (Kry) ( kry@amule.org )
-// Copyright (c) 2004-2008 aMule Project ( http://www.amule-project.net )
-// Copyright (C)2003 Barry Dunne (http://www.emule-project.net)
+// Copyright (c) 2004-2011 Angel Vidal ( kry@amule.org )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2011 Barry Dunne ( http://www.emule-project.net )
 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@
 
 // This work is based on the java implementation of the Kademlia protocol.
 // Kademlia: Peer-to-peer routing based on the XOR metric
-// Copyright (C) 2002  Petar Maymounkov [petar@post.harvard.edu]
+// Copyright (c) 2002-2011  Petar Maymounkov ( petar@post.harvard.edu )
 // http://kademlia.scs.cs.nyu.edu
 
 // Note To Mods //
@@ -48,6 +48,9 @@ using namespace Kademlia;
 CRoutingBin::GlobalTrackingMap	CRoutingBin::s_globalContactIPs;
 CRoutingBin::GlobalTrackingMap	CRoutingBin::s_globalContactSubnets;
 
+#define MAX_CONTACTS_SUBNET	10
+#define MAX_CONTACTS_IP		1
+
 CRoutingBin::~CRoutingBin()
 {
 	for (ContactList::const_iterator it = m_entries.begin(); it != m_entries.end(); ++it) {
@@ -78,31 +81,13 @@ bool CRoutingBin::AddContact(CContact *contact)
 	// This is supposed to add a bit of protection against several attacks and raise the resource needs (IPs) for a successful contact on the attacker side
 	// Such IPs are not banned from Kad, they still can index, search, etc so multiple KAD clients behind one IP still work
 
-	// no more than 1 KadID per IP
-	uint32_t sameIPCount = 0;
-	GlobalTrackingMap::const_iterator itIP = s_globalContactIPs.find(contact->GetIPAddress());
-	if (itIP != s_globalContactIPs.end()) {
-		sameIPCount = itIP->second;
-	}
-	if (sameIPCount >= 1) {
-		AddDebugLogLineM(false, logKadRouting, wxT("Ignored kad contact (IP=") + Uint32_16toStringIP_Port(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress()), contact->GetUDPPort()) + wxT(") - too many contacts with the same IP (global)"));
+	if (!CheckGlobalIPLimits(contact->GetIPAddress(), contact->GetUDPPort())) {
 		return false;
 	}
 
 	// no more than 2 IPs from the same /24 netmask in one bin, except if its a LANIP (if we don't accept LANIPs they already have been filtered before)
 	if (sameSubnets >= 2 && !::IsLanIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress()))) {
-		AddDebugLogLineM(false, logKadRouting, wxT("Ignored kad contact (IP=") + Uint32_16toStringIP_Port(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress()), contact->GetUDPPort()) + wxT(") - too many contact with the same subnet in RoutingBin"));
-		return false;
-	}
-
-	//  no more than 10 IPs from the same /24 netmask global, except if its a LANIP (if we don't accept LANIPs they already have been filtered before)
-	uint32_t sameSubnetGlobalCount = 0;
-	GlobalTrackingMap::const_iterator itSubnet = s_globalContactSubnets.find(contact->GetIPAddress() & 0xFFFFFF00);
-	if (itSubnet != s_globalContactSubnets.end()) {
-		sameSubnetGlobalCount = itSubnet->second;
-	}
-	if (sameSubnetGlobalCount >= 10 && !::IsLanIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress()))) {
-		AddDebugLogLineM(false, logKadRouting, wxT("Ignored kad contact (IP=") + Uint32_16toStringIP_Port(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress()), contact->GetUDPPort()) + wxT(") - too many contacts with the same subnet (global)"));
+		AddDebugLogLineN(logKadRouting, wxT("Ignored kad contact (IP=") + KadIPPortToString(contact->GetIPAddress(), contact->GetUDPPort()) + wxT(") - too many contact with the same subnet in RoutingBin"));
 		return false;
 	}
 
@@ -167,7 +152,7 @@ CContact *CRoutingBin::GetContact(uint32_t ip, uint16_t port, bool tcpPort) cons
 	return NULL;
 }
 
-void CRoutingBin::GetNumContacts(uint32_t& nInOutContacts, uint32_t& nInOutFilteredContacts, uint8_t minVersion) const
+void CRoutingBin::GetNumContacts(uint32_t& nInOutContacts, uint32_t& nInOutFilteredContacts, uint8_t minVersion) const throw()
 {
 	// count all nodes which meet the search criteria and also report those who don't
 	for (ContactList::const_iterator it = m_entries.begin(); it != m_entries.end(); ++it) {
@@ -237,14 +222,14 @@ void CRoutingBin::AdjustGlobalTracking(uint32_t ip, bool increase)
 		sameIPCount = itIP->second;
 	}
 	if (increase) {
-		if (sameIPCount >= 1) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Global IP Tracking inconsistency on increase (") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(ip)) + wxT(")"));
+		if (sameIPCount >= MAX_CONTACTS_IP) {
+			AddDebugLogLineN(logKadRouting, wxT("Global IP Tracking inconsistency on increase (") + KadIPToString(ip) + wxT(")"));
 			wxFAIL;
 		}
 		sameIPCount++;
 	} else /* if (!increase) */ {
 		if (sameIPCount == 0) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Global IP Tracking inconsistency on decrease (") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(ip)) + wxT(")"));
+			AddDebugLogLineN(logKadRouting, wxT("Global IP Tracking inconsistency on decrease (") + KadIPToString(ip) + wxT(")"));
 			wxFAIL;
 		}
 		sameIPCount--;
@@ -262,14 +247,14 @@ void CRoutingBin::AdjustGlobalTracking(uint32_t ip, bool increase)
 		sameSubnetCount = itSubnet->second;
 	}
 	if (increase) {
-		if (sameSubnetCount >= 10 && !::IsLanIP(wxUINT32_SWAP_ALWAYS(ip))) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Global Subnet Tracking inconsistency on increase (") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(ip)) + wxT("/24)"));
+		if (sameSubnetCount >= MAX_CONTACTS_SUBNET && !::IsLanIP(wxUINT32_SWAP_ALWAYS(ip))) {
+			AddDebugLogLineN(logKadRouting, wxT("Global Subnet Tracking inconsistency on increase (") + KadIPToString(ip) + wxT("/24)"));
 			wxFAIL;
 		}
 		sameSubnetCount++;
 	} else /* if (!increase) */ {
 		if (sameSubnetCount == 0) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Global Subnet Tracking inconsistency on decrease (") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(ip)) + wxT("/24)"));
+			AddDebugLogLineN(logKadRouting, wxT("Global Subnet Tracking inconsistency on decrease (") + KadIPToString(ip) + wxT("/24)"));
 			wxFAIL;
 		}
 		sameSubnetCount--;
@@ -299,8 +284,8 @@ bool CRoutingBin::ChangeContactIPAddress(CContact *contact, uint32_t newIP)
 	if (itIP != s_globalContactIPs.end()) {
 		sameIPCount = itIP->second;
 	}
-	if (sameIPCount >= 1) {
-		AddDebugLogLineM(false, logKadRouting, wxT("Rejected kad contact update on IP change (old IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress())) + wxT(", requested IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(newIP)) + wxT(") - too many contacts with the same IP (global)"));
+	if (sameIPCount >= MAX_CONTACTS_IP) {
+		AddDebugLogLineN(logKadRouting, wxT("Rejected kad contact IP change on update (old IP=") + KadIPToString(contact->GetIPAddress()) + wxT(", requested IP=") + KadIPToString(newIP) + wxT(") - too many contacts with the same IP (global)"));
 		return false;
 	}
 
@@ -311,8 +296,8 @@ bool CRoutingBin::ChangeContactIPAddress(CContact *contact, uint32_t newIP)
 		if (itGlobalSubnet != s_globalContactSubnets.end()) {
 			sameSubnetGlobalCount = itGlobalSubnet->second;
 		}
-		if (sameSubnetGlobalCount >= 10 && !::IsLanIP(wxUINT32_SWAP_ALWAYS(newIP))) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Rejected kad contact update on IP change (old IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress())) + wxT(", requested IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(newIP)) + wxT(") - too many contacts with the same Subnet (global)"));
+		if (sameSubnetGlobalCount >= MAX_CONTACTS_SUBNET && !::IsLanIP(wxUINT32_SWAP_ALWAYS(newIP))) {
+			AddDebugLogLineN(logKadRouting, wxT("Rejected kad contact IP change on update (old IP=") + KadIPToString(contact->GetIPAddress()) + wxT(", requested IP=") + KadIPToString(newIP) + wxT(") - too many contacts with the same Subnet (global)"));
 			return false;
 		}
 
@@ -325,13 +310,13 @@ bool CRoutingBin::ChangeContactIPAddress(CContact *contact, uint32_t newIP)
 			}
 		}
 		if (sameSubnets >= 2 && !::IsLanIP(wxUINT32_SWAP_ALWAYS(newIP))) {
-			AddDebugLogLineM(false, logKadRouting, wxT("Rejected kad contact update on IP change (old IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress())) + wxT(", requested IP=") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(newIP)) + wxT(") - too many contacts with the same Subnet (local)"));
+			AddDebugLogLineN(logKadRouting, wxT("Rejected kad contact IP change on update (old IP=") + KadIPToString(contact->GetIPAddress()) + wxT(", requested IP=") + KadIPToString(newIP) + wxT(") - too many contacts with the same Subnet (local)"));
 			return false;
 		}
 	}
 
 	// everything fine
-	AddDebugLogLineM(false, logKadRouting, wxT("Index contact IP change allowed ") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(contact->GetIPAddress())) + wxT(" -> ") + Uint32toStringIP(wxUINT32_SWAP_ALWAYS(newIP)));
+	AddDebugLogLineN(logKadRouting, wxT("Index contact IP change allowed ") + KadIPToString(contact->GetIPAddress()) + wxT(" -> ") + KadIPToString(newIP));
 	AdjustGlobalTracking(contact->GetIPAddress(), false);
 	contact->SetIPAddress(newIP);
 	AdjustGlobalTracking(contact->GetIPAddress(), true);
@@ -346,7 +331,7 @@ void CRoutingBin::PushToBottom(CContact *contact)
 	m_entries.push_back(contact);
 }
 
-CContact *CRoutingBin::GetRandomContact(uint32_t maxType, uint32_t minKadVersion) const throw()
+CContact *CRoutingBin::GetRandomContact(uint32_t maxType, uint32_t minKadVersion) const
 {
 	if (m_entries.empty()) {
 		return NULL;
@@ -377,3 +362,38 @@ void CRoutingBin::SetAllContactsVerified()
 		(*it)->SetIPVerified(true);
 	}
 }
+
+bool CRoutingBin::CheckGlobalIPLimits(uint32_t ip, uint16_t DEBUG_ONLY(port))
+{
+	// no more than 1 KadID per IP
+	uint32_t sameIPCount = 0;
+	GlobalTrackingMap::const_iterator itIP = s_globalContactIPs.find(ip);
+	if (itIP != s_globalContactIPs.end()) {
+		sameIPCount = itIP->second;
+	}
+	if (sameIPCount >= MAX_CONTACTS_IP) {
+		AddDebugLogLineN(logKadRouting, wxT("Ignored kad contact (IP=") + KadIPPortToString(ip, port) + wxT(") - too many contacts with the same IP (global)"));
+		return false;
+	}
+	//  no more than 10 IPs from the same /24 netmask global, except if its a LANIP (if we don't accept LANIPs they already have been filtered before)
+	uint32_t sameSubnetGlobalCount = 0;
+	GlobalTrackingMap::const_iterator itSubnet = s_globalContactSubnets.find(ip & 0xFFFFFF00);
+	if (itSubnet != s_globalContactSubnets.end()) {
+		sameSubnetGlobalCount = itSubnet->second;
+	}
+	if (sameSubnetGlobalCount >= MAX_CONTACTS_SUBNET && !::IsLanIP(wxUINT32_SWAP_ALWAYS(ip))) {
+		AddDebugLogLineN(logKadRouting, wxT("Ignored kad contact (IP=") + KadIPPortToString(ip, port) + wxT(") - too many contacts with the same subnet (global)"));
+		return false;
+	}
+	return true;
+}
+
+bool CRoutingBin::HasOnlyLANNodes() const throw()
+{
+	for (ContactList::const_iterator it = m_entries.begin(); it != m_entries.end(); ++it) {
+		if (!::IsLanIP(wxUINT32_SWAP_ALWAYS((*it)->GetIPAddress()))) {
+			return false;
+		}
+	}
+	return true;
+}
