@@ -5,7 +5,7 @@
  *
  *  Author:       Pedro de Oliveira <falso@rdk.homeip.net>
  *
- *  Copyright (C) 2004 by Pedro de Oliveira
+ *  Copyright (c) 2004-2011 Pedro de Oliveira ( falso@rdk.homeip-net )
  * 
  *  This file is part of aMule.
  *
@@ -46,7 +46,7 @@
 
 #ifndef HAVE_GETOPT_LONG
 /* Code from getopt_long.h - getopt_long() for systems that lack it
-  Copyright (C) 2001, 2002, 2008 Arthur de Jong, GPL 2 and later */
+  Copyright (c) 2001-2011 Arthur de Jong, GPL 2 and later */
 # define no_argument 0
 # define required_argument 1
 # define optional_argument 2
@@ -106,7 +106,7 @@ void usage(char *myname)
 #ifndef HAVE_GETOPT_LONG
 
 /* Code from getopt_long.c - getopt_long() for systems that lack it
-  Copyright (C) 2001, 2002, 2008 Arthur de Jong, GPL 2 and later
+  Copyright (c) 2001-2011 Arthur de Jong, GPL 2 and later
   Slightly edited for the sake of clarity by Gaznevada */
 
 int getopt_long(int argc,
@@ -163,7 +163,7 @@ int main(int argc, char *argv[])
 	char *config_path=NULL;
 	char *path;
 	char *stats[20];
-	char *lines[7];
+	char *lines[IMG_TEXTLINES];
 	long lSize;
 	char * buffer;
 	int i;
@@ -247,32 +247,27 @@ int main(int argc, char *argv[])
 	// obtain file size.
 	fseek (amulesig , 0 , SEEK_END);
 	lSize = ftell (amulesig);
+	if (0 == lSize) {
+		perror("aMule signature file is 0 Byte, exiting.\n");
+		exit(2);
+	}
 	rewind (amulesig);
 	buffer = (char*) malloc (lSize);
 	if (buffer == NULL) {
 		perror("Could not create buffer\n");
 		exit (2);
 	}
-	fread (buffer,1,lSize,amulesig);
+	if (fread(buffer,1,lSize,amulesig)){}	// // prevent GCC warning
 	fclose(amulesig);
 
-	stats[0] = strtok (buffer,"\n"); /* ed2k status*/
-	stats[1] = strtok (NULL,"\n"); /* server name */
-	stats[2] = strtok (NULL,"\n"); /* server ip */
-	stats[3] = strtok (NULL,"\n"); /* server port */
-	stats[4] = strtok (NULL,"\n"); /* high or low id */
-	stats[5] = strtok (NULL,"\n"); /* kad status */
-	stats[6] = strtok (NULL,"\n"); /* dl */
-	stats[7] = strtok (NULL,"\n"); /* ul*/
-	stats[8] = strtok (NULL,"\n"); /* queue*/
-	stats[9] = strtok (NULL,"\n"); /* clients*/
-	stats[10] = strtok (NULL,"\n"); /* nick*/
-	stats[11] = strtok (NULL,"\n"); /* total download */
-	stats[12] = strtok (NULL,"\n"); /* total upload */
-	stats[13] = strtok (NULL,"\n"); /* version */
-	stats[14] = strtok (NULL,"\n"); /*Session download*/
-	stats[15] = strtok (NULL,"\n"); /* Session upload */
-	stats[16] = strtok (NULL,"\n"); /* aMule running Time */
+	stats[0] = strtok (buffer,"\n");
+	for (i=1;i<17;i++) {
+		stats[i] = strtok (NULL,"\n");
+		if (NULL == stats[i]) {
+			perror("Too few fields in aMule signature file, exiting.\n");
+			exit(2);
+		}
+	}
 
 	// local time stored as stats[17]
 	lt = time(NULL);
