@@ -5,7 +5,7 @@
  *
  *  Author:       Pedro de Oliveira <falso@rdk.homeip.net>
  *
- *  Copyright (C) 2004 by Pedro de Oliveira
+ *  Copyright (c) 2004-2011 Pedro de Oliveira ( falso@rdk.homeip-net )
  * 
  *  This file is part of aMule.
  *
@@ -28,7 +28,8 @@
 #ifndef CAS_HTML_H
 #define CAS_HTML_H
 
-int create_html(char *stats[20], char *lines[6], char template[120]);
+int create_html(char *stats[20], char *lines[6], char template[120], char *path_for_html);
 
 #endif
 
+/* // File_checked_for_headers */
