@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 
-// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (C) 2005-2008 Froenchenko Leonid ( lfroen@amule.org )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2005-2011 Froenchenko Leonid ( lfroen@gmail.com / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -24,23 +24,30 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#      include "config.h"
+#	include "config.h"
 #endif
 
 #include <string> // Do_not_auto_remove (g++-4.0.1)
 
 #ifdef HAVE_SYS_TYPES_H
-#      include <sys/types.h>
+#	include <sys/types.h>
 #endif
-#include <regex.h>
 
-#include "WebServer.h"
-#include <ec/cpp/ECSpecialTags.h>
+#ifdef PHP_STANDALONE_EN
+#	include <map>
+#	include <string>
+#	include <list>
+#	include <regex.h>
+#else
+#	include "WebServer.h"
+#	include <ec/cpp/ECSpecialTags.h>
+#	include <wx/regex.h>
+#	include <wx/datetime.h>
+#endif
 
 #include "php_syntree.h"
 #include "php_core_lib.h"
-
-#include <wx/datetime.h>
+#include <stdarg.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -66,7 +73,7 @@ void php_var_dump(PHP_VALUE_NODE *node, int ident, int ref)
 		case PHP_VAL_BOOL: printf("bool(%s)\n", node->int_val ? "true" : "false"); break;
 		case PHP_VAL_INT: printf("int(%"PRIu64")\n", node->int_val); break;
 		case PHP_VAL_FLOAT: printf("float(%f)\n", node->float_val); break;
-		case PHP_VAL_STRING: printf("string(%d) \"%s\"\n", strlen(node->str_val), node->str_val); break;
+		case PHP_VAL_STRING: printf("string(%d) \"%s\"\n", (int)strlen(node->str_val), node->str_val); break;
 		case PHP_VAL_OBJECT: printf("Object(%s)\n", node->obj_val.class_name); break;
 		case PHP_VAL_ARRAY: {
 			int arr_size = array_get_size(node);
@@ -140,7 +147,7 @@ bool operator<(const SortElem &o1, const SortElem &o2)
 	value_value_free(&SortElem::callback->params[0].si_var->var->value);
 	value_value_free(&SortElem::callback->params[1].si_var->var->value);
 	
-	return result.int_val;
+	return result.int_val != 0;
 }
 
 void php_native_usort(PHP_VALUE_NODE *)
@@ -306,56 +313,84 @@ void php_native_split(PHP_VALUE_NODE *result)
 		php_report_error(PHP_ERROR, "Invalid or missing argument: string");
 		return;		
 	}
+#ifdef PHP_STANDALONE_EN
 	regex_t preg;
 	char error_buff[256];
 	int reg_result = regcomp(&preg, pattern->str_val, REG_EXTENDED);
 	if ( reg_result ) {
 		regerror(reg_result, &preg, error_buff, sizeof(error_buff));
 		php_report_error(PHP_ERROR, "Failed in regcomp: %s", error_buff);
+#else
+	wxRegEx preg;
+	if (!preg.Compile(wxString(char2unicode(pattern->str_val)), wxRE_EXTENDED)) {
+		php_report_error(PHP_ERROR, "Failed in Compile of: %s", pattern->str_val);
+#endif
 		return;
 	}
+
+#ifdef PHP_STANDALONE_EN
 	size_t nmatch = strlen(string_to_split->str_val);
 	regmatch_t *pmatch = new regmatch_t[nmatch];
-	
+#endif
 	char *str_2_match = string_to_split->str_val;
 	char *tmp_buff = new char[strlen(string_to_split->str_val)+1];
 	
 	while ( 1 ) {
 //		printf("matching: %s\n", str_2_match);
+#ifdef PHP_STANDALONE_EN
 		reg_result = regexec(&preg, str_2_match, nmatch, pmatch, 0);
 		if ( reg_result ) {
+#else
+		if (!preg.Matches(wxString(char2unicode(str_2_match)))) {
+#endif
 			// no match
 			break;
 		}
-//		for(int i = 0; pmatch[i].rm_so >= 0; i++) {
-//			printf("match [%d] %d - %d\n", i, pmatch[i].rm_so, pmatch[i].rm_eo);
-//		}
-	
+#ifndef PHP_STANDALONE_EN
+		// get matching position
+		size_t start, len;
+		if (!preg.GetMatch(&start, &len)) {
+			break;	// shouldn't happen
+		}
+#endif	
 		/*
 		 * I will use only first match, since I don't see any sense to have more
 		 * then 1 match in split() call
 		 */
+#ifdef PHP_STANDALONE_EN
 		for(int i = 0; i < pmatch[0].rm_so; i++) {
+#else
+		for(size_t i = 0; i < start; i++) {
+#endif
 			tmp_buff[i] = str_2_match[i];
 		}
+#ifdef PHP_STANDALONE_EN
 		tmp_buff[pmatch[0].rm_so] = 0;
+#else
+		tmp_buff[start] = 0;
+#endif
 //		printf("Match added [%s]\n", tmp_buff);
 		
 		PHP_VAR_NODE *match_val = array_push_back(result);
 		match_val->value.type = PHP_VAL_STRING;
 		match_val->value.str_val = strdup(tmp_buff);
 
+#ifdef PHP_STANDALONE_EN
 		str_2_match += pmatch[0].rm_eo;
+#else
+		str_2_match += start + len;
+#endif
 	}
 
 	PHP_VAR_NODE *match_val = array_push_back(result);
 	match_val->value.type = PHP_VAL_STRING;
 	match_val->value.str_val = strdup(str_2_match);
 	
-	delete [] pmatch;
 	delete [] tmp_buff;
-	
+#ifdef PHP_STANDALONE_EN
+	delete [] pmatch;
 	regfree(&preg);
+#endif
 }
 
 #ifdef ENABLE_NLS
@@ -500,12 +535,12 @@ CPhPLibContext::CPhPLibContext(CWebServerBase *server, const char *file)
 	m_server = server;
 
 	php_engine_init();
-	yyin = fopen(file, "r");
-	if ( !yyin ) {
+	phpin = fopen(file, "r");
+	if ( !phpin ) {
 		return;
 	}
 
-	yyparse();
+	phpparse();
 	
 	m_syn_tree_top = g_syn_tree_top;
 	m_global_scope = g_global_scope;
@@ -522,7 +557,7 @@ CPhPLibContext::CPhPLibContext(CWebServerBase *server, char *php_buf, int len)
 	m_global_scope = g_global_scope;
 
 	php_set_input_buffer(php_buf, len);
-	yyparse();
+	phpparse();
 	
 	m_syn_tree_top = g_syn_tree_top;
 }
@@ -580,20 +615,24 @@ void CPhPLibContext::Print(const char *str)
 	}
 }
 
+
 CPhpFilter::CPhpFilter(CWebServerBase *server, CSession *sess,
 			const char *file, CWriteStrBuffer *buff)
 {
 	FILE *f = fopen(file, "r");
 	if ( !f ) {
+		printf("ERROR: php can not open source file [%s]\n", file);
 		return;
 	}
 	if ( fseek(f, 0, SEEK_END) != 0 ) {
+		printf("ERROR: fseek failed on php source file [%s]\n", file); 
 		return;
 	}
 	int size = ftell(f);
 	char *buf = new char [size+1];
 	rewind(f);
-	fread(buf, 1, size, f);
+	// fread may actually read less if it is a CR-LF-file in Windows
+	size = fread(buf, 1, size, f);
 	buf[size] = 0;
 	fclose(f);
 	char *scan_ptr = buf;
@@ -617,24 +656,30 @@ CPhpFilter::CPhpFilter(CWebServerBase *server, CSession *sess,
 
 		CPhPLibContext *context = new CPhPLibContext(server, scan_ptr, len);
 
-
+#ifndef PHP_STANDALONE_EN
 		load_session_vars("HTTP_GET_VARS", sess->m_get_vars);
 		load_session_vars("_SESSION", sess->m_vars);
+#endif
 
 		context->Execute(buff);
 
+#ifndef PHP_STANDALONE_EN
 		save_session_vars(sess->m_vars);
+#endif
 
 		delete context;
 		
 		scan_ptr = curr_code_end;
 	}
 
+#ifndef PHP_STANDALONE_EN
 	sess->m_get_vars.clear();
+#endif
 
 	delete [] buf;
 }
 
+
 /*
  * String buffer: almost same as regular 'string' class, but,
  * without reallocation when full. Instead, new buffer is
