@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (C) 2005-2006Froenchenko Leonid ( lfroen@amule.org )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2005-2011 Froenchenko Leonid ( lfroen@gmail.com / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -23,25 +23,22 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#include <cstdio>
-#include <cstring>
-#include <cassert>
-#include <cstdarg>
-#include <inttypes.h>
 
-#include <list>
-#include <map>
-#include <vector>
-#include <string>
-#include <algorithm>
+#include <string> // Do_not_auto_remove (g++-4.0.1)
 
-#ifndef PHP_STANDALONE_EN
+#ifdef PHP_STANDALONE_EN
+	#include <map>
+	#include <list>
+	#include <stdarg.h>
+#else
 	#include "WebServer.h"
 #endif
 
+
 #include "php_syntree.h"
 #include "php_core_lib.h"
 
+
 PHP_SYN_NODE *g_syn_tree_top = 0;
 
 /* scope table */
@@ -271,7 +268,7 @@ PHP_SYN_NODE *make_foreach_loop_syn_node(PHP_EXP_NODE *elems,
 	return syn_node;
 }
 
-PHP_SYN_NODE *make_func_decl_syn_node(char *name, PHP_EXP_NODE *param_list)
+PHP_SYN_NODE *make_func_decl_syn_node(const char *name, PHP_EXP_NODE *param_list)
 {
 	PHP_SYN_NODE *syn_node = new PHP_SYN_NODE;
 	memset(syn_node, 0, sizeof(PHP_SYN_NODE));
@@ -368,7 +365,7 @@ PHP_VAR_NODE *make_array_var()
 /*
  * Called from lexer when ${IDENT} is recognized
  */
-PHP_EXP_NODE *get_var_node(char *name)
+PHP_EXP_NODE *get_var_node(const char *name)
 {
 	PHP_EXP_NODE *node = make_zero_exp_node();
 	node->op = PHP_OP_VAR;
@@ -403,7 +400,7 @@ void free_var_node(PHP_VAR_NODE *v)
  *  2. Lvalue-evaluate all by-ref params and adjust pointers
  */
 void func_scope_init(PHP_FUNC_PARAM_DEF *params, int param_count,
-	PHP_SCOPE_TABLE_TYPE *scope_map, PHP_VALUE_NODE *arg_array,
+	PHP_SCOPE_TABLE_TYPE * /*scope_map*/, PHP_VALUE_NODE *arg_array,
 	std::map<std::string, PHP_VAR_NODE *> &saved_vars)
 {
 	//
@@ -472,7 +469,7 @@ void func_scope_init(PHP_FUNC_PARAM_DEF *params, int param_count,
  *  2. Next call may be using same params by-value, so it need independent varnode
  */
 void func_scope_copy_back(PHP_FUNC_PARAM_DEF *params, int param_count,
-	PHP_SCOPE_TABLE_TYPE *scope_map, PHP_VALUE_NODE *arg_array,
+	PHP_SCOPE_TABLE_TYPE * /*scope_map*/, PHP_VALUE_NODE *arg_array,
 	std::map<std::string, PHP_VAR_NODE *> &saved_vars)
 {
 	/*
@@ -884,7 +881,7 @@ void cast_value_dnum(PHP_VALUE_NODE *val)
 		case PHP_VAL_FLOAT: val->int_val = (int)val->float_val; break;
 		case PHP_VAL_STRING: {
 			char *str = val->str_val;
-			val->int_val = atoi(val->str_val);
+			val->int_val = atoll(val->str_val);
 			free(str);
 			break;
 		}
@@ -929,7 +926,7 @@ void cast_value_str(PHP_VALUE_NODE *val)
 	switch(val->type) {
 		case PHP_VAL_NONE: buff[0] = 0; break;
 		case PHP_VAL_BOOL:
-		case PHP_VAL_INT: snprintf(buff, sizeof(buff), "%lld", val->int_val); break;
+		case PHP_VAL_INT: snprintf(buff, sizeof(buff), "%"PRIu64, val->int_val); break;
 		case PHP_VAL_FLOAT: snprintf(buff, sizeof(buff), "%.02f", val->float_val); break;
 		case PHP_VAL_STRING: return;
 		case PHP_VAL_ARRAY: {
@@ -1039,7 +1036,7 @@ void php_add_native_func(PHP_BLTIN_FUNC_DEF *def)
 	add_func_2_scope(g_global_scope, decl_node);
 }
 
-void php_add_native_class(char *name, PHP_NATIVE_PROP_GET_FUNC_PTR prop_get_native_ptr)
+void php_add_native_class(const char *name, PHP_NATIVE_PROP_GET_FUNC_PTR prop_get_native_ptr)
 {
 	if ( get_scope_item_type(g_global_scope, name) != PHP_SCOPE_NONE ) {
 		//
@@ -1065,6 +1062,7 @@ void php_engine_init()
 	
 	// here built-in functions/objects/vars are loaded
 	php_init_core_lib();
+	php_init_amule_lib();
 }
 
 void php_exp_tree_free(PHP_EXP_NODE *tree)
@@ -1833,14 +1831,16 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 			case PHP_ST_CONTINUE:
 			case PHP_ST_BREAK:
 				if (  node->node_expr ) {
+					// 'break' or 'continue' used with an argument
 					php_expr_eval(node->node_expr, &cond_result);
 				} else {
+					// without an argument break or continue just 1 loop
 					cond_result.type = PHP_VAL_INT;
 					cond_result.int_val = 1;
 				}
 				cast_value_dnum(&cond_result);
 				if ( node->type == PHP_ST_BREAK ) {
-					curr_exec_result = -cond_result.int_val;
+					curr_exec_result = -(int)(cond_result.int_val);
 				} else {
 					curr_exec_result = cond_result.int_val;
 				}
@@ -1855,12 +1855,29 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 					cond_result.int_val = 1;
 				}
 				while ( cond_result.int_val ) {
+					// evaluate code within while loop
 					curr_exec_result = php_execute(node->node_while.code, 0);
+					// if 'break' was used
 					if ( curr_exec_result < 0 ) {
+						// decrease number of loops to break
+						curr_exec_result++;
+						// and break current loop
+						break;
+					}
+					// normal execution of loop or continue
+					if ( curr_exec_result == 0 || curr_exec_result == 1 ) {
+						curr_exec_result = 0;
+						// evaluate 'while' loop conditions
+						php_expr_eval(node->node_while.cond, &cond_result);
+						cast_value_bool(&cond_result);
+					}
+					// if 'continue' was used with an argument > 1
+					if ( curr_exec_result > 1 ) {
+						// decrease number of loops to skip
+						curr_exec_result--;
+						// and break current loop
 						break;
 					}
-					php_expr_eval(node->node_while.cond, &cond_result);
-					cast_value_bool(&cond_result);
 				}
 				break;
 			case PHP_ST_FOR:
@@ -1869,12 +1886,22 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 				cast_value_bool(&cond_result);
 				while ( cond_result.int_val ) {
 					curr_exec_result = php_execute(node->node_for.code, 0);
+					// handle 'break' and 'continue' 
 					if ( curr_exec_result < 0 ) {
+						curr_exec_result++;
+						break;
+					}
+					if ( curr_exec_result == 0 || curr_exec_result == 1 ) {
+						curr_exec_result = 0;
+						// evaluate 'for' loop conditions
+						php_expr_eval(node->node_for.do_next, &cond_result);
+						php_expr_eval(node->node_for.cond, &cond_result);
+						cast_value_bool(&cond_result);
+					}
+					if ( curr_exec_result > 1 ) {
+						curr_exec_result--;
 						break;
 					}
-					php_expr_eval(node->node_for.do_next, &cond_result);
-					php_expr_eval(node->node_for.cond, &cond_result);
-					cast_value_bool(&cond_result);
 				}
 				break;
 			case PHP_ST_FOREACH: {
@@ -1901,6 +1928,7 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 						PHP_VALUE_NODE *curr_value = &array->array[*array->current]->value;
 						value_value_assign(&i_val->var->value, curr_value);
 						curr_exec_result = php_execute(node->node_foreach.code, 0);
+						// clean up
 						if ( i_key ) {
 							value_value_free(&i_key->var->value);
 						}
@@ -1908,31 +1936,50 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 							value_value_assign(curr_value, &i_val->var->value);
 						}
 						value_value_free(&i_val->var->value);
+						// handle 'break' and 'continue' 
 						if ( curr_exec_result < 0 ) {
+							curr_exec_result++;
+							break;
+						}
+						if ( curr_exec_result == 0 || curr_exec_result == 1 ) {
+							curr_exec_result = 0;
+							// next element
+							array->current++;
+						}
+						if ( curr_exec_result > 1 ) {
+							curr_exec_result--;
 							break;
 						}
-						array->current++;
 					}
 				}
 				break;
 			case PHP_ST_SWITCH: {
 					PHP_SYN_NODE *cur_exec = 0;
+					// evaluate switch argument
 					php_expr_eval(node->node_switch.cond, &cond_result);
+					// loop through list of case statements
 					PHP_EXP_NODE *curr = node->node_switch.case_list;
 					while (curr) {
 						PHP_VALUE_NODE cur_value, cmp_result;
 						cur_value.type = cmp_result.type = PHP_VAL_NONE;
+						// TODO make amuleweb not crash when 'default' is used
 						php_expr_eval(curr->exp_node->tree_node.left, &cur_value);
 
+						// switch argument equal to case argument?
 						php_eval_compare(PHP_OP_EQ, &cur_value, &cond_result, &cmp_result);
 						if ( cmp_result.int_val ) {
+							// execute code and rest of case statements
 							cur_exec = (PHP_SYN_NODE *)curr->exp_node->tree_node.syn_right;
 							break;
 						}
 						curr = curr->next;
 					}
 					if ( cur_exec ) {
-						php_execute(cur_exec, result);
+						curr_exec_result = php_execute(cur_exec, result);
+						// break
+						if (curr_exec_result < 0) curr_exec_result++;
+						// continue
+						if (curr_exec_result > 0) curr_exec_result--;
 					}
 				}
 				break;
@@ -1951,7 +1998,7 @@ int php_execute(PHP_SYN_NODE *node, PHP_VALUE_NODE *result)
 //
 // call it when something gone wrong
 //
-void php_report_error(PHP_MSG_TYPE err_type, char *msg, ...)
+void php_report_error(PHP_MSG_TYPE err_type, const char *msg, ...)
 {
 	//
 	// hope my error message will never be that big.
@@ -1962,7 +2009,7 @@ void php_report_error(PHP_MSG_TYPE err_type, char *msg, ...)
 	// leak here and create stack overrun exploit. Be warned.
 	//
 	char msgbuf[1024];
-	char *type_msg = 0;
+	const char *type_msg = 0;
 	switch(err_type) {
 		case PHP_MESAGE:
 			type_msg = "PHP:";
@@ -1988,31 +2035,10 @@ void php_report_error(PHP_MSG_TYPE err_type, char *msg, ...)
 }
 
 
-int yyerror(char *s)
+int phperror(char *s)
 {
-	printf("ERROR in grammar %s after [%s] near line %d\n", s, yytext, yylineno);
+	printf("ERROR in grammar %s after [%s] near line %d\n", s, phptext, phplineno);
 	return 0;
 }
 
-#ifdef PHP_STANDALONE_EN
-
-int main(int argc, char *argv[])
-{
-	const char *filename = ( argc == 2 ) ? argv[1] : "test.php";
-
-	CWriteStrBuffer buffer;
-
-	CPhpFilter php_filter((CWebServerBase*)0, (CSession *)0,filename, &buffer);
-	
-	yydebug = 1;
-	
-	int size = buffer.Length();
-	char *buf = new char [size+1];
-	buffer.CopyAll(buf);
-	printf("%s", buf);
-	delete [] buf;
-	
-	return 0;
-}
-
-#endif
+// File_checked_for_headers
